module microp_aero

!---------------------------------------------------------------------------------
! Purpose:
!   CAM Interface for aerosol activation 
!
! ***N.B.*** This module is currently hardcoded to recognize only the aerosols/modes that
!            affect the climate calculation.  This is implemented by using list
!            index 0 in all the calls to rad_constituent interfaces.
!
! Author: Andrew Gettelman
! Based on code from: Hugh Morrison, Xiaohong Liu and Steve Ghan
! May 2010
! Description in: Morrison and Gettelman, 2008. J. Climate (MG2008)
!                 Gettelman et al., 2010 J. Geophys. Res. - Atmospheres (G2010)         
! for questions contact Andrew Gettelman  (andrew@ucar.edu)
! Modifications: A. Gettelman Nov 2010  - changed to support separation of 
!                microphysics and macrophysics and concentrate aerosol information here
!
!---------------------------------------------------------------------------------

use shr_kind_mod,     only: r8=>shr_kind_r8
use spmd_utils,       only: masterproc
use ppgrid,           only: pcols, pver, pverp
use physconst,        only: rair, tmelt,pi    ! sxj add pi
use constituents,     only: cnst_get_ind, pcnst
use physics_types,    only: physics_state, physics_ptend, physics_ptend_init
use physics_buffer,   only: physics_buffer_desc, pbuf_get_index, pbuf_old_tim_idx, pbuf_get_field
use phys_control,     only: phys_getopts
use rad_constituents, only: rad_cnst_get_info, rad_cnst_get_aer_mmr, rad_cnst_get_aer_props, &
                            rad_cnst_get_mode_num, rad_cnst_get_mode_props
use shr_spfn_mod,     only: erf => shr_spfn_erf_nonintrinsic, &
                            erfc => shr_spfn_erfc_nonintrinsic
use wv_saturation,    only: qsat_water
use nucleate_ice,     only: nucleati,frachom   ! sxj, add frachom
use ndrop,            only: ndrop_init, dropmixnuc
use ndrop_bam,        only: ndrop_bam_init, ndrop_bam_run, ndrop_bam_ccn
use cam_history,      only: addfld, phys_decomp, add_default, outfld
use cam_logfile,      only: iulog
use abortutils,       only: endrun
use ref_pres,         only: top_lev => trop_cloud_top_lev
	
use wv_saturation,    only: svp_water, svp_ice !added by Cheng
use meso_temp_cirrus, only: meso_temp_gary0608 !added by Cheng 
use um_aerosol_number,only: um_IN_numbers !added by Cheng
use um_SOA_IN_intr,   only: um_SOA_IN_num_cal ! added by Cheng
use iceBN_mod,        only: IceBN
use um_brunt_vaisala, only: um_brunt_vaisala_cal ! added by Cheng
use um_KL_ni,         only:	KL_ni
implicit none
private
save

public :: microp_aero_init, microp_aero_run, microp_aero_readnl, microp_aero_register

! Private module data

character(len=16)   :: eddy_scheme  ! eddy scheme

! contact freezing due to dust
! dust number mean radius (m), Zender et al JGR 2003 assuming number mode radius of 0.6 micron, sigma=2
real(r8), parameter :: rn_dst1 = 0.258e-6_r8
real(r8), parameter :: rn_dst2 = 0.717e-6_r8
real(r8), parameter :: rn_dst3 = 1.576e-6_r8
real(r8), parameter :: rn_dst4 = 3.026e-6_r8
!++sxj
logical,  parameter :: OR_preice = .false.     ! whether take preexisting ice into account for ice nucleation 
logical,  parameter :: OR_purehom = .false.   ! true, pure hom freezing in cirrus
logical,  parameter :: OR_fhom= .false.        ! true, only a fraction of cirrus process hom freezing 
real(r8), parameter :: WSUBIweight =  1.0_r8   !  WSUBI = WSUBI * WSUBIweight
integer,  parameter :: ice_para = 4    ! option for ice nucleation parameterizaiton  2=LP; 3=BN; 4=KL  
!integer  :: ww_para    ! option w, 1: wsubi<=0.2  2: no limit for wsubi 3:wGary 4: grid box mean large scale
integer,  parameter :: ww_para = 3    ! option w, 1: wsubi<=0.2  2: no limit for wsubi 3:wGary 4: grid box mean large scale
		
real(r8), public :: bulk_scale    ! prescribed aerosol bulk sulfur scale factor
	


! smallest mixing ratio considered in microphysics
real(r8), parameter :: qsmall = 1.e-18_r8

! minimum allowed cloud fraction
real(r8), parameter :: mincld = 0.0001_r8

! indices in state%q and pbuf structures
integer :: cldliq_idx = -1
integer :: cldice_idx = -1
integer :: numliq_idx = -1
integer :: numice_idx = -1
integer :: kvh_idx = -1
integer :: tke_idx = -1
integer :: wp2_idx = -1
integer :: ast_idx = -1
integer :: cldo_idx = -1
integer :: dgnum_idx    = -1
integer :: dgnumwet_idx = -1

! Bulk aerosols
character(len=20), allocatable :: aername(:)
real(r8), allocatable :: num_to_mass_aer(:)

integer :: naer_all      ! number of aerosols affecting climate
integer :: idxsul   = -1 ! index in aerosol list for sulfate
integer :: idxdst1  = -1 ! index in aerosol list for dust1
integer :: idxdst2  = -1 ! index in aerosol list for dust2
integer :: idxdst3  = -1 ! index in aerosol list for dust3
integer :: idxdst4  = -1 ! index in aerosol list for dust4
integer :: idxbcphi = -1 ! index in aerosol list for Soot (BCPHIL)

! modal aerosols
logical :: prog_modal_aero
logical :: clim_modal_aero

integer :: mode_accum_idx  = -1  ! index of accumulation mode
integer :: mode_aitken_idx = -1  ! index of aitken mode
integer :: mode_coarse_idx = -1  ! index of coarse mode
integer :: mode_coarse_dst_idx = -1  ! index of coarse dust mode
integer :: mode_coarse_slt_idx = -1  ! index of coarse sea salt mode
integer :: coarse_dust_idx = -1  ! index of dust in coarse mode
integer :: coarse_nacl_idx = -1  ! index of nacl in coarse mode

integer :: naai_idx, naai_hom_idx, npccn_idx, rndst_idx, nacon_idx

real(r8) :: sigmag_aitken
logical  :: separate_dust = .false.

!===============================================================================
contains
!===============================================================================
subroutine microp_aero_register
   !----------------------------------------------------------------------- 
   ! 
   ! Purpose: 
   ! Register pbuf fields for aerosols needed by microphysics
   ! 
   ! Author: Cheryl Craig October 2012
   ! 
   !-----------------------------------------------------------------------
   use ppgrid,         only: pcols
   use physics_buffer, only: pbuf_add_field, dtype_r8

   call pbuf_add_field('NAAI',       'physpkg',dtype_r8,(/pcols,pver/), naai_idx)
   call pbuf_add_field('NAAI_HOM',   'physpkg',dtype_r8,(/pcols,pver/), naai_hom_idx)
   call pbuf_add_field('NPCCN',      'physpkg',dtype_r8,(/pcols,pver/), npccn_idx)
   call pbuf_add_field('RNDST',      'physpkg',dtype_r8,(/pcols,pver,4/), rndst_idx)
   call pbuf_add_field('NACON',      'physpkg',dtype_r8,(/pcols,pver,4/), nacon_idx)
 

end subroutine microp_aero_register

subroutine microp_aero_init

   !----------------------------------------------------------------------- 
   ! 
   ! Purpose: 
   ! Initialize constants for aerosols needed by microphysics
   ! 
   ! Author: Andrew Gettelman May 2010
   ! 
   !-----------------------------------------------------------------------

   ! local variables
   integer  :: iaer
   integer  :: m, n, nmodes, nspec

   character(len=32) :: str32
   character(len=*), parameter :: routine = 'microp_aero_init'
   logical :: history_amwg
   !-----------------------------------------------------------------------

   ! Query the PBL eddy scheme
   call phys_getopts(eddy_scheme_out          = eddy_scheme, &
                     history_amwg_out = history_amwg)

   ! Access the physical properties of the aerosols that are affecting the climate
   ! by using routines from the rad_constituents module.

   ! get indices into state and pbuf structures
   call cnst_get_ind('CLDLIQ', cldliq_idx)
   call cnst_get_ind('CLDICE', cldice_idx)
   call cnst_get_ind('NUMLIQ', numliq_idx)
   call cnst_get_ind('NUMICE', numice_idx)

   select case(trim(eddy_scheme))
   case ('diag_TKE')
      tke_idx      = pbuf_get_index('tke')
   case ('CLUBB_SGS')
      wp2_idx = pbuf_get_index('WP2')
   case default
      kvh_idx      = pbuf_get_index('kvh')
   end select

   ! prog_modal_aero determines whether prognostic modal aerosols are present in the run.
   call phys_getopts(prog_modal_aero_out=prog_modal_aero)

   ! clim_modal_aero determines whether modal aerosols are used in the climate calculation.
   ! The modal aerosols can be either prognostic or prescribed.
   call rad_cnst_get_info(0, nmodes=nmodes)
   clim_modal_aero = (nmodes > 0)

   ast_idx      = pbuf_get_index('AST')

   if (clim_modal_aero) then

      cldo_idx     = pbuf_get_index('CLDO')
      dgnum_idx    = pbuf_get_index('DGNUM' )
      dgnumwet_idx = pbuf_get_index('DGNUMWET')

      call ndrop_init()

      ! Init indices for specific modes/species

      ! mode index for specified mode types
      do m = 1, nmodes
         call rad_cnst_get_info(0, m, mode_type=str32)
         select case (trim(str32))
         case ('accum')
            mode_accum_idx = m
         case ('aitken')
            mode_aitken_idx = m
         case ('coarse')
            mode_coarse_idx = m
         case ('coarse_dust')
            mode_coarse_dst_idx = m
         case ('coarse_seasalt')
            mode_coarse_slt_idx = m
         end select
      end do

      ! check if coarse dust is in separate mode
      separate_dust = mode_coarse_dst_idx > 0

      ! for 3-mode 
      if ( mode_coarse_dst_idx<0 ) mode_coarse_dst_idx = mode_coarse_idx
      if ( mode_coarse_slt_idx<0 ) mode_coarse_slt_idx = mode_coarse_idx

      ! Check that required mode types were found
      if (mode_accum_idx == -1 .or. mode_aitken_idx == -1 .or. &
          mode_coarse_dst_idx == -1.or. mode_coarse_slt_idx == -1) then
         write(iulog,*) routine//': ERROR required mode type not found - mode idx:', &
            mode_accum_idx, mode_aitken_idx, mode_coarse_dst_idx, mode_coarse_slt_idx
         call endrun(routine//': ERROR required mode type not found')
      end if

      ! species indices for specified types
      ! find indices for the dust and seasalt species in the coarse mode
      call rad_cnst_get_info(0, mode_coarse_dst_idx, nspec=nspec)
      do n = 1, nspec
         call rad_cnst_get_info(0, mode_coarse_dst_idx, n, spec_type=str32)
         select case (trim(str32))
         case ('dust')
            coarse_dust_idx = n
         end select
      end do
      call rad_cnst_get_info(0, mode_coarse_slt_idx, nspec=nspec)
      do n = 1, nspec
         call rad_cnst_get_info(0, mode_coarse_slt_idx, n, spec_type=str32)
         select case (trim(str32))
         case ('seasalt')
            coarse_nacl_idx = n
         end select
      end do

      ! Check that required mode specie types were found
      if ( coarse_dust_idx == -1 .or. coarse_nacl_idx == -1) then
         write(iulog,*) routine//': ERROR required mode-species type not found - indicies:', &
            coarse_dust_idx, coarse_nacl_idx
         call endrun(routine//': ERROR required mode-species type not found')
      end if

      ! get specific mode properties
      call rad_cnst_get_mode_props(0, mode_aitken_idx, sigmag=sigmag_aitken)

   else

      ! Props needed for BAM number concentration calcs.

      call rad_cnst_get_info(0, naero=naer_all)
      allocate( &
         aername(naer_all),        &
         num_to_mass_aer(naer_all) )

      do iaer = 1, naer_all
         call rad_cnst_get_aer_props(0, iaer, &
            aername         = aername(iaer), &
            num_to_mass_aer = num_to_mass_aer(iaer) )

         ! Look for sulfate, dust, and soot in this list (Bulk aerosol only)
         if (trim(aername(iaer)) == 'SULFATE') idxsul = iaer
         if (trim(aername(iaer)) == 'DUST1') idxdst1 = iaer
         if (trim(aername(iaer)) == 'DUST2') idxdst2 = iaer
         if (trim(aername(iaer)) == 'DUST3') idxdst3 = iaer
         if (trim(aername(iaer)) == 'DUST4') idxdst4 = iaer
         if (trim(aername(iaer)) == 'BCPHIL') idxbcphi = iaer
      end do

      call ndrop_bam_init()

   end if

   call addfld('LCLOUD', ' ', pver, 'A', 'Liquid cloud fraction used in stratus activation', phys_decomp)

   call addfld('WSUB     ', 'm/s     ', pver, 'A', 'Diagnostic sub-grid vertical velocity'                   ,phys_decomp)
   call addfld('WSUBI    ', 'm/s     ', pver, 'A', 'Diagnostic sub-grid vertical velocity for ice'           ,phys_decomp)
   call addfld('NIHF',  '1/m3', pver, 'A', 'Activated Ice Number Concentation due to homogenous freezing',  phys_decomp)
   call addfld('NIDEP', '1/m3', pver, 'A', 'Activated Ice Number Concentation due to deposition nucleation',phys_decomp)
   call addfld('NIIMM', '1/m3', pver, 'A', 'Activated Ice Number Concentation due to immersion freezing',   phys_decomp)
   call addfld('NIMEY', '1/m3', pver, 'A', 'Activated Ice Number Concentation due to meyers deposition',    phys_decomp)


!++++++ below are added by Cheng
        call addfld      ('um_nm', '1/s', pver, 'A', 'Brunt-Vaisala frequency', phys_decomp) 
        call add_default ('um_nm', 1, 'A')
        
        call addfld      ('WSUB0', 'm/s', pver, 'A', 'wsub from sqrt(0.6667*TKE), no limiters applied', phys_decomp) 
        call add_default ('WSUB0', 1, 'A')
        call add_default ('WSUBI', 1, 'A')
        
        call addfld      ('WGARY', 'm/s', pver, 'A', 'Subgrid updraft velocity derived from Gary 2006/2008, see Wang and Penner 2010', phys_decomp) 
        call add_default ('WGARY', 1, 'A')

        call addfld      ('WLG', 'm/s', pver, 'A', 'Subgrid updraft velocity derived from Gary 2006/2008, see Wang and Penner 2010', phys_decomp) 
        call add_default ('WLG', 1, 'A')
                
        call addfld('NAAI',  '1/m3', pver, 'A', 'Activated Ice Number Concentation due to all ice nucleation (hom+immersion+Meyer mixed)',  phys_decomp) !added by Cheng
            

        call addfld('RHI_LP',  '%', pver, 'A', 'RHi at ice nucleation',  phys_decomp) !added by Cheng

        call addfld('NUC_CLOUD',  '%', pver, 'A', 'Cloud fraction at ice nucleation step',  phys_decomp) !added by Cheng
        
        call addfld('NUC_NI',  '#/kg', pver, 'A', 'Ni at ice nucleation step ',  phys_decomp) !added by Cheng

        call addfld('NUC_QI',  'kg/kg', pver, 'A', 'Qi at ice nucleation step ',  phys_decomp) !added by Cheng
        call add_default ('NUC_QI', 4, 'A')

        call addfld('NUC_RHO',  'kg/kg', pver, 'A', 'Rho at ice nucleation step ',  phys_decomp) !added by Cheng
                
                
        call addfld      ('FREQNI', '1', pver, 'A', 'Occurence frequency of ice nucleation (0 or 1)', phys_decomp) 
        call add_default ('FREQNI', 1, 'A')
        
        call addfld      ('FREQHF', '1', pver, 'A', 'Occurence frequency of homogeneous ice nucleation(0 or 1)', phys_decomp) 
        call add_default ('FREQHF', 1, 'A')        
        
        call addfld      ('WSUBINI', 'm/s', pver, 'A', 'WSUBI x FREQNI homogeneous ice nucleation', phys_decomp) 
        call add_default ('WSUBINI', 1, 'A')        
        
        call addfld      ('WSUBIHF', 'm/s', pver, 'A', 'WSUBI x FREQHF homogeneous ice nucleation', phys_decomp) 
        call add_default ('WSUBIHF', 1, 'A')    

        call addfld      ('WSUB0NI', 'm/s', pver, 'A', 'WSUB0 x FREQNI homogeneous ice nucleation', phys_decomp) 
        call add_default ('WSUB0NI', 1, 'A')        
        
        call addfld      ('WSUB0HF', 'm/s', pver, 'A', 'WSUB0 x FREQHF homogeneous ice nucleation', phys_decomp) 
        call add_default ('WSUB0HF', 1, 'A') 
                

        call addfld      ('WGARYNI', 'm/s', pver, 'A', 'WGARY x FREQNI homogeneous ice nucleation', phys_decomp) 
        call add_default ('WGARYNI', 1, 'A')        
        
        call addfld      ('WGARYHF', 'm/s', pver, 'A', 'WGARY x FREQHF homogeneous ice nucleation', phys_decomp) 
        call add_default ('WGARYHF', 1, 'A') 
 
        call addfld      ('WLGNI', 'm/s', pver, 'A', 'W x FREQNI homogeneous ice nucleation', phys_decomp) 
        call add_default ('WLGNI', 1, 'A')        
        
        call addfld      ('WLGHF', 'm/s', pver, 'A', 'W x FREQHF homogeneous ice nucleation', phys_decomp) 
        call add_default ('WLGHF', 1, 'A') 
 
                   	

   call addfld('fhom     ', 'fraction', pver, 'A', 'Fraction of cirrus where homogeneous freezingoccur'      ,phys_decomp)
   call addfld('WICE     ', 'm/s     ', pver, 'A', 'Vertical velocity Reducing by preexisting ice'           ,phys_decomp)
   call addfld('WEFF     ', 'm/s     ', pver, 'A', 'Effective Vertical velocity for ice nucleation'          ,phys_decomp)
   call addfld ('INnso4    ','#/m3   ', pver, 'A','Number Concentation so4 used for ice_nucleation',phys_decomp)
   call addfld ('INnbc     ','#/m3   ', pver, 'A','Number Concentation SOA only  used for ice_nucleation',phys_decomp)
   call addfld ('INndust   ','#/m3   ', pver, 'A','Number Concentation dustused for ice_nucleation',phys_decomp)
   call addfld ('INhet     ','#/m3   ', pver, 'A','contribution for in-cloud ice number density increase by het nucleation in ice cloud',phys_decomp)
   call addfld ('INhom     ','#/m3   ', pver, 'A','contribution for in-cloud ice number density increase by hom nucleation in ice cloud',phys_decomp)
   call addfld ('INFrehom  ','frequency',pver, 'A','hom IN frequency ice cloud',phys_decomp)
   call addfld ('INFreIN   ','frequency',pver, 'A','frequency of ice nucleation occur',phys_decomp)

      call add_default ('fhom     ', 1, ' ')
      call add_default ('WICE     ', 1, ' ')
      call add_default ('WEFF     ', 1, ' ')
      call add_default ('WSUBI    ', 1, ' ')
      call add_default ('INnso4  ', 1, ' ')
      call add_default ('INnbc   ', 1, ' ')
      call add_default ('INndust ', 1, ' ')
      call add_default ('INhet   ', 1, ' ')
      call add_default ('INhom   ', 1, ' ')
      call add_default ('INFrehom', 1, ' ')
      call add_default ('INFreIN ', 1, ' ')

      

!------
   if (history_amwg) then
      call add_default ('WSUB     ', 1, ' ')
   end if

end subroutine microp_aero_init

!===============================================================================

subroutine microp_aero_readnl(nlfile)

   use namelist_utils,  only: find_group_name
   use units,           only: getunit, freeunit
   use mpishorthand

   character(len=*), intent(in) :: nlfile  ! filepath for file containing namelist input

   ! Namelist variables
   real(r8) :: microp_aero_bulk_scale = 2._r8  ! prescribed aerosol bulk sulfur scale factor
 
   ! Local variables
   integer :: unitn, ierr
   character(len=*), parameter :: subname = 'microp_aero_readnl'

   namelist /microp_aero_nl/ microp_aero_bulk_scale
   !-----------------------------------------------------------------------------

   if (masterproc) then
      unitn = getunit()
      open( unitn, file=trim(nlfile), status='old' )
      call find_group_name(unitn, 'microp_aero_nl', status=ierr)
      if (ierr == 0) then
         read(unitn, microp_aero_nl, iostat=ierr)
         if (ierr /= 0) then
            call endrun(subname // ':: ERROR reading namelist')
         end if
      end if
      close(unitn)
      call freeunit(unitn)
   end if

#ifdef SPMD
   ! Broadcast namelist variable
   call mpibcast(microp_aero_bulk_scale, 1, mpir8, 0, mpicom)
#endif

   ! set local variables
   bulk_scale = microp_aero_bulk_scale

end subroutine microp_aero_readnl

!===============================================================================

subroutine microp_aero_run ( &
   sgh, & !added by Cheng
   state, ptend, deltatin, pbuf)

   ! input arguments
       real(r8), intent(in) :: sgh(pcols)                   ! Std. deviation of orography for tms ! added by Cheng
   type(physics_state), target, intent(in)    :: state
   type(physics_ptend),         intent(out)   :: ptend
   real(r8),                    intent(in)    :: deltatin     ! time step (s)
   type(physics_buffer_desc),   pointer       :: pbuf(:)




   ! local workspace
   ! all units mks unless otherwise stated

   integer :: i, k, m
   integer :: itim
   integer :: lchnk
   integer :: ncol
   integer :: nmodes
   integer :: nucboast

   real(r8), pointer :: ast(:,:)        

   real(r8)          :: icecldf(pcols,pver)    ! ice cloud fraction   
   real(r8)          :: liqcldf(pcols,pver)    ! liquid cloud fraction

   real(r8), pointer :: naai(:,:)       ! number of activated aerosol for ice nucleation 
   real(r8), pointer :: naai_hom(:,:)   ! number of activated aerosol for ice nucleation (homogeneous freezing only)
   real(r8), pointer :: npccn(:,:)      ! number of CCN (liquid activated)
   real(r8), pointer :: rndst(:,:,:)    ! radius of 4 dust bins for contact freezing
   real(r8), pointer :: nacon(:,:,:)    ! number in 4 dust bins for contact freezing

   real(r8), pointer :: t(:,:)          ! input temperature (K)
   real(r8), pointer :: qn(:,:)         ! input water vapor mixing ratio (kg/kg)
   ! note: all input cloud variables are grid-averaged
   real(r8), pointer :: qc(:,:)         ! cloud water mixing ratio (kg/kg)
   real(r8), pointer :: qi(:,:)         ! cloud ice mixing ratio (kg/kg)
   real(r8), pointer :: nc(:,:)         ! cloud water number conc (1/kg)
   real(r8), pointer :: ni(:,:)         ! cloud ice number conc (1/kg)
   real(r8), pointer :: pmid(:,:)       ! pressure at layer midpoints (pa)
   real(r8), pointer :: pdel(:,:)       ! pressure difference across level (pa)
   real(r8), pointer :: pint(:,:)       ! air pressure layer interfaces (pa)
   real(r8), pointer :: rpdel(:,:)      ! inverse pressure difference across level (pa)
   real(r8), pointer :: zm(:,:)         ! geopotential height of model levels (m)
   real(r8), pointer :: omega(:,:)      ! vertical velocity (Pa/s)
   real(r8), pointer :: num_accum(:,:)  ! number m.r. of accumulation mode
   real(r8), pointer :: num_aitken(:,:) ! number m.r. of aitken mode
   real(r8), pointer :: num_coarse(:,:) ! number m.r. of coarse mode
   real(r8), pointer :: coarse_dust(:,:) ! mass m.r. of coarse dust
   real(r8), pointer :: coarse_nacl(:,:) ! mass m.r. of coarse nacl

   real(r8), pointer :: kvh(:,:)        ! vertical eddy diff coef (m2 s-1)
   real(r8), pointer :: tke(:,:)        ! TKE from the UW PBL scheme (m2 s-2)
   real(r8), pointer :: wp2(:,:)        ! CLUBB vertical velocity variance

   real(r8), pointer :: cldn(:,:)       ! cloud fraction
   real(r8), pointer :: cldo(:,:)       ! old cloud fraction

   real(r8), pointer :: dgnum(:,:,:)    ! aerosol mode dry diameter
   real(r8), pointer :: dgnumwet(:,:,:) ! aerosol mode diameter

   real(r8), pointer :: aer_mmr(:,:)    ! aerosol mass mixing ratio

   real(r8) :: rho(pcols,pver)     ! air density (kg m-3)
   real(r8) :: relhum(pcols,pver)  ! relative humidity
   real(r8) :: icldm(pcols,pver)   ! ice cloud fraction
   real(r8) :: lcldm(pcols,pver)   ! liq cloud fraction
   real(r8) :: nfice(pcols,pver)   ! fice variable
   real(r8) :: dumfice             ! dummy var in fice calc
   real(r8) :: lcldn(pcols,pver)   ! fractional coverage of new liquid cloud
   real(r8) :: lcldo(pcols,pver)   ! fractional coverage of old liquid cloud
   real(r8) :: qcld                ! total cloud water
   real(r8) :: nctend_mixnuc(pcols,pver)
   real(r8) :: dum, dum2           ! temporary dummy variable
   real(r8) :: dmc, ssmc           ! variables for modal scheme.

   real(r8) :: so4_num                               ! so4 aerosol number (#/cm^3)
   real(r8) :: soot_num                              ! soot (hydrophilic) aerosol number (#/cm^3)
   real(r8) :: dst1_num,dst2_num,dst3_num,dst4_num   ! dust aerosol number (#/cm^3)
   real(r8) :: dst_num                               ! total dust aerosol number (#/cm^3)

   real(r8) :: qs(pcols)            ! liquid-ice weighted sat mixing rat (kg/kg)
   real(r8) :: es(pcols)            ! liquid-ice weighted sat vapor press (pa)
   real(r8) :: gammas(pcols)        ! parameter for cond/evap of cloud water

   ! bulk aerosol variables
   real(r8), allocatable :: naer2(:,:,:)    ! bulk aerosol number concentration (1/m3)
   real(r8), allocatable :: maerosol(:,:,:) ! bulk aerosol mass conc (kg/m3)

   real(r8) :: wsub(pcols,pver)    ! diagnosed sub-grid vertical velocity st. dev. (m/s)
   real(r8) :: wsubi(pcols,pver)   ! diagnosed sub-grid vertical velocity ice (m/s)
!++sxj
   real(r8) :: fhom(pcols,pver)   ! how much fraction of ice cloud where hom freezing might occur
   real(r8) :: wice(pcols,pver)    ! diagnosed Vertical velocity Reducing by preexisting ice (m/s)
   real(r8) :: weff(pcols,pver)    ! effective Vertical velocity for ice nucleation (m/s); weff=wsubi-wice 
   real(r8) :: INnso4(pcols,pver)   ! #/m3
   real(r8) :: INnbc(pcols,pver)    ! #/m3
   real(r8) :: INndust(pcols,pver)  ! #/m3
   real(r8) :: INhet(pcols,pver)  ! #/m3
   real(r8) :: INhom(pcols,pver)  ! #/m3
   real(r8) :: INFrehom(pcols,pver) !  hom IN frequency
   real(r8) :: INFreIN(pcols,pver)  !  IN frequency
   real(r8) :: so4_rwet,so4_sig,dst_rwet,dst_sig,Simean,detaT
   real(r8) :: naaitemp,nhomtemp,nhettemp,nmeytemp
!--sxj  
   ! history output for ice nucleation
   real(r8) :: nihf(pcols,pver)  !output number conc of ice nuclei due to heterogenous freezing (1/m3)
   real(r8) :: niimm(pcols,pver) !output number conc of ice nuclei due to immersion freezing (hetero nuc) (1/m3)
   real(r8) :: nidep(pcols,pver) !output number conc of ice nuclei due to deoposion nucleation (hetero nuc) (1/m3)
   real(r8) :: nimey(pcols,pver) !output number conc of ice nuclei due to meyers deposition (1/m3)
   real(r8) :: wght

   !below are added by Cheng
   real(r8) :: ww!! final updraft velocity used in LP
   real(r8) :: wlarge(pcols,pver) ! large-scale updraft velocity
   real(r8) :: wsub0(pcols,pver)    ! diagnosed sub-grid vertical velocity st. dev. (m/s),w/o limiters ! added by Cheng
   real(r8) :: naai_temp(pcols,pver)  !output number conc of ice nuclei due to all above pathways (1/m3) !added by Cheng
   real(r8) :: rhi_lp(pcols,pver) !rh w.r.t. ice !added by Cheng
   real(r8)::  NUC_CLOUD(pcols,pver),NUC_QI(pcols,pver),NUC_NI(pcols,pver) 
   real(r8) :: wgary(pcols,pver)  !added by Cheng
   real(r8) :: freq_ni(pcols,pver) ! ice nucleation occurence frequence
   real(r8) :: freq_hf(pcols,pver) ! homo ice nucleation occurence frequence
   real(r8) :: wsubi_ni(pcols,pver) ! ice nucleation occurence frequence x wsubi
   real(r8) :: wsubi_hf(pcols,pver) ! ice nucleation occurence frequence x wsubi
   real(r8) :: wsub0_ni(pcols,pver) ! ice nucleation occurence frequence x wsubi
   real(r8) :: wsub0_hf(pcols,pver) ! ice nucleation occurence frequence x wsubi   
   real(r8) :: wgary_ni(pcols,pver) ! ice nucleation occurence frequence x wsubi   
   real(r8) :: wgary_hf(pcols,pver) ! ice nucleation occurence frequence x wsubi
   real(r8) :: wlarge_ni(pcols,pver) ! ice nucleation occurence frequence x wsubi   
   real(r8) :: wlarge_hf(pcols,pver) ! ice nucleation occurence frequence x wsubi
   real(r8), dimension(pcols,pver) :: um_SOA_IN_num,IN_hetero_dust,IN_hetero_asoot,IN_hetero_soot, IN_homo1, IN_homo2
   real(r8) :: nm(pcols,pver)       ! midpoint Brunt-Vaisalla frequency
   	
   real(r8):: esl, esi, deles,Na_soot_air,Na_so42
   !------------------------------------------------------------------------------

   lchnk = state%lchnk
   ncol  = state%ncol
   t     => state%t
   qn    => state%q(:,:,1)
   qc    => state%q(:,:,cldliq_idx)
   qi    => state%q(:,:,cldice_idx)
   nc    => state%q(:,:,numliq_idx)
   ni    => state%q(:,:,numice_idx)
   pmid  => state%pmid
   pdel  => state%pdel
   pint  => state%pint
   rpdel => state%rpdel
   zm    => state%zm
   omega => state%omega

   itim = pbuf_old_tim_idx()
   call pbuf_get_field(pbuf, ast_idx,      ast, start=(/1,1,itim/), kount=(/pcols,pver,1/))

   liqcldf(:ncol,:pver) = ast(:ncol,:pver)
   icecldf(:ncol,:pver) = ast(:ncol,:pver)

   call pbuf_get_field(pbuf, naai_idx, naai)
   call pbuf_get_field(pbuf, naai_hom_idx, naai_hom)
   call pbuf_get_field(pbuf, npccn_idx, npccn)
   call pbuf_get_field(pbuf, nacon_idx, nacon)
   call pbuf_get_field(pbuf, rndst_idx, rndst)

   if (clim_modal_aero) then

      itim = pbuf_old_tim_idx()
      call pbuf_get_field(pbuf, ast_idx,  cldn, start=(/1,1,itim/), kount=(/pcols,pver,1/) )
      call pbuf_get_field(pbuf, cldo_idx, cldo, start=(/1,1,itim/), kount=(/pcols,pver,1/) )

      call rad_cnst_get_info(0, nmodes=nmodes)
      call pbuf_get_field(pbuf, dgnum_idx,    dgnum,    start=(/1,1,1/), kount=(/pcols,pver,nmodes/) )
      call pbuf_get_field(pbuf, dgnumwet_idx, dgnumwet, start=(/1,1,1/), kount=(/pcols,pver,nmodes/) )
   end if

   ! initialize output
   naai(1:ncol,1:pver)     = 0._r8  
   naai_hom(1:ncol,1:pver) = 0._r8  
   npccn(1:ncol,1:pver)    = 0._r8  
   nacon(1:ncol,1:pver,:)  = 0._r8

   ! set default or fixed dust bins for contact freezing
   rndst(1:ncol,1:pver,1) = rn_dst1
   rndst(1:ncol,1:pver,2) = rn_dst2
   rndst(1:ncol,1:pver,3) = rn_dst3
   rndst(1:ncol,1:pver,4) = rn_dst4

   ! initialize history output fields for ice nucleation
   nihf(1:ncol,1:pver)  = 0._r8  
   niimm(1:ncol,1:pver) = 0._r8  
   nidep(1:ncol,1:pver) = 0._r8 
   nimey(1:ncol,1:pver) = 0._r8 
   	
   naai_temp(1:pcols,1:pver) = 0._r8 !added by Cheng,warning, naai is a pointer, must assign all naai_temp to avoid unwanted values
	 rhi_lp(1:pcols,1:pver) = 0._r8  !added by Cheng
	 wgary (1:pcols,1:pver) = 0._r8  !added by Cheng
	 freq_ni (1:pcols,1:pver) = 0._r8  !added by Cheng
	 freq_hf (1:pcols,1:pver) = 0._r8  !added by Cheng
	 wlarge(1:pcols,1:pver) = 0._r8  !added by Cheng
	 wsub0(1:pcols,1:pver) = 0._r8  !added by Cheng 	
	 NUC_CLOUD(1:pcols,1:pver) = 0._r8  !added by Cheng
	 NUC_QI(1:pcols,1:pver) = 0._r8  !added by Cheng
	 NUC_NI(1:pcols,1:pver) = 0._r8  !added by Cheng
	 			
   ! initialize time-varying parameters
   do k = top_lev, pver
      do i = 1, ncol
         rho(i,k) = pmid(i,k)/(rair*t(i,k))
      end do
   end do

   if (clim_modal_aero) then
      ! mode number mixing ratios
      call rad_cnst_get_mode_num(0, mode_accum_idx,  'a', state, pbuf, num_accum)
      call rad_cnst_get_mode_num(0, mode_aitken_idx, 'a', state, pbuf, num_aitken)
      call rad_cnst_get_mode_num(0, mode_coarse_dst_idx, 'a', state, pbuf, num_coarse)

      ! mode specie mass m.r.
      call rad_cnst_get_aer_mmr(0, mode_coarse_dst_idx, coarse_dust_idx, 'a', state, pbuf, coarse_dust)
      call rad_cnst_get_aer_mmr(0, mode_coarse_slt_idx, coarse_nacl_idx, 'a', state, pbuf, coarse_nacl)

   else
      ! init number/mass arrays for bulk aerosols
      allocate( &
         naer2(pcols,pver,naer_all), &
         maerosol(pcols,pver,naer_all))

      do m = 1, naer_all
         call rad_cnst_get_aer_mmr(0, m, state, pbuf, aer_mmr)
         maerosol(:ncol,:,m) = aer_mmr(:ncol,:)*rho(:ncol,:)

         if (m .eq. idxsul) then
            naer2(:ncol,:,m) = maerosol(:ncol,:,m)*num_to_mass_aer(m)*bulk_scale
         else
            naer2(:ncol,:,m) = maerosol(:ncol,:,m)*num_to_mass_aer(m)
         end if
      end do
   end if

   !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
   ! More refined computation of sub-grid vertical velocity 
   ! Set to be zero at the surface by initialization.

   select case (trim(eddy_scheme))
   case ('diag_TKE')
      call pbuf_get_field(pbuf, tke_idx, tke)
   case ('CLUBB_SGS')
      itim = pbuf_old_tim_idx()
      call pbuf_get_field(pbuf, wp2_idx, wp2, start=(/1,1,itim/),kount=(/pcols,pverp,1/))
      allocate(tke(pcols,pverp))
      tke(:ncol,:) = (3._r8/2._r8)*wp2(:ncol,:)
   case default
      call pbuf_get_field(pbuf, kvh_idx, kvh)
   end select

   ! Set minimum values above top_lev.
   wsub(:ncol,:top_lev-1)  = 0.20_r8
   wsubi(:ncol,:top_lev-1) = 0.001_r8
   wsub0(:ncol,:top_lev-1)  = 0.0_r8 ! added by Cheng
!++sxj
   fhom(:,:) = 0.0_r8
   wice(:,:) = 0.0_r8
   weff(:,:) = 0.0_r8
   INnso4(:,:) = 0.0_r8
   INnbc(:,:)  = 0.0_r8
   INndust(:,:)= 0.0_r8
   INhet(:,:)  = 0.0_r8
   INhom(:,:) = 0.0_r8
   INFrehom(:,:) = 0.0_r8
   INFreIN(:,:) = 0.0_r8
!--sxj


!++++ calculate the Brunt-Vaisala frequency, by Cheng
    call um_brunt_vaisala_cal (lchnk, ncol, t, pmid, pint, nm)
    call outfld( 'um_nm'      , nm,     pcols, lchnk )
!----	


   do k = top_lev, pver
      do i = 1, ncol

         select case (trim(eddy_scheme))
         case ('diag_TKE', 'CLUBB_SGS')
               wsub(i,k) = sqrt(0.5_r8*(tke(i,k) + tke(i,k+1))*(2._r8/3._r8))
               wsub0(i,k) =wsub(i,k) !added by Cheng
               wsub(i,k) = min(wsub(i,k),10._r8)
         case default 
            ! get sub-grid vertical velocity from diff coef.
            ! following morrison et al. 2005, JAS
            ! assume mixing length of 30 m
               dum = (kvh(i,k) + kvh(i,k+1))/2._r8/30._r8
            ! use maximum sub-grid vertical vel of 10 m/s
               dum = min(dum, 10._r8)
            ! set wsub to value at current vertical level
               wsub(i,k)  = dum
	 end select

         wsubi(i,k) = min(max(0.001_r8, wsub(i,k)),2.0_r8)
!         wsubi(i,k) = min(wsubi(i,k), 0.2_r8)


!++++ output the large_scale updraft velocity
   wlarge(:ncol,:) =  state%pmid(:ncol,:)/state%t(:ncol,:)/287.058_r8 !! get rho, p = rho R t
   wlarge(:ncol,:) = -state%omega(:ncol,:)/wlarge(:ncol,:)/9.80665_r8 !! omega = -rho g W
   call outfld ('WLG      ',wlarge    ,pcols   ,lchnk     )	
            
! derive subgrid updraft velocity following Wang and Penner 2010 which is based on Gary 2006/2008
   call meso_temp_gary0608(lchnk, ncol, state, sgh, wgary)
!-----
	 
#ifdef CLUBB_SGS
	 if (wsubi(i,k) .le. 0.04_r8) then
           nucboast=100._r8
	   wsubi(i,k)=nucboast*wsubi(i,k)  ! boost ice SGS vertical velocity in CAM-CLUBB
	   				   ! to force nucleation in upper-level stratiform 
					   ! clouds.  Temporary fix until cloud-top radiative
					   ! cooling parameterization is added to CLUBB similar
					   ! to the one of appendix C of Bretherton and Park (2009).  
	 endif
#endif
	 
         wsub(i,k)  = max(0.20_r8, wsub(i,k))
      end do
   end do
   call outfld( 'WSUB'       , wsub,      pcols, lchnk )
   call outfld( 'WSUBI'      , wsubi,     pcols, lchnk )
   call outfld( 'WSUB0'      , wsub0,     pcols, lchnk ) !added by Cheng
   
   if (trim(eddy_scheme) == 'CLUBB_SGS') deallocate(tke)



   !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
   !Get humidity and saturation vapor pressures

   ! find wet bulk temperature and saturation value for provisional t and q without
   ! condensation

   do k = top_lev, pver

      call qsat_water(t(:ncol,k), pmid(:ncol,k), &
           es(:ncol), qs(:ncol), gam=gammas(:ncol))

      do i = 1, ncol

         relhum(i,k) = qn(i,k)/qs(i)
         rhi_lp(i,k) = relhum(i,k)*svp_water(t(i,k))/svp_ice(t(i,k)) !added by Cheng

         ! get cloud fraction, check for minimum
         icldm(i,k) = max(icecldf(i,k), mincld)
         lcldm(i,k) = max(liqcldf(i,k), mincld)

         ! calculate nfice based on liquid and ice mmr (no rain and snow mmr available yet)
         nfice(i,k) = 0._r8
         dumfice    = qc(i,k) + qi(i,k)
         if (dumfice > qsmall .and. qi(i,k) > qsmall) then
            nfice(i,k) = qi(i,k)/dumfice
         end if
      end do
   end do

   !++ Cheng, get homo/heterogenous IN number
      call um_IN_numbers(state,IN_hetero_dust,IN_hetero_asoot,IN_hetero_soot, IN_homo1, IN_homo2)
      call um_SOA_IN_num_cal(um_SOA_IN_num,relhum, t, rho, ncol,lchnk) ! the fraction of SOA being IN is already applied in um_SOA_IN_num_cal, do not apply any fraction here
      !JIALEI add rho
      
   !--
   
   
   !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
   !ICE Nucleation

   do k = top_lev, pver
      do i = 1, ncol

         if (t(i,k).lt.tmelt - 5._r8) then


            ! compute aerosol number for so4, soot, and dust with units #/cm^3
            so4_num  = 0._r8
            soot_num = 0._r8
            dst1_num = 0._r8
            dst2_num = 0._r8
            dst3_num = 0._r8
            dst4_num = 0._r8
            dst_num  = 0._r8

            if (clim_modal_aero) then
               !For modal aerosols, assume for the upper troposphere:
               ! soot = accumulation mode
               ! sulfate = aiken mode
               ! dust = coarse mode
               ! since modal has internal mixtures.
               soot_num = num_accum(i,k)*rho(i,k)*1.0e-6_r8
               dmc  = coarse_dust(i,k)*rho(i,k)
               ssmc = coarse_nacl(i,k)*rho(i,k)

               if ( separate_dust ) then
                  ! 7-mode -- has separate dust and seasalt mode types and no need for weighting 
                  wght = 1._r8
               else
                  ! 3-mode -- needs weighting for dust since dust and seasalt are combined in the "coarse" mode type
                  wght = dmc/(ssmc + dmc)
               endif

               if (dmc > 0._r8) then
                  dst_num = wght * num_coarse(i,k)*rho(i,k)*1.0e-6_r8
               else 
                  dst_num = 0.0_r8
               end if

               if (dgnum(i,k,mode_aitken_idx) > 0._r8) then
                  ! only allow so4 with D>0.1 um in ice nucleation
                  so4_num  = num_aitken(i,k)*rho(i,k)*1.0e-6_r8 &
                     * (0.5_r8 - 0.5_r8*erf(log(0.1e-6_r8/dgnum(i,k,mode_aitken_idx))/  &
                     (2._r8**0.5_r8*log(sigmag_aitken))))
               else 
                  so4_num = 0.0_r8 
               end if
               so4_num = max(0.0_r8, so4_num)

            else

               if (idxsul > 0) then 
                  so4_num = naer2(i,k,idxsul)/25._r8 *1.0e-6_r8
               end if
               if (idxbcphi > 0) then 
                  soot_num = naer2(i,k,idxbcphi)/25._r8 *1.0e-6_r8
               end if
               if (idxdst1 > 0) then 
                  dst1_num = naer2(i,k,idxdst1)/25._r8 *1.0e-6_r8
               end if
               if (idxdst2 > 0) then 
                  dst2_num = naer2(i,k,idxdst2)/25._r8 *1.0e-6_r8
               end if
               if (idxdst3 > 0) then 
                  dst3_num = naer2(i,k,idxdst3)/25._r8 *1.0e-6_r8
               end if
               if (idxdst4 > 0) then 
                  dst4_num = naer2(i,k,idxdst4)/25._r8 *1.0e-6_r8
               end if
               dst_num = dst1_num + dst2_num + dst3_num + dst4_num

            end if

            ! *** Turn off soot nucleation ***
            soot_num = 0.0_r8

!--- Cheng, modified by Cheng, 0.1% of soot acting as IN
              !so4_num  = IN_homo(i,k)    !  from IMPACT module, 100% Aitken + Accumu mode and 10% hygroscopic fossil soot. x0.5 tunning factor is applied   
              !dst_num  = IN_hetero(i,k)  ! + um_SOA_IN_num(i,k)  !  from IMPACT module, 0.05% hydraphobic fossil soot, 0.1% hydraphilic fossil soot, 1% biomass soot, 10% dust, no aircraft soot    
              dst_num  = IN_hetero_dust(i,k) 
              Na_soot_air = IN_hetero_asoot(i,k) 
              soot_num =    IN_hetero_soot(i,k) + um_SOA_IN_num(i,k) 
              so4_num  = IN_homo1(i,k) 
              Na_so42  = IN_homo2(i,k) 

!---

            INnso4(i,k) =(so4_num+Na_so42)*1e6_r8  ! (convert from #/cm3 -> #/m3)
            INnbc(i,k)  = um_SOA_IN_num(i,k)*1e6_r8 !(soot_num+Na_soot_air)*1e6_r8
            INndust(i,k)=dst_num*1e6_r8
            INFreIN(i,k)=1.0_r8          ! 1,ice_nucleation occur
            

!! for cold cirrus (<205K) use large-scale w, for warm cirrus use wsubi
!           if (t(i,k) .le. 205.0_r8) then
!           	 ww_para = 4
!           	 else
!           	 ww_para = 2	
!           endif
!!           

! if Brunt-Vaisala frequency <0.01 use large-scale w else use wsubi
!           if (nm(i,k) .ge. 0.01_r8 .and. t(i,k) .le. 240.0_r8) then
!           	 ww_para = 4
!           	 else
!           	 ww_para = 2	
!           endif
!           
      
! Update Wgary 1) consider the chagne of Brunt-Vaisala frequency. 2) scale by Podlagen at 19km using 0.17 m/s
           !nm(i,k) = max(nm(i,k),0.001_r8) ! set the minimum frequency , nm is already limited >1e-4
           wgary(i,k) = wgary(i,k)*sqrt(0.02_r8/nm(i,k))*1.7 ! consider stability change, futher scaled by Podglajen et al. 2016 so that at 19km at the equator, wgary=0.17m/s. 
           wgary(i,k) = min(wgary(i,k),2.0_r8) ! set upper limit to 2 m/s

  

! store ice cloud fractions  
           NUC_QI(i,k)= qi(i,k)
           NUC_NI(i,k)= Ni(i,k)
           NUC_CLOUD(i,k)= icldm(i,k)
  	         
           select case (ww_para)   
           case(1)
             ww = min(wsubi(i,k), 0.2_r8) !! CAM5.3 default wsubi
           case(2) 
             ww = max(0.0001_r8, wsubi(i,k))  !! no limitation on wsub
           case(3)
             ww = wgary(i,k) ! use gary
           case(4)
             ww = max(0.0001_r8,  wlarge(i,k) ) !!! use grid box mean updraft velocity
            case default
               write(*,*) " ww_para should be  1 2 3 or 4"
               call endrun ('ww_para')
            end select
            
            if (OR_fhom) then
               !detaT=wsubi(i,k)/0.23_r8
               detaT=ww/0.23_r8
               Simean=1.0_r8
               call frachom(t(i,k),Simean,detaT,fhom(i,k))
            else
               fhom(i,k)=1.0_r8
            endif
            

            select case (ice_para)            
!            call nucleati( &
!               wsubi(i,k), t(i,k), relhum(i,k), icldm(i,k), qc(i,k), &
!               nfice(i,k), rho(i,k), so4_num, dst_num, soot_num,     &
!               naai(i,k), nihf(i,k), niimm(i,k), nidep(i,k), nimey(i,k))

!            call nucleati( &
!               wgary(i,k), t(i,k), relhum(i,k), icldm(i,k), qc(i,k), &
!               nfice(i,k), rho(i,k), so4_num, dst_num, soot_num,     &
!               naai(i,k), nihf(i,k), niimm(i,k), nidep(i,k), nimey(i,k))
               
            case (2)
               call nucleati( &
                  ww, t(i,k), relhum(i,k), icldm(i,k), qc(i,k), &    !!! add large-scale updraft velocity 
                  nfice(i,k), rho(i,k), so4_num, dst_num, soot_num,     &
                  fhom(i,k),OR_purehom, OR_preice, qi(i,k), ni(i,k), pmid(i,k), & ! sxj
                  wice(i,k), weff(i,k),  & ! sxj
                  naai(i,k), nihf(i,k), niimm(i,k), nidep(i,k), nimey(i,k)) ! output unit is #/kg
                  
                  
            naai_hom(i,k) = nihf(i,k)

            ! output activated ice (convert from #/kg -> #/m3)
            nihf(i,k)     = nihf(i,k) *rho(i,k)
            niimm(i,k)    = niimm(i,k)*rho(i,k)
            nidep(i,k)    = nidep(i,k)*rho(i,k)
            nimey(i,k)    = nimey(i,k)*rho(i,k)
            naai_temp(i,k) = naai(i,k)*rho(i,k)!added by Cheng
                !sxj, variables for making plots
                INhet(i,k) = niimm(i,k) + nidep(i,k)   ! #/m3, nimey not in cirrus
                INhom(i,k) = nihf(i,k)     ! #/m3
                if (INhom(i,k).gt.1e3_r8)   then ! > 1/L
                   INFrehom(i,k)=1.0_r8       ! 1, hom freezing occur
                endif
            
            !!!!!!!!!BN!!!!!!!!!!!!!!!! 
           case (3)
               
               call IceBN(t(i,k),pmid(i,k), &
                      so4_num, dst_num, soot_num, &
                      ww, relhum(i,k), rho(i,k), icldm(i,k), qc(i,k), &
                      fhom(i,k),OR_purehom, OR_preice, qi(i,k), ni(i,k), &
                      wice(i,k), weff(i,k), & 
                      naaitemp, nhomtemp, nhettemp,nmeytemp)

                naai(i,k)=naaitemp
                naai_hom(i,k) = nhomtemp ! unit is #/kg
                
                ! output activated ice (convert from #/kg -> #/m3)
                INhom(i,k) = nhomtemp *rho(i,k)
                INhet(i,k) = nhettemp *rho(i,k)
                nimey(i,k) = nmeytemp *rho(i,k)           
 
                !sxj, variables for making plots                    
                if (INhom(i,k).gt.1e3_r8)   then ! > 1/L
                   INFrehom(i,k)=1.0_r8          ! 1, hom freezing occur
                endif
 
 
             !!!!!!!!!KL!!!!!!!!!!!!!!!! 
           case (4)

! already done at the beginning              
!              naai(i,k)  = 0.0_r8
!              nihf(i,k)  = 0.0_r8
!              niimm(i,k) = 0.0_r8
!              nidep(i,k) = 0.0_r8
!              nimey(i,k) = 0.0_r8
              
              !Na_soot_air = 0.0_r8
              !Na_so42     = 0.0_r8
              
              if (t(i,k).lt.tmelt - 35._r8 .and. ww .ge. 0.001_r8) then
               call KL_ni(so4_num,Na_so42, &!r_so41,r_so42,sg1_so4,sg2_so4, 
                       soot_num,Na_soot_air,dst_num,&
                       ww,t(i,k),rhi_lp(i,k),pmid(i,k), &
                       naai(i,k), nihf(i,k), niimm(i,k), nidep(i,k) ) !,RHmax) ! output's unit is #/cm3
              end if
!      SUBROUTINE KL_ni(Na_so41,Na_so42, !r_so41,r_so42,sg1_so4,sg2_so4, &
!                       Na_soot_sf,Na_soot_air,Na_dust,&
!                       wstd,TK,RH,P, &
!                       Nitot,Fhomo,Fimme,Fdepo) !,RHmax)
                       

   ! deposition/condensation nucleation in mixed clouds (-40<T<0C) (Meyers, 1992)                                     	            	         
              if (  (t(i,k) .lt. tmelt) .and. (t(i,k).gt.tmelt - 37._r8).and.  qc(i,k).gt.1.e-12_r8) then 
               esl = svp_water(t(i,k))    ! over water in mixed clouds
               esi = svp_ice(t(i,k))      ! over ice
               deles = (esl - esi)
               nimey(i,k)=1.e-3_r8*exp(12.96_r8*deles/esi - 0.639_r8)  ! #/cm3
              end if
           	   naai(i,k) = naai(i,k) +   nimey(i,k)
           	   
               naai(i,k) = naai(i,k)     * 1e6 / rho(i,k) ! from #/cm3 to #/m3 then to #/kg           	       
               naai_hom(i,k) = nihf(i,k) * 1e6 / rho(i,k)

            ! output activated ice (convert from #/cm3 -> #/m3)
            nihf(i,k)     = nihf(i,k) *1e6
            niimm(i,k)    = niimm(i,k)*1e6
            nidep(i,k)    = nidep(i,k)*1e6
            nimey(i,k)    = nimey(i,k)*1e6
            naai_temp(i,k) = naai(i,k)*rho(i,k)!added by Cheng
            
                !sxj, variables for making plots
                INhet(i,k) = niimm(i,k) + nidep(i,k)   ! #/m3, nimey not in cirrus
                INhom(i,k) = nihf(i,k)     ! #/m3
                if (INhom(i,k).gt.1e3_r8)   then ! > 1/L
                   INFrehom(i,k)=1.0_r8       ! 1, hom freezing occur
                endif
            
            
                      
                               
            case default
               write(*,*) " ice_para should be  1 2 3 or 4"
               call endrun ('ice_para')
            end select
            
                                          
            !++ added by Cheng
            if ( nihf(i,k) .gt. 10) then
            	freq_hf(i,k)=1.0_r8
            endif
            if ( naai_temp(i,k) .gt. 10) then
            	freq_ni(i,k)=1.0_r8
            endif            	
            !--
            
            
                        ! exclude  no ice nucleaton occur 
            if ((INFrehom(i,k).lt.0.5_r8).and.(INhet(i,k).lt.1.1_r8))   then   
                INnso4(i,k) =0.0_r8
                INnbc(i,k)  =0.0_r8
                INndust(i,k)=0.0_r8
                INFreIN(i,k)=0.0_r8
                INFrehom(i,k)=0.0_r8    
                INhom(i,k) = 0.0_r8
                INhet(i,k) = 0.0_r8
                wice(i,k) = 0.0_r8
                weff(i,k) = 0.0_r8
             !   fhom(i,k) = 0.0_r8 
            endif
            
            
         end if
      end do
   end do

!++sxj
   call outfld( 'fhom' , fhom, pcols, lchnk)
   call outfld( 'WICE' , wice, pcols, lchnk)
   call outfld( 'WEFF' , weff, pcols, lchnk)
   call outfld('INnso4  ',INnso4 , pcols,lchnk)
   call outfld('INnbc   ',INnbc  , pcols,lchnk)
   call outfld('INndust ',INndust, pcols,lchnk)
   call outfld('INhet   ',INhet  , pcols,lchnk)
   call outfld('INhom   ',INhom  , pcols,lchnk)
   call outfld('INFrehom',INFrehom,pcols,lchnk)
   call outfld('INFreIN ',INFreIN, pcols,lchnk)
!--sxj

   call outfld('NIHF',   nihf, pcols, lchnk)
   call outfld('NIIMM', niimm, pcols, lchnk)
   call outfld('NIDEP', nidep, pcols, lchnk)
   call outfld('NIMEY', nimey, pcols, lchnk)
   
   
   !+++++Cheng
   wsubi_ni (:ncol,:pver) = wsubi(:ncol,:pver) * freq_ni(:ncol,:pver)
   wsubi_hf (:ncol,:pver) = wsubi(:ncol,:pver) * freq_hf(:ncol,:pver)
   	
   wsub0_ni (:ncol,:pver) = wsub0(:ncol,:pver) * freq_ni(:ncol,:pver)
   wsub0_hf (:ncol,:pver) = wsub0(:ncol,:pver) * freq_hf(:ncol,:pver)
 
   wgary_ni (:ncol,:pver) = wgary(:ncol,:pver) * freq_ni(:ncol,:pver)
   wgary_hf (:ncol,:pver) = wgary(:ncol,:pver) * freq_hf(:ncol,:pver)

   wlarge_ni (:ncol,:pver) = wlarge(:ncol,:pver) * freq_ni(:ncol,:pver)
   wlarge_hf (:ncol,:pver) = wlarge(:ncol,:pver) * freq_hf(:ncol,:pver)
   	   	
   	  	  	 	
   call outfld('NAAI',   naai_temp(:ncol,:pver), ncol, lchnk) 
   call outfld('RHI_LP',   rhi_lp(:ncol,:pver), ncol, lchnk)
   call outfld('WGARY',   wgary(:ncol,:pver), ncol, lchnk)
   
   call outfld('FREQNI',   freq_ni(:ncol,:pver), ncol, lchnk)
   call outfld('FREQHF',   freq_hf(:ncol,:pver), ncol, lchnk)
   call outfld('WSUBINI',   wsubi_ni(:ncol,:pver), ncol, lchnk)
   call outfld('WSUBIHF',   wsubi_hf(:ncol,:pver), ncol, lchnk)
   call outfld('WSUB0NI',   wsub0_ni(:ncol,:pver), ncol, lchnk)
   call outfld('WSUB0HF',   wsub0_hf(:ncol,:pver), ncol, lchnk)   
   call outfld('WGARYNI',   wgary_ni(:ncol,:pver), ncol, lchnk)
   call outfld('WGARYHF',   wgary_hf(:ncol,:pver), ncol, lchnk)      
   call outfld('WLGNI',   wlarge_ni(:ncol,:pver), ncol, lchnk)
   call outfld('WLGHF',   wlarge_hf(:ncol,:pver), ncol, lchnk)
   	
      call outfld('NUC_QI',   NUC_QI(:ncol,:pver), ncol, lchnk)	
      call outfld('NUC_NI',   NUC_NI(:ncol,:pver), ncol, lchnk)	
      call outfld('NUC_CLOUD',   NUC_CLOUD(:ncol,:pver), ncol, lchnk)	
      call outfld('NUC_RHO',   rho(:ncol,:pver), ncol, lchnk)	
      	
      	
   !-----Cheng
      
   if (clim_modal_aero) then

      !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      !droplet activation for modal aerosol

      ! partition cloud fraction into liquid water part
      lcldn = 0._r8
      lcldo = 0._r8
      do k = top_lev, pver
         do i = 1, ncol
            qcld = qc(i,k) + qi(i,k)
            if (qcld > qsmall) then
               lcldn(i,k) = cldn(i,k)*qc(i,k)/qcld
               lcldo(i,k) = cldo(i,k)*qc(i,k)/qcld
            end if
         end do
      end do

      call outfld('LCLOUD', lcldn, pcols, lchnk)

      call dropmixnuc( &
         state, ptend, deltatin, pbuf, wsub, &
         lcldn, lcldo, nctend_mixnuc)

      npccn(:ncol,:) = nctend_mixnuc(:ncol,:)

   else

      !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      !droplet activation for bulk aerosol

      ! no tendencies returned from ndrop_bam_run, so just init ptend here
      call physics_ptend_init(ptend, state%psetcols, 'none')

      do k = top_lev, pver
         do i = 1, ncol

            if (qc(i,k) >= qsmall) then

               ! get droplet activation rate

               call ndrop_bam_run( &
                  wsub(i,k), t(i,k), rho(i,k), naer2(i,k,:), naer_all, &
                  naer_all, maerosol(i,k,:),  &
                  dum2)
               dum = dum2
            else
               dum = 0._r8
            end if

            ! note: deltatin/2.  accounts for sub step in microphysics
            ! ***** This assumes two sub-steps in microphysics.  It's dangerous to 
            ! ***** make that assumption here.  Should move all coding related to 
            ! ***** microphysics substepping into the microphysics.
            npccn(i,k) = (dum - nc(i,k)/lcldm(i,k))/(deltatin/2._r8)*lcldm(i,k)
         end do
      end do

   end if


   !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
   ! Contact freezing  (-40<T<-3 C) (Young, 1974) with hooks into simulated dust
   ! estimate rndst and nanco for 4 dust bins here to pass to MG microphysics

   do k = top_lev, pver
      do i = 1, ncol

         if (t(i,k) < 269.15_r8) then

            if (clim_modal_aero) then

               ! For modal aerosols:
               !  use size '3' for dust coarse mode...
               !  scale by dust fraction in coarse mode
               
               dmc  = coarse_dust(i,k)
               ssmc = coarse_nacl(i,k)

               if ( separate_dust ) then
                  ! 7-mode -- has separate dust and seasalt mode types and no need for weighting 
                  wght = 1._r8
               else
                  ! 3-mode -- needs weighting for dust since dust and seasalt are combined in the "coarse" mode type
                  wght = dmc/(ssmc + dmc)
               endif

               if (dmc > 0.0_r8) then
                  nacon(i,k,3) = wght*num_coarse(i,k)*rho(i,k)
               else
                  nacon(i,k,3) = 0._r8
               end if

               !also redefine parameters based on size...

               rndst(i,k,3) = 0.5_r8*dgnumwet(i,k,mode_coarse_dst_idx)
               if (rndst(i,k,3) <= 0._r8) then 
                  rndst(i,k,3) = rn_dst3
               end if

            else

               !For Bulk Aerosols: set equal to aerosol number for dust for bins 2-4 (bin 1=0)

               if (idxdst2 > 0) then 
                  nacon(i,k,2) = naer2(i,k,idxdst2)
               end if
               if (idxdst3 > 0) then 
                  nacon(i,k,3) = naer2(i,k,idxdst3)
               end if
               if (idxdst4 > 0) then 
                  nacon(i,k,4) = naer2(i,k,idxdst4)
               end if
            end if

         end if
      end do
   end do

   !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
   !bulk aerosol ccn concentration (modal does it in ndrop, from dropmixnuc)

   if (.not. clim_modal_aero) then

      ! ccn concentration as diagnostic
      call ndrop_bam_ccn(lchnk, ncol, maerosol, naer2)

      deallocate( &
         naer2,    &
         maerosol)

   end if

end subroutine microp_aero_run

!===============================================================================


!===============================================================================

end module microp_aero

