
c=============================================================================
c
c $Id: drydep_update.F,v 1.23 2003/05/27 21:14:25 jrt Exp $
c
c CODE DEVELOPER
c   Dan Bergmann, LLNL
c   dbergmann@llnl.gov
c
c FILE
c   drydep_update.F
c
c ROUTINES
c   Update_Drydep
c   Depvel
c
c=============================================================================


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Update_Drydep
c
c DESCRIPTION
c   This routine updates const based on dry deposition using dry deposition
c   velocities calculated by the Harvard dry deposition model.
c
c ARGUMENTS
c   pr_dry_depos  : should dry depositions be written to periodic const
c                   output file?
c   lwi_flags  : array of flags that indicate land, water, or ice
c   ai         : pressure = (ai * pt) + (bi * psf), ai at zone interface
c   bi         : pressure = (ai * pt) + (bi * psf), bi at zone interface
c   am         : pressure = (am * pt) + (bm * psf), am at zone midpoint
c   bm         : pressure = (am * pt) + (bm * psf), bm at zone midpoint
c   mcor       : area of grid box (m^2)
c   latdeg     : latitude  (deg)
c   londeg     : longitude (deg)
c   humidity   : specific humidity (g/kg)
c   max_cloud  : maximum overlap cloud fraction for LW
c   radswg     : net downward shortwave radiation at ground (W/m^2)
c   ran_cloud  : random  overlap cloud fraction for LW
c   surf_air_temp : surface air temperature (degK)
c   surf_rough    : surface roughness (m)
c   ustar      : friction velocity (m/s)
c   mass       : total mass of the atmosphere within each grid box (kg)
c   psf        : surface pressure field at t1, known at zone centers (mb)
c   kel        : temperature (degK)
c   const      : species concentration, known at zone centers (mixing ratio)
c   dry_depos  : dry deposition accumulated since last output (kg/m^2)
c   diffaer    : aerosol diffusivity at bottom layer (m^2/s)
c   s_radius   : aerosol radius at bottom layer (m)
c   s_velocity : aerosol settling velocity at bottom layer (m/s)
c
c-----------------------------------------------------------------------------

      subroutine Update_Drydep
     &(tdt,mcor, latdeg, londeg,  max_cloud, ran_cloud, mass, const, 
     &radswg, surf_air_temp, surf_rough, ustar, lwi_flags,box_zm,box_zi,
     &diffaer, s_radius, s_velocity,dry_depos,drydep_vel,
     &ireg,iland,iuse,xlai)
             
      use h_imp_phys_constants
      use h_imp_time_constants
      use h_imp_dims, only: i1_gl, i2_gl,ju1_gl,j2_gl, ilo,ihi,julo,jhi,
     &k1,k2,i1,i2, ju1,j2, ilong, num_species
      use h_setkin_par
      use h_setkin_depos
      use h_imp_control,   only: nhms, nymd
      use h_setkin_mw,     only: mw => mw_data

      implicit none
 
!#     include "imp_phys_constants.h"
#     include "imp_emiss_constants.h"
!#     include "imp_time_constants.h"
!#     include "imp_dims.h"
!#     include "imp_control.h"
!#     include "imp_emiss_harvard.h"
!cc?   Tight coupling to setkin?
!#     include "setkin_par.h"
!#     include "setkin_depos.h"
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
!      logical :: pr_dry_depos
      integer :: lwi_flags (i1:i2, ju1:j2)
!      real*8  :: ai(k1-1:k2)
!      real*8  :: bi(k1-1:k2)
!      real*8  :: am(k1:k2)
!      real*8  :: bm(k1:k2)
      real*8  :: tdt
      real*8  :: mcor      (i1:i2, ju1:j2)
      real*8  :: latdeg    (ju1_gl:j2_gl)
      real*8  :: londeg    (i1_gl:i2_gl)
!      real*8  :: humidity  (i1:i2, ju1:j2, k1:k2)
      real*8  :: max_cloud (i1:i2, ju1:j2, k1:k2)
      real*8  :: radswg    (i1:i2, ju1:j2)
      real*8  :: ran_cloud (i1:i2, ju1:j2, k1:k2)
      real*8  :: surf_air_temp(i1:i2, ju1:j2)
      real*8  :: surf_rough(i1:i2, ju1:j2)
      real*8  :: ustar     (i1:i2, ju1:j2)
      real*8  :: mass      (i1:i2, ju1:j2, k1:k2)
!      real*8  :: psf       (ilo:ihi, julo:jhi)
!      real*8  :: kel       (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: const     (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: dry_depos (i1:i2, ju1:j2, 1:num_species)
      real*8  :: diffaer   (i1:i2, ju1:j2, 1:num_species)
      real*8  :: s_radius  (i1:i2, ju1:j2, 1:num_species)
      real*8  :: s_velocity(i1:i2, ju1:j2, 1:num_species)
      real*8  :: drydep_vel(i1:i2, ju1:j2, 1:num_species)
      real*8  :: box_zm     (i1:i2, ju1:j2)
      real*8  :: box_zi     (i1:i2, ju1:j2)
     
      integer :: ireg   (i1:i2, ju1:j2)
      integer :: iland  (i1:i2, ju1:j2, 1:NTYPE)
      integer :: iuse   (i1:i2, ju1:j2, 1:NTYPE)
      real*8  :: xlai   (i1:i2, ju1:j2, 1:NTYPE) !leaf area index of land type elements
c     -----------------------
c     Parameter declarations.
c     -----------------------
 
      real*8, parameter :: GOLDER_A(6) =
     &  (/ -0.096d0, -0.037d0, -0.002d0, 1.0d-5,   0.004d0,  0.035d0 /)

      real*8, parameter :: GOLDER_B(6) =
     &  (/  0.029d0,  0.029d0,  0.018d0, 0.000d0, -0.018d0, -0.036d0 /)


c     ----------------------
c     Function declarations.
c     ----------------------

      logical :: Is_Fixed_Const

 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      logical, save :: first = .true.

      integer :: ic, il, ij
      integer :: nsec_jan1

      integer :: pasquill(i1:i2)

      real*8  :: days
      real*8  :: decl
      real*8  :: loc_depos
      real*8  :: rdistsq
      real*8  :: time

      real*8  :: box_height(i1:i2)
      real*8  :: cloud_frac(i1:i2)
      real*8  :: csza      (i1:i2)
      real*8  :: cz1       (i1:i2)
      real*8  :: obk       (i1:i2)
      real*8  :: windspeed (i1:i2)

      real*8  :: diffa(i1:i2, num_species)
      real*8  :: dvel (i1:i2, num_species)
      real*8  :: s_ra (i1:i2, num_species)
      real*8  :: s_vel(i1:i2, num_species)
  
      integer :: chem_opt
     
c     ----------------
c     Begin execution.
c     ----------------

!      if (pr_diag) then
!        Write (6,*) 'Update_Drydep called by ', loc_proc
!      end if


!c     ==========
!      if (first) then
!c     ==========
!
!        first = .false.
!
!        if (pr_dry_depos) dry_depos(i1:i2,ju1:j2,1:num_species) = 0.0d0
!
!      end if
!      ireg (:,:)  = 1
!      iland(:,:,:)= 0
!      iland(:,:,1)= 15
!      iuse (:,:,:)= 0
!      iuse (:,:,1)= 1000
!      xlai (:,:,:)= 0.0d0
      
      pasquill(:) = 0

      csza(:)   = 0.0d0
      dvel(:,:) = 0.0d0


ccc!? For now just hardwire a few values for the radon/lead problem
c     (chem_opt = 1).  Setkin will eventually provide this information
c     for all species when doing a full chemistry calculation.
      chem_opt = 8 
      if (chem_opt == 1) then

        aerosol(1) = 0
        aerosol(2) = 1
  
        hstar(1)   = 9.3d-3
        hstar(2)   = 0.0d0
  
        oxidize(1) = 0.0d0
        oxidize(2) = 0.0d0

      else if (chem_opt == 6) then

        aerosol(1) = 15
        aerosol(2) = 15

        hstar(1)   = 0.0d0
        hstar(2)   = 0.0d0

        oxidize(1) = 0.0d0
        oxidize(2) = 0.0d0

      end if

      
c     =================
      call Get_Sec_Jan1
c     =================
     &  (nsec_jan1, nymd, nhms)

      time = nsec_jan1
      days = time / SECPDY

      
c     ============
      call Solrpos
c     ============
     &  (days, decl, rdistsq)


c     ================
      do ij = ju1, j2
c     ================

c       -----------------------------------------------------------
c       Put reasonable values into met fields for testing purposes.
c       -----------------------------------------------------------

!        do il = i1, i2
!
!          cz1(il) =
!     &      29.3d0 * kel(il,ij,k1) *
!     &      (1.0d0 + 0.61d-3*humidity(il,ij,k1)) *
!     &      Log (psf(il,ij) / (am(k1)*pt + bm(k1)*psf(il,ij)))
!
!          box_height(il) =
!     &      29.3d0 * kel(il,ij,k1) *
!     &      (1.0d0 + 0.61d-3*humidity(il,ij,k1)) *
!     &      Log (psf(il,ij) / (ai(k1)*pt + bi(k1)*psf(il,ij)))
!
!        end do
        cz1(:) = box_zm(:,ij)
        box_height(:) = box_zi(:,ij)
        
        
c       -----------------------------------------------------
c       Calculate solar zenith angle at this latitude for all
c       longitudes.
c       -----------------------------------------------------
        
c       ===========
        call Solrza 
c       ===========
     &    (days, decl, latdeg(ij), londeg(i1), ilong, csza)


c       -------------------------------------------------------
c       Calculate the Monin-Obukhov length by first calculating
c       the Pasquill stability class (A-F = 1-6) and then use 
c       Golder's formula (see Seinfeld pp 509-511 1986).
c       -------------------------------------------------------

        windspeed(:) =
     &    (ustar(i1:i2,ij) / 0.4d0) *
     &    Log (10.0d0 / surf_rough(i1:i2,ij))


cc?     Flint does not seem to like the following equation; it thinks that 
c       ran_cloud is unused.
        cloud_frac(:) =
     &    1.0d0 -
     &    ((1.0d0 - Maxval (max_cloud(:,ij,:), dim=2)) *
     &     (Product ((1.0d0 - ran_cloud(:,ij,:)), dim=2)))

cc?     Possible replacement for statement above, to appease flint
cc      do il = i1, i2
cc        max_mc  = Maxval (max_cloud(il,ij,:))
cc        prod_rc = 1.0d0
cc        do ik = k1, k2
cc          prod_rc = prod_rc * (1.0d0 - ran_cloud(il,ij,ik))
cc        end do
cc        cloud_frac(il) = 1.0d0 - (1.0d0 - max_mc) * prod_rc
cc      end do


        do il = i1, i2
 
          if (csza(il) < -0.05d0) then
            if (cloud_frac(il) > 0.4375001d0) then
              if      (windspeed(il) < 2.0d0) then
                pasquill(il) = 6
              else if (windspeed(il) < 3.0d0)  then
                pasquill(il) = 5
              else
                pasquill(il) = 4
              end if
            else
              if      (windspeed(il) < 3.0d0) then
                pasquill(il) = 6
              else if (windspeed(il) < 5.0d0) then
                pasquill(il) = 5
              else
                pasquill(il) = 4
              end if
            end if
          else 
            if (radswg(il,ij) > 700.0d0) then
              if      (windspeed(il) < 2.5d0) then
                pasquill(il) = 1
              else if (windspeed(il) < 5.0d0) then
                pasquill(il) = 2
              else
                pasquill(il) = 3
              end if
            else if (radswg(il,ij) < 350.0d0) then
              if      (windspeed(il) < 2.0d0) then
                pasquill(il) = 2
              else if (windspeed(il) < 5.0d0) then
                pasquill(il) = 3
              else
                pasquill(il) = 4
              end if
            else
              if      (windspeed(il) < 1.0d0) then
                pasquill(il) = 1
              else if (windspeed(il) < 4.0d0) then
                pasquill(il) = 2
              else if (windspeed(il) < 5.5d0) then
                pasquill(il) = 3
              else
                pasquill(il) = 4
              end if
            end if
          end if

        end do


        obk(:) =
     &    1.0d0 /
     &    (GOLDER_A(pasquill) +
     &     (GOLDER_B(pasquill) * Log (surf_rough(i1:i2,ij))))
 

c       --------------------------------------------------------
c       Calculate deposition velocities at this latitude for all
c       longitudes and all species.
c       --------------------------------------------------------

        do il = i1, i2
          do ic = 1, num_species
            diffa(il,ic) = diffaer   (il,ij,ic)
            s_ra (il,ic) = s_radius  (il,ij,ic)
            s_vel(il,ic) = s_velocity(il,ij,ic)
          end do
        end do


c       ===========
        call Depvel
c       ===========
     &   (ilong, ij, radswg(:,ij), surf_air_temp(:,ij), csza, oxidize,
     &    hstar, mw, aerosol, ustar(:,ij), cz1, obk, cloud_frac,
     &    lwi_flags(i1,ij), dvel, ireg, iland, iuse, xlai, diffa,
     &    s_ra, s_vel, delh_298_over_r)


cc?     Note that the two inner loops below used to be a single loop,
c       but the single loop version caused the code to crash on
c       pengra when the loop was vectorized.
        icloop: do ic = 1, num_species

c         ====================================
          if (Is_Fixed_Const(ic)) cycle icloop
c         ====================================

          do il = i1, i2

c           --------------------------------------------------------
c           Subtract mixing ratio amount from existing mixing ratio
c           of const.
c           --------------------------------------------------------

            loc_depos =
     &         const(il,ij,k1,ic) *
     &         (1.0d0 - Exp (-dvel(il,ic) * tdt / box_height(il)))

            const(il,ij,k1,ic) = const(il,ij,k1,ic) - loc_depos

c           ------------------------------------------------------
c           Accumulate the total dry deposited in units of kg/m^2.
c           ------------------------------------------------------

!            if (pr_dry_depos) then

              dry_depos(il,ij,ic) =
     &          dry_depos(il,ij,ic) +
     &          loc_depos * (mass(il,ij,k1) / mcor(il,ij)) *
     &          mw(ic) / MWTAIR

!            end if

          end do

c         if (pr_dry_depos) then

c           ------------------------------------------------------
c           Accumulate the total dry deposited in units of kg/m^2.
c           ------------------------------------------------------

c           do il = i1, i2
c             loc_depos =
c    &           const(il,ij,k1,ic) *
c    &           (1.0d0 - Exp (-dvel(il,ic) * tdt / box_height(il)))

c             dry_depos(il,ij,ic) = 
c    &          dry_depos(il,ij,ic) + 
c    &          loc_depos * (mass(il,ij,k1) / mcor(il,ij)) *
c    &          mw(ic) / MWTAIR
c           end do

c         end if

        end do icloop
        
        drydep_vel(:, ij, :) = dvel(:,:)
c     ======
      end do
c     ======


      return
 
      end
 

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Depvel
c
c DESCRIPTION
c   This routine computes the dry deposition velocities using a
c   resistance-in-series model.
c
c   Routine reads data which:
c     - converts land type id to deposition surface type id
c     - gives roughness heights for each land type id
c     - identifies water land type id's, for stability and z0 calculations
c     - reads surface resistance data for each deposition surface type id
c
c   Changes from version 3.1 to version 3.2:
c     * In unstable atmospheres with |zlmo| < zo, as can happen occasionally
c       under very low wind conditions with tall canopies, application of
c       Monin-Obukhov similarity yields negative values for ra.  This was a
c       problem in version 3.1.  In fact, Monin-Obukhov similarity does not
c       apply under such conditions, so we now set ra to zero and let the
c       boundary resistance rb define the overall aerodynamic resistance.
c       Since rb varies inversely with u*, it will impose a large aerodynamic
c       resistance under very low wind conditions.
c     * The range of applicability of stability correction functions to
c       Monin-Obukhov similarity has been extended to -2.5 < z/zmo < 1.5,
c       based on Figure 2 of Businger et al. [1971].  The range used to be
c       -1 < z/zmo < 1 in version 3.1.
c                                  
c  Literature cited:
c     Baldocchi, D.D., B.B. Hicks, and P. Camara, A canopy stomatal
c       resistance model for gaseous deposition to vegetated surfaces,
c       Atmos. Environ. 21, 91-101, 1987.
c     Brutsaert, W., Evaporation into the Atmosphere, Reidel, 1982.
c     Businger, J.A., et al., Flux-profile relationships in the atmospheric
c       surface layer, J. Atmos. Sci., 28, 181-189, 1971.
c     Dwight, H.B., Tables of integrals and other mathematical data,
c       MacMillan, 1957.
c     Guenther, A., and 15 others, A global model of natural volatile
c       organic compound emissions, J. Geophys. Res., 100, 8873-8892, 1995.
c     Hicks, B.B., and P.S. Liss, Transfer of SO2 and other reactive
c       gases across the air-sea interface, Tellus, 28, 348-354, 1976.
c     Jacob, D.J., and S.C. Wofsy, Budgets of reactive nitrogen,
c       hydrocarbons, and ozone over the Amazon forest during the wet season,
c       J.  Geophys. Res., 95, 16737-16754, 1990.
c     Jacob, D.J., and 9 others, Deposition of ozone to tundra,
c       J. Geophys. Res., 97, 16473-16479, 1992.
c     Levine, I.N., Physical Chemistry, 3rd ed., McGraw-Hill, New York, 1988.
c     Munger, J.W., and 8 others, Atmospheric deposition of reactive
c       nitrogen oxides and ozone in a temperate deciduous forest and a
c       sub-arctic woodland, J. Geophys. Res., in press, 1996.
c     Walcek, C.J., R.A. Brost, J.S. Chang, and M.L. Wesely, SO2, sulfate,
c       and HNO3 deposition velocities computed using regional landuse and
c       meteorological data, Atmos. Environ., 20, 949-964, 1986.
c     Wang, Y.H., paper in preparation, 1996.
c     Wesely, M.L, Improved parameterizations for surface resistance to
c       gaseous dry deposition in regional-scale numerical models, 
c       Environmental Protection Agency Report EPA/600/3-88/025,
c       Research Triangle Park (NC), 1988.
c     Wesely, M.L., same title, Atmos. Environ., 23, 1293-1304, 1989.
c
c ARGUMENTS
c   npts    : tbd
c   ij      : tbd
c   radiat  : solar radiation (W*m^-2)
c   tempk   : surface air temperature (degK)
c   suncos  : cosine of solar zenith angle
c   f0      : reactivity factor for oxidation of biological substances
c   hstar   : Henry's Law constant
c   xmw     : molecular weights; used to calculate molecular diffusivities
c             (kg/mol)
c   airosol : 0 => gas-phase species, 1 => aerosol species
c   ustar   : friction velocity (m/s)
c   cz1     : altitude at which deposition velocity is computed (m)
c   obk     : Monin-Obukhov length, set to 1.0d5 under neutral conditions (m)
c   cfrac   : fractional cloud cover
c   lsnow   : integer for snow and sea ice (1=>water, 2=>land, 3=>ice)
c   dvel    : deposition velocities (m/s)
c   ireg    : # of landtypes in grid square
c   iland   : land type id for elements ldt = 1, ireg;
c             could be from any source, mapped to deposition surface id
c   iuse    : fraction of gridbox area occupied by land type elements (mil^-1)
c   xlai    : leaf area index of land type elements
c   diffa   : aerosol diffusivity (m^2/s)
c   s_ra    : aerosol radius (m)
c   s_vel   : aerosol settling velocity (m/s)
c   delh_298_over_r : temperature scaling term for Henry's law
c
c-----------------------------------------------------------------------------

      subroutine Depvel
     &  (npts, ij, radiat, tempk, suncos, f0, hstar, xmw, airosol,
     &   ustar, cz1, obk, cfrac, lsnow, dvel, ireg, iland, iuse, xlai,
     &   diffa, s_ra, s_vel, delh_298_over_r)
     
      use h_imp_phys_constants
      use h_imp_dims, only: i1,i2, ju1,j2, num_species
      
      implicit none

!#     include "imp_phys_constants.h"
#     include "imp_emiss_constants.h"
!#     include "imp_dims.h"
#     include "imp_drydep_data.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: npts
      integer :: ij
      real*8  :: radiat (npts)
      real*8  :: tempk  (npts)
      real*8  :: suncos (npts)
      real*8  :: f0     (num_species)
      real*8  :: hstar  (num_species)
      real*8  :: xmw    (num_species)
      integer :: airosol(num_species)
      real*8  :: ustar  (npts)
      real*8  :: cz1    (npts)
      real*8  :: obk    (npts)
      real*8  :: cfrac  (npts)
      integer :: lsnow  (npts)
      real*8  :: dvel   (npts, num_species)
      integer :: ireg   (i1:i2, ju1:j2)
      integer :: iland  (i1:i2, ju1:j2, 1:NTYPE)
      integer :: iuse   (i1:i2, ju1:j2, 1:NTYPE)
      real*8  :: xlai   (i1:i2, ju1:j2, 1:NTYPE)
      real*8  :: diffa  (npts, num_species)
      real*8  :: s_ra   (npts, num_species)
      real*8  :: s_vel  (npts, num_species)
      real*8  :: delh_298_over_r(num_species)


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter :: XCKMAN = 0.4d0


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8  :: Diffg


c     ----------------------
c     Variable declarations.
c     ----------------------

      logical :: is_found

      logical :: ldep(num_species)

      logical :: lrgera(npts)  ! T -> stable atmosphere; a high aerodynamic
                               ! resistance (ra = 1.0d4 m/s) is imposed; else
                               ! ra is calculated

      integer :: idep1
      integer :: ijloop
      integer :: iolson
      integer :: iw
      integer :: k
      integer :: ldt
      integer :: nwaterx

      real*8  :: c1
      real*8  :: ckustr
      real*8  :: corr1
      real*8  :: cz       ! altitude where deposition velocity is computed (m)
      real*8  :: dair
      real*8  :: dummy1, dummy2, dummy3, dummy4
      real*8  :: ra, rb
      real*8  :: rdc
      real*8  :: reyno
      real*8  :: schno
      real*8  :: stono
      real*8  :: ebr, eim, ein, r1
      real*8  :: rix
      real*8  :: rt
      real*8  :: tempc1   ! surface air temperatures in degC
      real*8  :: tempk1   ! surface air temperatures in degK
      real*8  :: xnu
      real*8  :: z0obk

      real*8, save :: press = 1.5d5

      real*8  :: c1x (num_species)  ! total resistance to deposition for
                                    ! each species (s/m)
      real*8  :: vd  (num_species)
      real*8  :: vk  (num_species)              

      real*8  :: henry_eff(num_species)

      real*8  :: rac (NTYPE)
      real*8  :: rclo(NTYPE)
      real*8  :: rcls(NTYPE)
      real*8  :: rgso(NTYPE)
      real*8  :: rgss(NTYPE)
      real*8  :: ri  (NTYPE)
      real*8  :: rlu (NTYPE)

      real*8  :: zo  (NTYPE)  ! roughness height for specific surface types (m)

      real*8  :: rsurfc(num_species, NTYPE)  ! bulk surface resistance for
                                             ! each species (s/m)


c     ----------------
c     Begin execution.
c     ----------------

c     --------------------------------------------------
c     If ldep(k)=F, species does not deposit.
c     Deposition is applied only to species with ldep=T.
c     --------------------------------------------------

      do k = 1, num_species
        ldep(k) =
     &    ((hstar(k) > 0.0d0) .or.
     &     (f0(k)    > 0.0d0) .or.
     &     (airosol(k) /=0))
      end do


      do k = 1, num_species 
        do ijloop = 1, npts
          dvel(ijloop,k) = 0.0d0
        end do
      end do


c     ============================
      IJLOOPX: do ijloop = 1, npts
c     ============================

c       ------------------------------
c       Compute deposition velocities.
c       ------------------------------

        cz = cz1(ijloop)

        tempk1 = tempk(ijloop)
        tempc1 = tempk1 + ABS_ZERO


        do k = 1, num_species
          vd(k) = 0.0d0
          do ldt = 1, NTYPE
           rsurfc(k,ldt) = 0.0d0
          end do
        end do


c       ---------------------------------------------------------------------
c       Calculate the kinematic viscosity xnu (m^2/s) of air as a function of
c       temperature.  The kinematic viscosity is used to calculate the
c       roughness heights over water surfaces and to diagnose whether such
c       surfaces are aerodynamically rough or smooth using a Reynolds number
c       criterion.  The expression for the temperature dependence of xnu is
c       from the Fortran code in Appendix II of Wesely [1988].
c       ---------------------------------------------------------------------

        c1  = -tempk1 / ABS_ZERO

        xnu = 0.151d0 * (c1**1.77d0) * 1.0d-04


c       -----------------------------------------------------------------
c       Compute bulk surface resistance for gases.    
c
c       Adjust external surface resistances for temperature; 
c       from Wesely [1989], expression given in text on p. 1296.        
c
c       There is no evidence that the resistance continues to increase at
c       temperatures below -18 C, so at colder temperatures, hold the
c       resistance fixed.
c       -----------------------------------------------------------------

        rt = 1000.0d0 * Exp (-tempc1 - 4.0d0)

        if (tempc1 < -18.0d0) rt = 1.2d9


c       ------------------------------------------------------------------
c       Get surface resistances - loop over land types ldt.
c
c       The land types within each grid square are defined using the Olson
c       land type database.  Each of the Olson land types is assigned a 
c       corresponding "deposition land type" with characteristic values of
c       surface resistance components.  There are 74 Olson land-types, but
c       only 11 deposition land types (i.e., many of the Olson land types
c       share the same deposition characteristics).  Surface resistance
c       components for the "deposition land types" are from Wesely [1989],
c       except for tropical forests [Jacob and Wofsy, 1990] and for tundra
c       [Jacob et. al., 1992].  All surface resistance components are
c       normalized to a leaf area index of unity.
c 
c       Olson land types, deposition land types, and surface resistance
c       components are read from the dry deposition file; check that file
c       for further details.
c       ------------------------------------------------------------------

c       ==========================================
        LDTLOOP1: do ldt = 1, ireg(ijloop+i1-1,ij)
c       ==========================================

c                                            ==============
          if (iuse(ijloop+i1-1,ij,ldt) == 0) cycle LDTLOOP1
c                                            ==============

          if (lsnow(ijloop) == 3) then  ! snow or ice
            idep1  = 1
          else
            iolson = iland(ijloop+i1-1,ij,ldt) + 1
            idep1  = IDEP(iolson)
          end if


c         =======================
          call Surface_Resistance
c         =======================
     &      (idep1, rac(ldt), rclo(ldt), rcls(ldt), rgso(ldt),
     &       rgss(ldt), ri(ldt), rlu(ldt), rt, tempc1,
     &       cfrac(ijloop), radiat(ijloop), suncos(ijloop),
     &       xlai(ijloop+i1-1,ij,ldt), rix)


c         ----------------------------------------------------------------
c         Compute aerodynamic resistance to lower elements in lower part
c         of the canopy or structure, assuming level terrain; equation (5)
c         of Wesely [1989].
c         ----------------------------------------------------------------

          rdc =
     &      100.0d0 *
     &      (1.0d0 + (1000.0d0 / (radiat(ijloop) + 10.0d0)))


c         =============================
          KLOOP1: do k = 1, num_species
c         =============================

            if (.not. ldep(k)) cycle KLOOP1

            if (airosol(k) == 0) then

              henry_eff(k) =
     &          hstar(k) *
     &          Exp (-delh_298_over_r(k) *
     &               ((1.0d0 / tempk1) -
     &                (1.0d0 / 298.0d0)))

c             -----------------------------------------------------
c             Species-dependent corrections to resistances are from
c             equations (6)-(9) of Wesely [1989].
c             -----------------------------------------------------

c             ======================
              call Canopy_Resistance
c             ======================
     &          (rdc, rix, press, tempk1, f0(k), henry_eff(k), xmw(k),
     &           rac(ldt), rclo(ldt), rcls(ldt), rgso(ldt), rgss(ldt),
     &           rlu(ldt), rsurfc(k,ldt))

            else

c             ------------------------------------
c             Get surface resistance for aerosols;
c             Eqs.(5)-(9) of Zhang et al. [2001]
c             ------------------------------------

              schno = xnu / diffa(ijloop,k)

              if (A_VEG(idep1) > 0.0d0) then
                stono = s_vel(ijloop,k) * ustar(ijloop) /
     &                  (IMP_G * A_VEG(idep1))

                ein = 2.0d0 * (s_ra(ijloop,k) / A_VEG(idep1)) ** 2.0d0
              else
                stono = s_vel(ijloop,k) * ustar(ijloop)**2.0d0 /
     &                  (IMP_G * xnu)
                ein = 0.0d0
              end if

              ebr = schno ** (-GAMA_BR(idep1))
 
              eim = (stono / (ALFA_IM(idep1) + stono)) ** 2.0d0

              if (IWET(idep1) > 0) then
                r1 = 1.0d0
              else
                r1 = Exp (-Sqrt (stono))
                if (r1 < 1.0d-10) r1 = 1.0d-10
              end if
 
              rsurfc(k,ldt) = 1.0d0 / (3.0d0 * ustar(ijloop) *
     &                        (ebr + eim + ein) * r1)

            end if

c           ----------------------------------------------------
c           Set min and max values for bulk surface resistances.
c           ----------------------------------------------------

            rsurfc(k,ldt) =
     &        Max (1.0d0, Min (rsurfc(k,ldt), 9999.0d0))


c         =============
          end do KLOOP1
c         =============

c       ===============
        end do LDTLOOP1
c       ===============


c       ==========================================
        LDTLOOP3: do ldt = 1, ireg(ijloop+i1-1,ij)
c       ==========================================

c         -------------------------------------------------------
c         Loop through the different landuse types present in the
c         grid square.
c         -------------------------------------------------------

c                                            ==============
          if (iuse(ijloop+i1-1,ij,ldt) == 0) cycle LDTLOOP3
c                                            ==============

          iolson = iland(ijloop+i1-1,ij,ldt) + 1


c         -------------------------------------------------------------
c         Get roughness heights; they are specified constants for each
c         surface type, except over water where zo = f(u*).  The latter
c         dependence is from equation (6) of Hicks and Liss [1976]. 
c         -------------------------------------------------------------

          nwaterx = NWATER

          is_found = .false.

          IWLOOP: do iw = 1, nwaterx

            if (iolson /= iwater(iw)) cycle IWLOOP

            zo(ldt) =
     &        1.4d-02 * ustar(ijloop) * ustar(ijloop) / 9.8d0 +
     &        1.1d-01 * xnu / ustar(ijloop)

            is_found = .true.

            exit IWLOOP

          end do IWLOOP

          if (.not. is_found) then
            zo(ldt) = izo(iolson)
            zo(ldt) = zo(ldt) * 1.0d-4
          end if


c         ------------------------------------------------------------------
c         Get aerodynamic resistances ra and rb.
c
c         The aerodynamic resistance ra is integrated from altitude z0+d up
c         to the altitude z1, at which the dry deposition velocity is to be
c         referenced.  The integration corrects for stability using
c         Monin-Obukhov similarity formulas from Businger et. al. [1971],
c         which apply over the range -2.5 < z/zMO < 1.5  (see their
c         Figure 2).  Under very unstable conditions when z1 > -2.5 zMO, we
c         assume that there is no resistance to transfer in the convective
c         column between zmo and z1.  Under very stable conditions when
c         z1 > 1.5 zMO, we assume that vertical transfer in the column
c         between zmo and z1 is strongly suppressed so that the deposition
c         velocity at altitude z1 is very low.  Under these conditions, we
c         just specify a very large ra=1.0d4 s/m (lrgera = T).
c
c         The Reynolds number reyno diagnoses whether a surface is
c         aerodynamically rough (reyno > 10) or smooth.
c         NOTE: The criterion "reyno > 10" is now replaced by "reyno > 1".

c         Surface is rough in all cases except over water with low wind
c         speeds.  In the smooth case, vertical transport IN THE SUBLAYER
c         near the surface is limited by molecular diffusion and is
c         therefore very slow; we assign a large value of ra + rb to
c         account for this effect. (djj, hyl, bmy, 5/8/00)
c
c         In the aerodynamically rough case, the expression for ra is as
c         given in equation (5) of Jacob et. al. [1992]:
c           ra = (1/ku*)*int(from z0 to z1) (phi(x)/z)dz
c         where x = (z-d)/zmo, z is the height above ground, and d is the
c         displacement height, which is typically 70-80% of the canopy
c         height [Brutsaert, 1982].  We change the vertical coordinate so
c         that z=0 at the displacement height; that's OK since for all
c         practical applications, z1 >> d.  In this manner, we don't need to
c         assume any specific value for the displacement height.  Applying
c         the variable transformation z -> x = z/zmo, the equation above
c         becomes:
c           ra = (1/ku*)*int(from x0 to x1) (phi(x)/x)dx   with x=z/zmo
c         Here phi is a stability correction function originally formulated
c         by Businger et. al. [1971] and given in eqns 5a and 5b of Jacob
c         et. al. [1992].
c         For unstable conditions:
c           phi(x) = a/sqrt(1-bx)  where a=0.74, b = 9
c         The analytical solution to the integral is [Dwight, 1957, integral
c         192.11]:
c           int(dx/(x*sqrt(1-bx))) = log(abs((sqrt(1-bx)-1)/(sqrt(1-bx)+1)))
c         which yields the expression for ra used in the code for unstable
c         conditions.
c         For stable conditions,
c           phi(x) = a + bx        where a=0.74, b = 4.7
c         and the analytical solution to the integral is:
c           int((a/x)+b)dx = a*ln(x) + bx
c         which yields the expression of ra used in the code for stable
c         conditions.
c
c         The formulation of rb for gases is equation (12) of Walcek et. al.
c         [1986].
c         ------------------------------------------------------------------

          ckustr = XCKMAN * ustar(ijloop)

          reyno = ustar(ijloop) * zo(ldt) / xnu 

          if (obk(ijloop) == 0.0d0) then

            Write (6,910) obk(ijloop), ijloop, ldt                

 910        format (1x, 'obk(ijloop) = ', e11.2, 1x, 'ijloop = ',
     &              i4, 1x, 'ldt = ', i3, /) 

          end if

          corr1 = cz / obk(ijloop)

          z0obk = zo(ldt) / obk(ijloop)

          lrgera(ijloop) = .false.

          if (corr1 > 0.0d0) then
            if (corr1 >   1.5d0) lrgera(ijloop) = .true.
          else if (corr1 <= 0.0d0) then
            if (corr1 <= -2.5d0) corr1 = -2.5d0
          end if

          if (ckustr == 0.0d0) then

            Write (6,920) ijloop, ckustr, XCKMAN, ustar(ijloop)

 920        format (1x, 'ijloop = ', i4, 1x, 'ckustr = ', e10.1, 1x,
     &              'XCKMAN = ', e12.4, 1x, 'ustar(ijloop) = ', e12.4)

cc?
            Close (98)
            
           stop "ckustr == 0.0d0 in  Depvel" 
          end if


c         ------------------------------------------------------------------
c         Aerodynamically rough or smooth surface.
c
c         In the classic study by Nikuradse (1933) the transition from
c         smooth to rough was examined in pipe flow.  He introduced a
c         roughness Reynolds number rr = u* z0 / nu, and found the flow to
c         be smooth for rr < 0.13, and rough for rr > 2.5, with a transition
c         regime in between (E.B. Kraus and J.A. Businger, Atmosphere-Ocean
c         Interaction, second edition, p.144-145, 1994).  Similar statements
c         can be found in the books:
c           Evaporation into the Atmosphere, by Wilfried Brutsaert, p.59,89,
c             1982; or
c           Seinfeld & Pandis, p.858, 1998.
c         Here we assume a sudden transition point rr = 1 from smooth to
c         rough, following L. Merlivat (1978, The dependence of bulk
c         evaporation coefficients on air-water interfacial conditions as
c         determined by the isotopic method, J. Geophys. Res.,
c         Oceans & Atmos., 83, C6, 2977-2980).  Also refer to Brutsaert's
c         book, p.125.  we used to use the criterion "reyno > 10" for
c         aerodynamically rough surface and now change to "reyno > 1".
c         (hyl, 10/15/99)
c         ------------------------------------------------------------------

c         ===================
          if (reyno >= 1.0d0) then
c         ===================

c           ------------------------------
c           Aerodynamically rough surface.
c           ------------------------------

            if ((corr1 <= 0.0d0) .and. (z0obk < -1.0d0)) then

c             ----------------------------------------
c             Unstable condition with Abs (zlmo) < Z0;
c             set ra to zero (version 3.2).
c             ----------------------------------------

              ra = 0.0d0

            else if ((corr1 <= 0.0d0) .and. (z0obk >= -1.0d0)) then

c             ---------------------------------------------------
c             Unstable conditions; compute ra as described above.
c             ---------------------------------------------------

              dummy1 = (1.0d0 - (9.0d0 * corr1))**0.5d0
              dummy2 = (1.0d0 - (9.0d0 * z0obk))**0.5d0
              dummy3 = Abs ((dummy1 - 1.0d0) / (dummy1 + 1.0d0))
              dummy4 = Abs ((dummy2 - 1.0d0) / (dummy2 + 1.0d0))

              ra = 0.74d0* (1.0d0 / ckustr) * Log (dummy3 / dummy4)
                                 
            else if ((corr1 > 0.0d0) .and.
     &               (.not. lrgera(ijloop))) then

c             -----------------------------------------
c             Moderately stable conditions (z/zmo < 1);
c             compute ra as described above
c             -----------------------------------------

              ra =
     &          (1.0d0 / ckustr) *
     &          (0.74d0 * Log (corr1 / z0obk) +
     &           4.7d0  * (corr1 - z0obk))

            else if (lrgera(ijloop)) then

c             -----------------------
c             Very stable conditions.
c             -----------------------

              ra = 1.0d+04

            end if


            if (ra < 0.0d0) then

c             ------------------------------------------------------------
c             If ra is negative (as occasionally happened in version 3.1),
c             send a warning message.
c             ------------------------------------------------------------

              Write (6,930) ra, cz, zo(ldt), obk(ijloop)

 930          format ('WARNING:  ra < 0 in Depvel', 4(1x, e12.5))

            end if


            KLOOP3: do k = 1, num_species

c             ------------------------------------
c             Get total resistance for deposition.
c             ------------------------------------

              if (.not. ldep(k)) cycle KLOOP3

              if (airosol(k) == 0) then

c               ------------------------------------------------------------
c               dair is the thermal diffusivity of air; value of
c               0.2*1.E-4 m^2/s, cited on p. 16,476 of Jacob et. al. [1992].
c               ------------------------------------------------------------

                dair = 0.2d0 * 1.0d-4

                rb = (2.0d0 / ckustr) *
     &            (dair / Diffg (tempk1, press, xmw(k)*KGPG))**0.667d0
                c1x(k) = ra + rb + rsurfc(k,ldt)

              else

                c1x(k) = ra + rsurfc(k,ldt)

              end if

            end do KLOOP3

c         ====
          else
c         ====

c           -------------------------------
c           Aerodynamically smooth surface.
c           -------------------------------

            KLOOP4: do k = 1, num_species

              if (.not. ldep(k)) cycle KLOOP4

c             --------------------------------------------------------------
c             Suppress drydep over smooth surfaces by setting ra to a
c             large value.  This prevents negative dry deposition
c             velocities when ustar is very small (djj, bmy, 5/8/00)
c             --------------------------------------------------------------

              ra = 1.0d4

              c1x(k) = ra + rsurfc(k,ldt)

            end do KLOOP4

c         ======
          end if
c         ======


c         ----------------------------------------------------------------
c         iuse is the fraction of the grid square occupied by surface ldt
c         in units of mil^-1 (iuse=500 => 50% of the grid square).  Add
c         the contribution of surface type ldt to the deposition velocity;
c         this is a loop over all surface types in the gridbox.
c         ----------------------------------------------------------------

          KLOOP5: do k = 1, num_species

            if (.not. ldep(k)) cycle KLOOP5

            vk(k) = vd(k)
            vd(k) = iuse(ijloop+i1-1,ij,ldt)

            if (airosol(k) == 0) then
              vd(k) = vk(k) + (0.001d0 * vd(k) / c1x(k))
            else
              vd(k) = vk(k) + (0.001d0 * vd(k) * (1./c1x(k) +
     &          s_vel(ijloop,k)))
            end if

          end do KLOOP5


c       ===============
        end do LDTLOOP3
c       ===============


        KLOOP6: do k = 1, num_species

          if (.not. ldep(k)) cycle KLOOP6

          dvel(ijloop,k) = vd(k)

        end do KLOOP6


c     ==============
      end do IJLOOPX
c     ==============


      return                       

      end                          

