
c=============================================================================
c
c $Id: emiss_utils.F,v 1.4 2003/04/02 17:43:31 jrt Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c
c   Original code from:
c     Harvard tropospheric emissions module for 3D applications;
c       by Yuhang Wang, Gerry Gardner, and Prof. Daniel Jacob
c       of Harvard University (Release V1.0)
c
c FILE
c   emiss_utils.F
c
c ROUTINES
c   Biofit
c   Sun_Param
c   Diffg
c
c=============================================================================


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Biofit
c
c DESCRIPTION
c   This routine calculates the light correction.
c
c   Light adjustment by the function Biofit is described by Wang [1996].
c   It combines:
c     * local dependence of stomal resistance on the intensity I of light
c       impinging the leaf; this is expressed as a mutliplicative factor
c       I/(I+b) to the stomatal resistance where b = 50 (W*m^-2)
c       (equation (7) of Baldocchi et al. [1987])
c     * radiative transfer of direct and diffuse radiation in the canopy
c       using equations (12)-(16) from Guenther et al. [1995]
c     * separate accounting of sunlit and shaded leaves using equation (12)
c       of Guenther et al. [1995]
c     * partitioning of the radiation at the top of the canopy into direct
c       and diffuse components using a parameterization of results from an
c       atmospheric radiative transfer model [Wang, 1996].
c
c ARGUMENTS
c   cloud_frac1 : fractional cloud cover
c   suncos1     : cosine of the solar zenith angle
c   xlai1       : leaf area index of land type for current month
c   coeff       : factor that integrates the light dependence over the canopy
c                 depth; "sp" even though "ri" is input per unit area of leaf
c
c-----------------------------------------------------------------------------

      function Biofit
     &  (cloud_frac1, suncos1, xlai1, coeff)

      use  h_imp_phys_constants
      implicit none

!#     include "imp_phys_constants.h"
#     include "imp_emiss_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      real*8  :: cloud_frac1
      real*8  :: suncos1
      real*8  :: xlai1
      real*8  :: coeff(NPOLY)


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8  :: Biofit


c     -----------------------
c     Parameter declarations.
c     -----------------------

      integer, parameter ::
     &  BIOFIT_DIM = 4   ! Biofit dimension


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: ii, nn
      integer :: k0, k1, k2, k3
      integer :: spdim

      real*8  :: bigterm(NPOLY)

      real*8  :: spterm (BIOFIT_DIM-1)

      real*8  :: term   (BIOFIT_DIM)


c     ----------------
c     Begin execution.
c     ----------------

      term(1) = 1.0d0
      term(2) = xlai1
      term(3) = suncos1
      term(4) = cloud_frac1


      spdim = BIOFIT_DIM - 1


      do ii = 1, spdim
        spterm(ii) = term(ii+1)
      end do


c     ==============
      call Sun_Param
c     ==============
     &  (spdim, spterm)


      do ii = 1, spdim
        term(ii+1) = spterm(ii)
      end do


      k0 = 0

      do k3 = 1, BIOFIT_DIM
        do k2 = k3, BIOFIT_DIM
          do k1 = k2, BIOFIT_DIM

            k0 = k0 + 1

            bigterm(k0) = term(k1) * term(k2) * term(k3)

          end do
        end do
      end do


      Biofit = 0.0d0

      do nn = 1, NPOLY

        Biofit = Biofit + (coeff(nn) * bigterm(nn))

      end do

      Biofit = Max (Biofit, 0.1d0)


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Sun_Param
c
c DESCRIPTION
c   This routine scales and constrains xlai, suncos, and cloud_frac; called
c   by Biofit.
c
c ARGUMENTS
c   spdim  : spterm dimension
c   spterm : array of terms containing xlai, suncos, cloud_frac
c
c-----------------------------------------------------------------------------

      subroutine Sun_Param
     &  (spdim, spterm)
      
      implicit none

!#     include "imp_dims.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: spdim
      real*8  :: spterm(spdim)


c     -----------------------
c     Parameter declarations.
c     -----------------------

      integer, parameter ::
     &  SPDIM_PARAM = 3

      integer, parameter ::
     &  ND (SPDIM_PARAM) =     ! scaling factor for each variable
     &    (/ 55, 20, 11 /)

      real*8,  parameter ::
     &  XHI(SPDIM_PARAM) =     ! maximum for each variable
     &    (/ 11.0d0, 1.0d0, 1.0d0 /)


c     ----------------------
c     Variable declarations.
c     ----------------------

      character (len=75) :: err_msg

      integer :: ii

      real*8  :: rnd
      real*8  :: xlow          ! minimum for each variable


c     ----------------
c     Begin execution.
c     ----------------

      if (spdim /= SPDIM_PARAM) then
        err_msg = 'spdim/SPDIM_PARAM problem in Sun_Param.'
!        call Do_Err_Out
!     &    (err_msg, .true., 2, spdim, SPDIM_PARAM, 0, 0.0d0, 0.0d0)
        stop "spdim/SPDIM_PARAM problem in Sun_Param"
      end if


      do ii = 1, spdim

        spterm(ii) = Min (spterm(ii), XHI(ii))

        if (ii /= spdim) then

          rnd  = ND (ii)

          xlow = XHI(ii) / rnd

        else

          xlow = 0.0d0

        end if

        spterm(ii) = Max (spterm(ii), xlow)

        spterm(ii) = spterm(ii) / XHI(ii)

      end do


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Diffg
c
c DESCRIPTION
c   This routine calculates the molecular diffusivity of a gas in air
c   (m^2/s).
c
c   The molecular radius of air is given in a table on p. 479 of Levine
c   [1988]; the table also gives radii for some other molecules.  Rather
c   than using a specific molecular radius a generic value is used for all
c   molecules, which is good enough in terms of calculating the diffusivity
c   as long as the molecule is not too big.
c
c ARGUMENTS
c   tk    : temperature (degK)
c   press : pressure    (Pa)
c   xm    : molecular weight of gas (kg)
c
c-----------------------------------------------------------------------------

      function Diffg
     &  (tk, press, xm)
      use h_imp_phys_constants
      implicit none

!#     include "imp_phys_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      real*8  :: tk
      real*8  :: press
      real*8  :: xm


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8  :: Diffg


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter ::
     &  RADAIR = 1.2d-10,  ! hard-sphere molecular radii of air (m)
     &  RADX   = 1.5d-10   ! hard-sphere molecular radii of the diffusing
                           ! gas (m)


c     ----------------------
c     Variable declarations.
c     ----------------------

      real*8  :: airden
      real*8  :: diam
      real*8  :: frpath
      real*8  :: speed
      real*8  :: zz


c     ----------------
c     Begin execution.
c     ----------------

      airden = press * AVOGAD / (GAS_CONST_J * tk)


c     -------------------------------------------------------------------
c     Calculate the mean free path for gas X in air:  eq. 8.5 of Seinfeld
c     [1986]; diam is the collision diameter for gas X with air.
c     -------------------------------------------------------------------

      zz = xm / (MWTAIR * KGPG)

      diam = RADX + RADAIR

      frpath = 1.0d0 / (IMP_PI * Sqrt (1.0d0 + zz ) *
     &         airden * (diam * diam))


c     -------------------------------------------------------------
c     Calculate average speed of gas X; eq. 15.47 of Levine [1988].
c     -------------------------------------------------------------

      speed = Sqrt (8.0d0 * GAS_CONST_J * tk / (IMP_PI * xm))


c     --------------------------------------------------------------------
c     Calculate diffusion coefficient of gas X in air; eq. 8.9 of Seinfeld
c     [1986].
c     --------------------------------------------------------------------

      Diffg = (3.0d0 * IMP_PI / 32.0d0) *
     &        (1.0d0 + zz) * frpath * speed


      return

      end

