
c=============================================================================
c
c $Id: resistance.F,v 1.2 2003/02/28 23:52:13 jrt Exp $
c
c CODE DEVELOPER
c   Dan Bergmann, LLNL
c   dbergmann@llnl.gov
c
c FILE
c   resistance.F
c
c ROUTINES
c   Canopy_Resistance
c   Surface_Resistance
c
c=============================================================================


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Canopy_Resistance
c
c DESCRIPTION
c   This routine calculates bulk surface resistance of the canopy from the
c   network of resistances in parallel and in series.
c
c ARGUMENTS
c   rdc     : tbd
c   rix     :
c   press   :
c   tempk1  :
c   f01     :
c   hstar1  :
c   xmw1    :
c   rac1    :
c   rclo1   :
c   rcls1   :
c   rgso1   :
c   rgss1   :
c   rlu1    :
c   rsurfc1 :
c
c-----------------------------------------------------------------------------

      subroutine Canopy_Resistance
     &  (rdc, rix, press, tempk1, f01, hstar1, xmw1,
     &   rac1, rclo1, rcls1, rgso1, rgss1, rlu1, rsurfc1)
      use  h_imp_phys_constants
      implicit none

!#     include "imp_phys_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      real*8  :: rdc
      real*8  :: rix
      real*8  :: press
      real*8  :: tempk1
      real*8  :: f01
      real*8  :: hstar1
      real*8  :: xmw1
      real*8  :: rac1
      real*8  :: rclo1
      real*8  :: rcls1
      real*8  :: rgso1
      real*8  :: rgss1
      real*8  :: rlu1
      real*8  :: rsurfc1


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter ::
     &  BIG1 = 1.0d22


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8, external  :: Diffg


c     ----------------------
c     Variable declarations.
c     ----------------------

      real*8  :: dtmp1, dtmp2, dtmp3, dtmp4

      real*8  :: rclx
      real*8  :: rgsx
      real*8  :: rixx
      real*8  :: rluxx


c     ----------------
c     Begin execution.
c     ----------------

      rixx =
     &  rix *
     &  Diffg (tempk1, press, MWTH2O*KGPG) /
     &  Diffg (tempk1, press, xmw1  *KGPG) +
     &  1.0d0 /
     &  (hstar1 / 3000.0d0 + 100.0d0 * f01)


      if (rlu1 < 9999.0d0) then
        rluxx = rlu1 / (hstar1 / 1.0d+05 + f01)
      else
        rluxx = BIG1
      end if

c     ---------------------------------------------------------------------
c     To prevent virtually zero resistance to species with huge HSTAR, such
c     as HNO3, a minimum value of rluxx needs to be set.  The rationality
c     of the existence of such a minimum is demonstrated by the observed
c     relationship between Vd(NOy-NOx) and ustar in Munger et. al. [1996];
c     Vd(HNO3) never exceeds 2 cm*s^-1 in observations.  The corresponding
c     minimum resistance is 50 s*m^-1.  This correction was introduced by
c     J.Y. Liang on 7/9/95.
c     ---------------------------------------------------------------------

      rluxx = Max (rluxx, 50.0d0)


      rgsx =
     &  1.0d0 /
     &  (hstar1 / 1.0d+05 / rgss1 + f01 / rgso1)

      rclx =
     &  1.0d0 /
     &  (hstar1 / 1.0d+05 / rcls1 + f01 / rclo1)


c     -----------------------------------------------------------------------
c     Get the bulk surface resistance of the canopy, rsurfc, from the network
c     of resistances in parallel and in series (Fig. 1 of Wesely [1989]).
c     -----------------------------------------------------------------------

      dtmp1 = 1.0d0 / rixx
      dtmp2 = 1.0d0 / rluxx
      dtmp3 = 1.0d0 / (rac1 + rgsx)
      dtmp4 = 1.0d0 / (rdc + rclx)

      rsurfc1 = 1.0d0 / (dtmp1 + dtmp2 + dtmp3 + dtmp4)


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Surface_Resistace
c
c DESCRIPTION
c   This routine tbd
c
c ARGUMENTS
c   idep1   : tbd
c   rac1    :
c   rclo1   :
c   rcls1   :
c   rgso1   :
c   rgss1   :
c   ri1     :
c   rlu1    :
c   rt      :
c   tempc1  :
c   cfrac1  :
c   radiat1 :
c   suncos1 :
c   xlai1   :
c   rix     :
c
c-----------------------------------------------------------------------------

      subroutine Surface_Resistance
     &  (idep1, rac1, rclo1, rcls1, rgso1, rgss1, ri1, rlu1,
     &   rt, tempc1, cfrac1, radiat1, suncos1, xlai1, rix)

      implicit none

#     include "imp_emiss_constants.h"
#     include "imp_drydep_data.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: idep1
      real*8  :: rac1
      real*8  :: rclo1
      real*8  :: rcls1
      real*8  :: rgso1
      real*8  :: rgss1
      real*8  :: ri1
      real*8  :: rlu1
      real*8  :: rt
      real*8  :: tempc1
      real*8  :: cfrac1
      real*8  :: radiat1
      real*8  :: suncos1
      real*8  :: xlai1
      real*8  :: rix


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter ::
     &  BIG1 = 1.0d6

      real*8, parameter ::
     &  BIG2 = 1.0d22


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8, external  :: Biofit


c     ----------------------
c     Variable declarations.
c     ----------------------

      real*8  :: gfaci, gfact

      real*8  :: xdrycoeff(NPOLY)


c     ----------------
c     Begin execution.
c     ----------------

c     ----------------------------------------------------------------------
c     Read the internal resistance ri (minimum stomatal resistance for water
c     vapor, per unit area of leaf) from the IRI array; a "9999" value means
c     no deposition to stomata, so we impose a very large value for ri.
c     ----------------------------------------------------------------------

      ri1 = iri(idep1)

      if (ri1 >= 9999.0d0) ri1 = 1.0d22


c     ----------------------------------------------------------------------
c     Cuticular resistances IRLU read in from drydep table are per unit area
c     of leaf; divide them by the leaf area index to get a cuticular
c     resistance for the bulk canopy.  If IRLU is "9999", it means there are
c     no cuticular surfaces on which to deposit, so we impose a very large
c     value for rlu.
c     ----------------------------------------------------------------------

      if ((irlu(idep1) >= 9999) .or. (xlai1 <= 0.0d0)) then

        rlu1 = 1.0d6

      else

        rlu1 = irlu(idep1)
        rlu1 = (rlu1 / xlai1) + rt

      end if


c     ----------------------------------------------------------
c     The following are the remaining resistances for the Wesely
c     resistance-in-series model for a surface canopy
c     (see Atmos. Environ. paper, Fig.1).  
c     ----------------------------------------------------------

      rac1  = Max (irac(idep1), 1)
      if (rac1  >= 9999.0d0) rac1  = BIG2

      rgss1 = irgss(idep1)
      rgss1 = Max (rgss1+rt, 1.0d0)

      rgso1 = irgso(idep1)
      rgso1 = Max (rgso1+rt, 1.0d0)
      if (rgso1 >= 9999.0d0) rgso1 = BIG2

      rcls1 = ircls(idep1)
      rcls1 = rcls1 + rt
      if (rcls1 >= 9999.0d0) rcls1 = BIG2

      rclo1 = irclo(idep1)
      rclo1 = rclo1 + rt
      if (rclo1 >= 9999.0d0) rclo1 = BIG2


c     ----------------------------------------------------------------------
c     Adjust stomatal resistances for insolation and temperature =>
c
c     Temperature adjustment is from Wesely [1989], equation (3).
c       
c     Light adjustment by the function Biofit is described by Wang [1996].
c     It combines:
c       - local dependence of stomal resistance on the intensity I of light 
c         impinging the leaf; this is expressed as a mutliplicative 
c         factor I/(I+b) to the stomatal resistance where b = 50 W*m^-2
c         (equation (7) of Baldocchi et. al. [1987]);
c       - radiative transfer of direct and diffuse radiation in the 
c         canopy using equations (12)-(16) from Guenther et. al. [1995];
c       - separate accounting of sunlit and shaded leaves using
c         equation (12) of Guenther et. al. [1995];
c       - partitioning of the radiation at the top of the canopy into direct
c         and diffuse components using a parameterization to results from
c         an atmospheric radiative transfer model [Wang, 1996].
c     The dependent variables of the function Biofit are the leaf area
c     index (xylai), the cosine of zenith angle (suncos) and the fractional
c     cloud cover (cfrac).  The factor gfaci integrates the light
c     dependence over the canopy depth; sp even though ri is input per
c     unit area of leaf, it need not be scaled by lai to yield a bulk
c     canopy value because that is already done in the gfaci formulation.
c     ----------------------------------------------------------------------

      if (ri1 >= 9999.0d0) then

        rix = ri1

      else

        if ((tempc1 > 0.0d0) .and. (tempc1 < 40.0d0)) then
          gfact = 400.0d0 / tempc1 / (40.0d0 - tempc1)
        else
          gfact = 100.0d0
        end if

        if ((radiat1 > 1.0d-5) .and. (xlai1 > 0.0d0)) then
          xdrycoeff(:) = DRYCOEFF(:)
          gfaci = 1.0d0 /  Biofit (cfrac1, suncos1, xlai1, xdrycoeff)
        else
          gfaci = 100.0d0
        end if

        rix = ri1 * gfact * gfaci

      end if


      return

      end

