 
c=============================================================================
c
c $Id: grav_set_update.F,v 1.13 2003/02/28 23:52:13 jrt Exp $
c
c CODE DEVELOPER
c   Dan Bergmann, LLNL 
c   dbergmann@llnl.gov
c
c FILE
c   grav_set_update.F
c
c ROUTINES
c   Update_Grav_Settling
c
c=============================================================================
 
 
c-----------------------------------------------------------------------------
c
c ROUTINE
c   Update_Grav_Settling
c
c DESCRIPTION
c   This routine updates the gravitational settling of aerosols.
c
c   These different aerosol types having settling velocities:
c      1) sulfate
c      2) organic carbon from biomass burning
c      3) black   carbon from biomass burning
c      4) organic carbon from fossil fuel
c      5) black   carbon from fossil fuel
c      6) organic carbon from natural sources
c      7) sea salt 1    (sub-micron)
c      8) sea salt 2    (super-micron)
c      9) winter dust 1 (sub-micron)
c     10) winter dust 2 (super-micron)
c     11) summer dust 1 (sub-micron)
c     12) summer dust 2 (super-micron)
c     13) desert dust 1 (sub-micron)
c     14) desert dust 2 (super-micron)
c     15) stratospheric sulfate (monodispersed)
c
c ARGUMENTS
c   grid_height : height of each grid box (m)
c   relhum      : relative humidity (%)
c   mass        : total mass of the atmosphere within each grid box (kg)
c   press3e     : atmospheric pressure at the edge of each grid box (mb)
c   kel         : temperature (degK)
c   const       : species concentration, known at zone centers (mixing ratio)
c   diffaer     : aerosol diffusivity at bottom layer
c   s_radius    : aerosol radius at bottom layer (m)
c   s_velocity  : aerosol settling velocity at bottom layer (m/s)
c
c-----------------------------------------------------------------------------

      subroutine Update_Grav_Settling
     &  (tdt,grid_height, humidity, mass, press3e, kel, const, diffaer,
     &   s_radius, s_velocity)

      use h_imp_phys_constants
      use h_imp_dims
      use h_imp_aerosol

      use h_setkin_par
      use h_setkin_depos
      use h_umaerosol
       
      implicit none

!#     include "imp_phys_constants.h"
!#     include "imp_dims.h"
!#     include "imp_control.h"
!#     include "imp_aerosol.h"
!cc?   Tight coupling to setkin?
!#     include "setkin_par.h"
!#     include "setkin_depos.h"
!#     include "umaerosol.h"


c     ----------------------
c     Argument declarations.
c     ----------------------
      real*8  :: tdt
      real*8  :: grid_height(i1:i2, ju1:j2, k1:k2)
      real*8  :: humidity   (i1:i2, ju1:j2, k1:k2)
      real*8  :: mass       (i1:i2, ju1:j2, k1:k2)
      real*8  :: press3e    (ilo:ihi, julo:jhi, k1-1:k2)
      real*8  :: kel        (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: const      (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: diffaer    (i1:i2, ju1:j2, num_species)
      real*8  :: s_radius   (i1:i2, ju1:j2, num_species)
      real*8  :: s_velocity (i1:i2, ju1:j2, num_species)


c     ----------------------
c     Function declarations.
c     ----------------------

      logical, external :: Is_Fixed_Const


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: il, ij, ik, ic

      real*8  :: aer_den_swel(i1:i2, ju1:j2, k1+1:k2)
      real*8  :: c3t         (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: del_grid_box(i1:i2, ju1:j2, k1+1:k2)
      real*8  :: kele        (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: mfp         (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: radius      (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: relhume     (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: slip        (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: velocity    (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: viscosity   (i1:i2, ju1:j2, k1+1:k2)

cc    for adding umaerosol

      real*8, parameter :: CMPM3 = CMPM * CMPM * CMPM

      real*8  :: h2osat  (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: h2ogas  (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: so4mfrac(i1:i2, ju1:j2, k1+1:k2)
      real*8  :: so4dens (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: wetmas  (i1:i2, ju1:j2, k1+1:k2)
      real*8  :: so4aer  (i1:i2, ju1:j2, k1+1:k2, naer)
      real*8  :: so4radv (i1:i2, ju1:j2, k1+1:k2, nso4)
#ifndef NONON
      real*8  :: so4non  (i1:i2, ju1:j2, k1+1:k2, nnon)
      real*8  :: aernon  (i1:i2, ju1:j2, k1+1:k2, nnon)
      real*8  :: xnonum  (i1:i2, ju1:j2, k1+1:k2, nnon)
      real*8  :: so4vnon (i1:i2, ju1:j2, k1+1:k2, nnon)
#endif

c     ----------------
c     Begin execution.
c     ----------------
 
!      if (pr_diag) then
!        Write (6,*) 'Update_Grav_Settling called by ', loc_proc
!      end if


c     -------------------------------------------------------
c     Make sure aerosols are set correctly for radon/lead (1)
c     and beryllium (6) chemistry options.
c     -------------------------------------------------------

!      if (chem_opt == 1) then
!
!        aerosol(1) = 0
!        aerosol(2) = 1
!
!      else if (chem_opt == 6) then
!
!        aerosol(1) = 15
!        aerosol(2) = 15
!
!      end if


c     ------------------------------------------------------------
c     Outside of the species loop, calculate all aerodynamic terms
c     which are not dependent on physical characteristics of each
c     aerosol type.
c     ------------------------------------------------------------

c     ----------------------------------------
c     Calculate temperature at grid interface.
c     ----------------------------------------

      kele(:,:,:) = (kel(i1:i2,ju1:j2,k1:k2-1) + 
     &               kel(i1:i2,ju1:j2,k1+1:k2)) * 0.5d0


c     -----------------------------------------------------------------
c     First calculate relative humidity from Seinfeld (1986) p. 181.
c     The first  relhume is the temperature dependent parameter a.
c     The second relhume is the saturation vapor pressure of water.
c     The third  relhume is the actual relative humidity as a fraction.
c     Then make sure relhume is between 0 and 0.95 because swelling 
c     parameterization goes too large at very high relative humidity.
c     -----------------------------------------------------------------


      relhume(:,:,:) = 1.0d0 - (373.15d0 / kele(:,:,:))

      relhume(:,:,:) =
     &  1013.25d0 * Exp (13.3185d0 * relhume(:,:,:)    -
     &                    1.9760d0 * relhume(:,:,:)**2 -
     &                    0.6445d0 * relhume(:,:,:)**3 -
     &                    0.1299d0 * relhume(:,:,:)**4)

      relhume(:,:,:) =
     &  (humidity(:,:,1:k2-1) + humidity(:,:,2:k2)) * 
     &  0.5d0 * MWTAIR / 18.0d0 /
     &  GPKG * press3e(i1:i2,ju1:j2,k1:k2-1) / relhume(:,:,:)

      relhume(:,:,:) = Max (Min (relhume(:,:,:), 0.95d0), 1.0d-30)


c     -----------------------------------------
c     Viscosity (kg/m/s) (fit to CRC handbook).
c     -----------------------------------------

      viscosity(:,:,:) = 1.8d-5 * (kele(:,:,:) / 298.0d0)**0.85d0


c     ----------------------------------------------------
c     Mean free path (m) (Seinfeld + Pandis 1998, p. 455). 
c     ----------------------------------------------------

      mfp(:,:,:) = 
     &  2.0d0 * viscosity(:,:,:) /
     &  (PASPMB * press3e(i1:i2,ju1:j2,k1:k2-1) *
     &   Sqrt (8.0d0 * MWTAIR / 
     &         (IMP_PI * GAS_CONST_J * 1.0d3 * kele(:,:,:))))


cc    --------------------
cc    for adding umaerosol
cc    --------------------

cc    compute wet radius of so4 aerosol

      so4aer(:,:,:,1) = (const(:,:,k1:k2-1,ISO4M1)+
     &                   const(:,:,k1+1:k2,ISO4M1)) * 0.5d0
      so4aer(:,:,:,2) = (const(:,:,k1:k2-1,ISO4N1)+
     &                   const(:,:,k1+1:k2,ISO4N1)) * 0.5d0
      so4aer(:,:,:,3) = (const(:,:,k1:k2-1,ISO4M2)+
     &                   const(:,:,k1+1:k2,ISO4M2)) * 0.5d0
      so4aer(:,:,:,4) = (const(:,:,k1:k2-1,ISO4N2)+
     &                   const(:,:,k1+1:k2,ISO4N2)) * 0.5d0
      so4aer(:,:,:,5) = (const(:,:,k1:k2-1,ISO4M3)+
     &                   const(:,:,k1+1:k2,ISO4M3)) * 0.5d0
      so4aer(:,:,:,6) = (const(:,:,k1:k2-1,ISO4N3)+
     &                   const(:,:,k1+1:k2,ISO4N3)) * 0.5d0

      do il = i1, i2
        do ij = ju1, j2
          do ik = k1+1, k2

            h2osat(il,ij,ik) = h2osat_f(kele(il,ij,ik))
            h2ogas(il,ij,ik) = relhume(il,ij,ik)*h2osat(il,ij,ik)

            so4mfrac(il,ij,ik) = so4mfrac_f(kele(il,ij,ik),
     &                                      h2osat(il,ij,ik),
     &                                      h2ogas(il,ij,ik))
            so4dens(il,ij,ik) = so4dens_f(kele(il,ij,ik),
     &                                    so4mfrac(il,ij,ik))

          end do
        end do
      end do

      do iso4 = 1, nso4

        iso4n = iso4 * nmomso4
        iso4m = iso4n - 1

        wetmas(:,:,:) = max(r2so4min, so4aer(:,:,:,iso4m)
     &                / max(epsilo,   so4aer(:,:,:,iso4n)))
     &                / so4mfrac(:,:,:)

        so4radv(:,:,:,iso4) = (r3q * wetmas(:,:,:)
     &                      / (IMP_PI * so4dens(:,:,:)))**r1td
        so4radv(:,:,:,iso4) = max(so4radvmin,
     &                        min(so4radv(:,:,:,iso4),so4radvmax))

      end do

#ifndef NONON
      so4non(:,:,:,1:nnon) =
     &      (const(:,:,k1:k2-1,ISO4NOC:ISO4S4) +
     &       const(:,:,k1+1:k2,ISO4NOC:ISO4S4)) * 0.5d0
      aernon(:,:,:,1:nnon) =
     &      (const(:,:,k1:k2-1,INOC:ISSLT4) +
     &       const(:,:,k1+1:k2,INOC:ISSLT4)) * 0.5d0

cc    non-so4 aerosol radius (m) and mass (kg/particle)
      do inon = 1, nnon

        radvolm(:) = radgnon(:,inon)
     &             * exp(r3h*log(siggnon(:,inon))**2)
        radvnon(inon) = sum(fracnon(:,inon)*radvolm(:)**3)
        pmsnon(inon) = r4td * IMP_PI * rhonon(inon)
     &               * radvnon(inon)
        radvnon(inon) = radvnon(inon)**r1td

      end do

cc    non-so4 aerosol number concentration (#particles/kg air)
      do inon = 1, nnon
        xnonum(:,:,:,inon) = aernon(:,:,:,inon) / pmsnon(inon)
      end do

cc    so4 volume on each non-so4 aerosol surface (m3/particle)
      do inon = 1, nnon

        so4vnon(:,:,:,inon) =
     &    max(1.0d-30,so4non(:,:,:,inon)) 
     &    / max(r1,xnonum(:,:,:,inon))
     &    / (r4td*IMP_PI*so4mfrac(:,:,:)*so4dens(:,:,:))

      end do
#endif


c     ==============================
      ICLOOP: do ic = 1, num_species
c     ==============================

c       -------------------------------------------------
c       Loop over each species and do settling when not a 
c       fixed species and when it is an aerosol.
c       -------------------------------------------------

        if (Is_Fixed_Const(ic) .or.
     &     (aerosol(ic) <= 0)  .or. 
     &     (aerosol(ic) >  NUM_AEROSOL)) then
c         ============
          cycle ICLOOP
c         ============
        end if

c       -------------------------------------------------
c       Calculate radius of sulfate for the gravitational
c       settlement of mass and number
c       -------------------------------------------------

        if (aerosol(ic) == 1 .or. aerosol(ic) == 2 .or.
     &      aerosol(ic) == 3) then

          xlnsg = log(sigmod(aerosol(ic)))
          xlnsg2 = xlnsg * xlnsg

          if (ic == ISO4M1 .or. ic == ISO4M2 .or.
     &        ic == ISO4M3 .or. ic == IASO4) then ! change 42 spec to 61 spec,  copy info of SO4m2 to IASO4

            radius(:,:,:) = so4radv(:,:,:,aerosol(ic))
     &                    * exp(r5h * xlnsg2)

          elseif (ic == ISO4N1 .or. ic == ISO4N2 .or.
     &            ic == ISO4N3) then

            radius(:,:,:) = so4radv(:,:,:,aerosol(ic))
     &                    * exp(-r1h * xlnsg2)

          end if

          aer_den_swel(:,:,:) = so4dens(:,:,:)

        else

c       -----------------------------------------------------------
c       Swelling of aerosol due to humidity 
c       (Gong, Barrie, and Blanchet 1997 JGR 3805-3818 equation 3).
c       -----------------------------------------------------------

#ifdef NONON
          if (C1(aerosol(ic)) == 0.0d0) then

            radius(:,:,:) = RADIUS_EFF(aerosol(ic))

            aer_den_swel(:,:,:) = AER_DENSITY(aerosol(ic))

          else
            c3t(:,:,:) =
     &        C3(aerosol(ic)) *
     &        (1.0d0 + 0.004d0 * (298.0d0 - kele(:,:,:)))

            radius(:,:,:) =
     &        (C1(aerosol(ic)) *
     &         (RADIUS_EFF(aerosol(ic))*CMPM)**C2(aerosol(ic)) /
     &        (c3t(:,:,:) *
     &         (RADIUS_EFF(aerosol(ic))*CMPM)**C4(aerosol(ic)) -
     &        Log10 (relhume(:,:,:))) +
     &        (RADIUS_EFF(aerosol(ic))*CMPM)**3)**(1.0d0/3.0d0)

            aer_den_swel(:,:,:) = ((AER_DENSITY(aerosol(ic)) - 1.0d3) *
     &        (RADIUS_EFF(aerosol(ic))*CMPM)**3 + 1.0d3 *
     &        radius(:,:,:)**3) / radius(:,:,:)**3

            radius(:,:,:) = radius(:,:,:) / CMPM

          end if
#else
!          inon = aerosol(ic) - nso4  ! since aerosol(:) is unique, we can not  use this statement anymore.           
          if ( ic .le. ISO4S4 .and. ic .gt. ISO4N3 ) inon = ic - ISO4N3
          if ( ic .gt. ISO4S4 .and. ic .le. ISSLT4 ) inon = ic - ISO4S4
          
          
          if (C1(aerosol(ic)) == 0.0d0) then

            radius(:,:,:) =
     &        (RADIUS_EFF(aerosol(ic))**3 +
     &         so4vnon(:,:,:,inon))**(1.0d0/3.0d0)

            aer_den_swel(:,:,:) = (AER_DENSITY(aerosol(ic)) *
     &        RADIUS_EFF(aerosol(ic))**3 +
     &        so4dens(:,:,:) * so4vnon(:,:,:,inon)) /
     &        radius(:,:,:)**3

          else

            c3t(:,:,:) =
     &        C3(aerosol(ic)) *
     &        (1.0d0 + 0.004d0 * (298.0d0 - kele(:,:,:)))

            radius(:,:,:) =
     &        (C1(aerosol(ic)) *
     &         (RADIUS_EFF(aerosol(ic))*CMPM)**C2(aerosol(ic)) /
     &        (c3t(:,:,:) *
     &         (RADIUS_EFF(aerosol(ic))*CMPM)**C4(aerosol(ic)) -
     &        Log10 (relhume(:,:,:))) +
     &        (RADIUS_EFF(aerosol(ic))*CMPM)**3 +
     &        so4vnon(:,:,:,inon)*CMPM3)**(1.0d0/3.0d0)

            aer_den_swel(:,:,:) = ((AER_DENSITY(aerosol(ic)) - 1.0d3) *
     &        (RADIUS_EFF(aerosol(ic))*CMPM)**3 + 1.0d3 *
     &         radius(:,:,:)**3 +
     &        (so4dens(:,:,:) - 1.0d3) * so4vnon(:,:,:,inon)*CMPM3) /
     &        radius(:,:,:)**3

            radius(:,:,:) = radius(:,:,:) / CMPM

          endif
#endif
        end if

c       --------------------------------------------------------
c       Slip correction factor (Seinfeld + Pandis 1998, p. 464).
c       --------------------------------------------------------

        slip(:,:,:) =
     &    1.0d0 + (mfp(:,:,:) / radius(:,:,:)) *
     &    (1.257d0 +
     &     (0.4d0 * Exp (-(1.1d0 * radius(:,:,:) / mfp(:,:,:)))))


c       ---------------------------------------------------
c       Settling velocity (Seinfeld + Pandis 1998, p. 466).
c       ---------------------------------------------------

        velocity(:,:,:) =
     &    4.0d0 * radius(:,:,:)**2 *
     &    aer_den_swel(:,:,:) * IMP_G * slip(:,:,:) /
     &    (18.0d0 * viscosity(:,:,:))

c       ------------------------------------------------------
c       Aerosol diffusivity (Seinfeld + Pandis 1998, p. 474).
c       and velocity at k = 2 
c       ------------------------------------------------------

        diffaer(:,:,ic) = BOLTZMN_J * kele(:,:,2) * slip(:,:,2) /
     &    (6.0d0 * IMP_PI * viscosity(:,:,2) * radius(:,:,2))

        s_radius(:,:,ic) = radius(:,:,2)

        s_velocity(:,:,ic) = velocity(:,:,2)

c       --------------------------------------------------------
c       Apply the settling velocity making sure that it does not
c       exceed the Courant limit.
c       --------------------------------------------------------

        del_grid_box(:,:,:) = 
     &    velocity(:,:,:) * tdt / grid_height(:,:,k1+1:k2)

        where (del_grid_box(:,:,:) > 1.0d0) 
     &    del_grid_box(:,:,:) = 1.0d0

        del_grid_box(:,:,:) = 
     &    del_grid_box(:,:,:) * const(:,:,k1+1:k2,ic)

        const(:,:,k1:k2-1,ic) =
     &    const(:,:,k1:k2-1,ic) + 
     &    (del_grid_box(:,:,k1+1:k2) * 
     &     mass(:,:,k1+1:k2) / mass(:,:,k1:k2-1))

        const(:,:,k1+1:k2,ic) = 
     &    const(:,:,k1+1:k2,ic) - 
     &    del_grid_box(:,:,k1+1:k2)

c     =============
      end do ICLOOP
c     =============


      return

      end

