module meso_temp_cirrus
!-------------------------------------------------------------------------------
!
! Purporse: parameterize meso-scale temperature perturbation and cooling rate. 
!
! Author: Minghuai Wang, 2009-11
!
!
!-------------------------------------------------------------------------------

  use shr_kind_mod,  only: r8 => shr_kind_r8
  use ppgrid,        only: pcols, pver, pverp
  use abortutils,    only: endrun   
  
  implicit none

  public meso_temp_gary0608

  private 
  save

  real(r8), parameter  :: n2min=1.0e-8             !min value of bouyancy frequency 

contains
!========================================================================================== 

subroutine meso_temp_gary0608(lchnk, ncol, state, sgh, wgary)
!------------------------------------------------------------------------------------------
!
! Method: This subroutine parameterized meso-scale temperature perturbation, 
!         accroding Gary (2006, 2008). 
!  Gary, 2006: Mesoscale temperature fluctuations in the stratosphere, ACP, 6, 4577-4589. 
!  Gary, 2008: Mesoscale temperature fluctuations in the Southern Hemispehre stratosphere, 
!              ACP, 8, 4677-4681. 
!
!  Minghuai Wang, 2009-11
!-----------------------------------------------------------------------------------------
  use physics_types, only: physics_state
  use phys_grid, only: get_rlat_all_p
  use cam_history,  only: outfld
  use cam_history_support, only: fillvalue
  use time_manager, only: get_curr_calday
  implicit none

! Input
  integer, intent(in)              ::   lchnk    ! chunk index
  integer, intent(in)              ::   ncol     
  type(physics_state), intent(in)  ::   state    ! state variables
  real(r8), intent(in)             ::   sgh(pcols)      ! standard deviation of orograph

! Output
  real(r8), intent(out)            :: wgary(pcols, pver)  ! subgrid-scale updraft velocity
  	
   
! Local variables
  real(r8)  :: rhoi(pcols, 0:pver)             ! air density in layer interface
  real(r8)  :: ni(pcols, 0:pver)               ! interface Brunt-Vaisalla density
  real(r8)  :: ti(pcols, 0:pver)               ! interface temperature
  real(r8)  :: nm(pcols, pver)                 ! midpoint Brunt-Vaisalla frequency
  real(r8)  :: ntmp                            ! Brunt_Vaisalla frequency
  real(r8)  :: ftmp
  real(r8)  :: finert(pcols)                   ! inertial frequency
  real(r8)  :: e_h, e_w
  real(r8)  :: clat(pcols)       ! current latitude (radians)
  real(r8)  :: calday            ! current calendar day
  real(r8)  :: wt                ! wintriness parameter (Eq. (3) in Gary (2006). 
  real(r8)  :: mfa               ! vertical displacment (meter)
  real(r8)  :: lat_deg        ! latitude in degree
  real(r8)  :: ptemp           ! pressure (Pa)
  real(r8)  :: topo(pcols)
  integer   :: i, k 
  real(r8)  :: sigmat  ! subgrid-scale temperature perturbations 
 
! latitude
  call get_rlat_all_p(lchnk, pcols, clat) 

! calendar day
  calday = get_curr_calday()

! Wintriness parameter (Eq. (3) in Gary (2006). 
! wt is 1 on 22 January, and 0 on 22 July. 

!  wt=0.5*(1+sin((calday-295.)/365*3.1415927/2.))    ! this formula was wrong

! correct wt formula  (Gary, personal communication)
   wt=0.5*(1+sin((calday-295)/365*3.1415927*2.))

! topography  set to be 0 for the momment
  topo = 0.0

! calculate sigmat based on Gary (2006, 2008). 
  do i=1, ncol
    if(sgh(i).le.10) then
      topo(i) = 0.0
    else if(sgh(i).le.100) then
      topo(i) = 0.4
    else if(sgh(i).le.300) then
      topo(i) = 0.6
    else if(sgh(i).le.500) then
      topo(i) = 0.8
    else 
      topo(i) = 1.0
    endif

    do k=1, pver
      ptemp = state%pmid(i,k)
!      ptemp = 10000._r8    ! ptemp fixed at 100 hPa
      lat_deg = clat(i)/(3.1415927/180.)
      if(lat_deg .gt. 0._r8) then   ! Northern hemisphere
        ! Eq. (6) in Gary (2006)
        !mfa = (112-1.21*(lat_deg)+2.2*wt*(lat_deg)+29.0*topo(i))*(ptemp/5885._r8)**(-0.4)
        
        ! change the density relationship from 0.4 to 0.5. So wave amplitude is inversely propotional to sqrt(rho). By Cheng 1/9/2017
         mfa = (112-1.21*(lat_deg)+2.2*wt*(lat_deg)+29.0*topo(i))*(ptemp/5885._r8)**(-0.5)
         
      else     ! Southern hemisphere
        ! Eq. (4) in Gary (2008), but add topography effects, following Gary (2006). 
        !mfa = (114-0.42*(lat_deg)+0.84*wt*(lat_deg)+29.0*topo(i))*(ptemp/5885._r8)**(-0.4)
         mfa = (112-1.21*(lat_deg)+2.2*wt*(lat_deg)+29.0*topo(i))*(ptemp/5885._r8)**(-0.5)  
      end if
      mfa = mfa/2.354  ! mfa calculted above is 'full-width at half-maximum', and this is need to converted 
                       ! the standard deviation. 
      sigmat = mfa/100.  
      
      wgary(i,k) = 0.23_r8 * sigmat ! see Wang and Penner 2010.  Cooling rate = 8.6 * sigmat (K/hour) according to KB08. Then use adabatic cooling elapse rate to get W.  
    end do
  end do

end subroutine meso_temp_gary0608
!--------------------------------------------------------------------------------


end module meso_temp_cirrus
