module um_KL_ni
!***********************************************************************
!                           KL_ni
!-----------------------------------------------------------------------
! Ref: Karcher et al. (2006)
! Intro: Ice cloud nucleation parameterization to calculate ni
! 2016/10/04, Use Ren and MacKenzie 2005 parameterization for d log (J)/dT  as suggested by Karcher et al 2006. The old formula from KL2002 overestimate the Ni for homo. 
! Initially coded by Yang Chen
! Cheng Zhou modified it to use W series.
!***********************************************************************

use shr_kind_mod,     only: r8=>shr_kind_r8      	
implicit none
private
save
	
public :: KL_ni
	
integer, parameter :: nw=29 
real(r8), parameter :: wdt=2.2*60 !seconds. generate nw points, each 2.2 min apart. So it about 1hour w series.  


logical, save  ::Is_first_call = .true. 

real(r8) :: ww (nw)                ![updraft velocity, m/s]
      	
contains


      SUBROUTINE KL_ni(Na_so41,Na_so42, & !r_so41,r_so42,sg1_so4,sg2_so4, 
                       Na_soot_sf,Na_soot_air,Na_dust,&
                       wstd,TK,RH,P, &
                       Nitot,Fhomo,Fimme,Fdepo) !,RHmax)
      implicit none
!-----------------------------------------------------------------------
!     Parameter declaration
!-----------------------------------------------------------------------
!      integer, parameter :: r8=8
! input
!      integer, intent(in) :: nw
      real(r8),intent(in) :: Na_so41,Na_so42,Na_soot_sf,Na_soot_air,Na_dust  	  ![sulfate/soot number, #/cm3]
                                          

!      real(r8),intent(in) :: ww (nw)                ![updraft velocity, m/s]
!      real(r8),intent(in) :: wdt!, nw ww points, each wdt seconds apart                    
      real(r8),intent(in) :: TK                            ![Temperature, K]
      real(r8),intent(in) :: RH                                       ![RHw], ~1, should be RHi not RHw as no conversion is done below to Si
      real(r8),intent(in) :: P                                ![pressure,Pa]
      real(r8),intent(in) :: wstd ! std of w series, m/s
      	
!      real(r8),intent(in) :: r_so41,r_so42         ![sulfate mode radius,um]      	
!      real(r8),intent(in) :: sg1_so4                        ![Sigma1 of SO4] 
!      real(r8),intent(in) :: sg2_so4                        ![Sigma2 of SO4]


      real(r8) :: r_so41,r_so42         ![sulfate mode radius,um]      	
      real(r8) :: sg1_so4                        ![Sigma1 of SO4] 
      real(r8) :: sg2_so4                        ![Sigma2 of SO4]

! output
      real(r8),intent(out) :: Nitot,Fhomo,Fimme,Fdepo 
      real(r8) :: RHmax                  ![max RHi from KL para]

! number of modes and bins
      integer,parameter :: nmode = 3            ![number of nucleation mode]
      integer,parameter :: nbin = 30   ![currently,1:soot; 2:dust;3:sulfate]

! time step
      real(r8) :: deltat                    ![large-scale model time step,s]
      real(r8) :: dt                                 ![adaptive time step,s]
      real(r8) :: t                             ![time from the beginning,s]

! meteorological parameters
      real(r8) :: temp                                      ![temperature,K]
      real(r8) :: Si                      ![vapor saturation ratio over ice]
      real(r8) :: pres                                        ![pressure,Pa]
      real(r8) :: w                                  ![updraft velocity,m/s]

! ice paras
      real(r8) :: na(nmode,nbin)    ![number concentration in each bin,#/m3]
      real(r8) :: na0(nmode,nbin)    ![number concentration in each bin,#/m3], store initial data but follow air density change
      real(r8) :: r0(nbin)                           ![radius of each bin,m]
      real(r8) :: ni(nmode)    ![ice number concentration of each mode,#/m3]
      real(r8) :: ri(nmode)            ![ice particle radius of each mode,m]
      real(r8) :: wp                      ![apparent downdraft velocity,m/s]
      real(r8) :: Sicr(nmode)      ![critical supersaturation for each mode]

! local
      integer imode,ibin,outloop
      real(r8) a1,a2,a3,b1,b2,sumice,nsum,sumo,nsumo,sumcomp,delta,c,k,f,delta_ice
!      real(r8) :: nu            ![h2o molecule spicific volume,m3/molecule]
      real(r8)  nuth                                ![h2o thermal speed,m/s]
      real(r8)  esi                  ![saturation vapor pressure on ice, Pa]
      real(r8)  nsat            ![h2o number density at ice saturation,#/m3]
      real(r8)  phi(nmode)                             ![ventilation factor]
      real(r8)  Dv                        ![diffusivity of water vapor,m2/s]
      real(r8)  Dvm              ![modified diffusivity of water vapor,m2/s]
      real(r8)  taug                         ![initial growth timescale,1/s]
      real(r8)  tau                                ![freezing timescale,1/s]
      real(r8)  rsk                               ![smallest r0 to freeze,m]
      real(r8)  Rim                                  ![freezing/growth term]
      real(r8)  tempnew                     ![new results after 1 time step]
      real(r8)  presnew
      real(r8)  Sinew
      real(r8)  ninew(nmode)
      real(r8)  rinew(nmode)
      real(r8)  ninew1(nmode)
      real(r8)  rinew1(nmode)
      real(r8)  ninewfm(nmode)             ![newly formed ice concentration]
      real(r8)  nanew(nmode,nbin)      ![num concentration in each bin,#/m3]

      real(r8)  rmin,rmax                            ![aerosol distribution]
      real(r8)  natot,rg1,sg1,rg2,sg2,rg3,sg3
      real(r8)  ag1,bg1,ag2,bg2,ag3,bg3,z
      real(r8)  fg1,fg2,fg3
      
      real(r8)  airdensity_factor

      real(r8) dt_sum !total time when a new ww  point is used
      integer  iw
      logical  evap_flag(nmode)
      real(r8) delta_ice_arr(nmode), ice_accum(nmode), ice_init(nmode)
! Constant
      real(r8),parameter :: g = 9.81            ![gravity acceleration,m/s2]
      real(r8),parameter :: R = 8.314      ![universal gas constant,J/mol/K]
      real(r8),parameter :: Ls = 2.836e6  ![latent heat of sublimation,J/kg]
      real(r8),parameter :: Mw = 0.018         ![molar mass of water,kg/mol]
      real(r8),parameter :: Ma = 0.029           ![molar mass of air,kg/mol]
      real(r8),parameter :: cp = 1005      ![specific heat capacity of air,
                                                                !1005J/kg/K]
      real(r8),parameter :: rhoi = 925             ![ice mass density,kg/m3]
      real(r8),parameter :: alpha = 0.5         ![deposition coefficient of
                                                             ! water on ice]
      real(r8),parameter :: kb = 1.381e-23         ![Boltzmann constant,J/K]
      real(r8),parameter :: Avog = 6.022e23        ![Avogadro's number,/mol]
      real(r8),parameter :: pi = 3.14159                      ![constant Pi]
      real(r8),parameter :: Capf = 1               ![Capacitance factor for
                                               ! non-spherical shape of ice]
      real(r8),parameter :: ridef = 4.0e-5           ![default ice radius,m]
      real(r8),parameter :: vjl = 1.04e-7             ![vapor jump length,m]
      real(r8),parameter :: nu = Mw/rhoi/Avog        ![specific volume of a
                                               ! water molecule,m3/molecule]
                                               
      real(r8),parameter :: dtmax=1.0 !0.2 ! maximum timestep is 30 seconds


 ! for random number
       INTEGER :: ii, nn, clock
       INTEGER, DIMENSION(:), ALLOCATABLE :: seed
       	
!-----------------------------------------------------------------------
!     Initialization of randon number
!-----------------------------------------------------------------------
      if (Is_first_call) then 
      	
      	 !!!!!!!!!  random seed 
!            CALL RANDOM_SEED(size = nn)
!            ALLOCATE(seed(nn))
!            CALL SYSTEM_CLOCK(COUNT=clock)
!            seed = clock + 37 * (/ (ii - 1, ii = 1, nn) /)
!            CALL RANDOM_SEED(PUT = seed)
!            DEALLOCATE(seed)
!            print*, 'nn, clock', nn, clock
            
      	    Is_first_call = .false.
      end if
      
      call RANDOM_NUMBER(ww)	 ![0,1)   
      !print*,ww
      do ii = 1, nw
      ww(ii)=ww(ii)-0.5; !(-0.5, 0.5)
      if (ww(ii) .ge. 0.0) then
       ww(ii)= -0.1202*log(1-2*ww(ii)); ! is the generated white noise following Laplace distribution. -0.1202 corresponds to a standard deviation of 0.17 m/s
       else
       ww(ii)=  0.1202*log(1+2*ww(ii)); ! is the generated white noise following Laplace distribution. 	
      end if
      end do 	
      ww(1) = abs(ww(1))+1e-2 !! let the first w always be positive so that if RH is big enough, Het always occurs
      ww = ww - sum(ww)/nw ! get rid of the mean
      
      ww(:) = ww(:)/0.17*wstd ! scale ww to have a std if
      !ww(:) = wstd ! set uniform ww
      !      print*, 'wstd, ww(2)=',wstd,ww(2)

!-----------------------------------------------------------------------
!     Initialization
!-----------------------------------------------------------------------


      Nitot=0.
      Fhomo=0.
      Fimme=0.
      Fdepo=0.

! meteorological data
      temp = TK
      pres = P
      !w    = 0.2
      Si   = RH !1. !RH*(273/temp)**2.66  !1
      RHmax = Si

! set Sicr of different modes and sort them
      Sicr(1)=1.35                                             ![soot]
      Sicr(2)=1.2                                           ![dust]
      Sicr(3)=2.583-temp/207.83                            ![sulfate]
      
!-----------------------------------------------------------------------
! exclude the grids with w and T out of para range:
!   1E-10 < W < 5 m/s
!   -80C  < T < -35 C
!-----------------------------------------------------------------------
      if(temp.lt.193.15)  temp=193.151
 
!      if (w.gt.1.e-10.and.w.lt.5.and.temp.gt.193.15.and.temp.lt.238.15) then

! aerosol distribution
      rmin = 1e-9 !5e-9
      rmax = 1e-5
      z = exp(log(rmax/rmin)/nbin)
      do ibin=1,nbin
       r0(ibin) = z**(ibin-0.5)*rmin
      end do

! Mode 1: Soot -- (1)surface source (2)aircraft source
      natot = (Na_soot_sf+Na_soot_air)*1.e+6
      rg1    = 7e-8
      sg1    = 1.5
      fg1    = Na_soot_sf/(Na_soot_sf+Na_soot_air+1e-30)
      
      rg2    = 2.3e-8!2.3e-8  from Yang Chen
      sg2    = 1.5
      fg2    = 1-fg1
!      rg1 = 0.5e-6
!      sg1 = 1.4
!      fg1 = 0.5
!      rg2 = rg1
!      sg2 = sg1
!      fg2 = fg1
      bg1    = 2.*log(sg1)**2.
      bg2    = 2.*log(sg2)**2.
      do ibin=1,nbin
       ag1 = (log(r0(ibin))-log(rg1))**2.
       ag2 = (log(r0(ibin))-log(rg2))**2.
       na(1,ibin) = natot*fg1/((2.*pi)**0.5*log(sg1))*exp(-ag1/bg1)*log(z) +      &
                    natot*fg2/((2.*pi)**0.5*log(sg2))*exp(-ag2/bg2)*log(z)
      end do

! Mode 2: Dust -- three mode size distribution
      natot = Na_dust*1.e+6
!      if (natot.le.1) natot=1    ! set background value of dust 1/m3

      rg1    = 1e-8
      sg1    = 2.3
      fg1    = 0.152
      rg2    = 4.5e-8
      sg2    = 1.6
      fg2    = 0.727
      rg3    = 27.5e-8
      sg3    = 2.5
      fg3    = 0.121
      bg1    = 2.*log(sg1)**2.
      bg2    = 2.*log(sg2)**2.
      bg3    = 2.*log(sg3)**2.
      do ibin=1,nbin
       ag1 = (log(r0(ibin))-log(rg1))**2.
       ag2 = (log(r0(ibin))-log(rg2))**2.
       ag3 = (log(r0(ibin))-log(rg3))**2.
       na(2,ibin) = natot*fg1/((2.*pi)**0.5*log(sg1))* exp(-ag1/bg1)*log(z) +       &
                    natot*fg2/((2.*pi)**0.5*log(sg2))* exp(-ag2/bg2)*log(z) +       &
                    natot*fg3/((2.*pi)**0.5*log(sg3))* exp(-ag3/bg3)*log(z) 
      end do

! Mode 3: SO4-- one mode size distribution
      sg1_so4=1.6                        ![Sigma1 of SO4]   ! Jensen et al., 1994: rg=0.02um, sig=2.3
      sg2_so4=1.6                        ![Sigma2 of SO4]        
      r_so41 = 0.02
      r_so42 = 0.06       ![sulfate mode radius,um]
      
      
      natot = (Na_so41+Na_so42)*1.e+6
      rg1    = r_so41*1e-6   !2e-8
      sg1    = sg1_so4  ! Roy
      fg1    = Na_so41/(Na_so41+Na_so42)
      
      rg2    = r_so42*1.e-6   !2e-8
      sg2    = sg2_so4  ! Roy
      fg2    = 1-fg1

      bg1    = 2.*log(sg1)**2.
      bg2    = 2.*log(sg2)**2.
      do ibin=1,nbin
       ag1 = (log(r0(ibin))-log(rg1))**2.
       ag2 = (log(r0(ibin))-log(rg2))**2.
       na(3,ibin) = natot*fg1/((2.*pi)**0.5*log(sg1))*exp(-ag1/bg1)*log(z) +           &
                    natot*fg2/((2.*pi)**0.5*log(sg2))*exp(-ag2/bg2)*log(z)
      end do

!store original na
      na0(:,:)=na(:,:)


! ice, initialization !!!!!!!!!!!!!!!!!
      do imode=1,nmode
       ni(imode)=0.
       ri(imode)=1e-7 !!
      end do
      RHmax  = Si 
      iw     = 1
      w      = ww(iw)
      dt_sum = 0.0
      ice_init(:) = 0.0_r8
      ice_accum(:) = 0.0_r8
      
! time step. 
      deltat = 30.*60. !30.*60.   seconds        ![GCM time step, 30min here]
      a1 = Ls*Mw*g/(cp*R*temp**2.)-Ma*g/(R*temp)
      dt = 1.e-2/(a1*abs(w))
      dt = min(dt, dtmax) ! ensure dt is always less than 30 s
      t  = 0.
      outloop=0
 
!      open(100, file='Si_IN.txt')
 
!!----------------------------- Start the main loop.  -----------------------------
!
!----------------------------------------------------------------------------------
      do while(t.le.deltat)
!-----------------------------------------------------------------------
! Step 1:    Initial growth of existing ice 
!-----------------------------------------------------------------------
! calculate local parameters
      nuth= (8*Kb*temp*Avog/pi/Mw)**0.5
      esi = 3.41e12*exp(-6.13e3/temp)
      nsat = esi*Avog/(R*temp)
      Dv = 0.211e-4*(temp/273.15)**1.94*(1.013e5/pres)
      Dvm = Dv/                       &![alpha is deposition coefficient]
          (ridef/(ridef+vjl)+4*Dv/ridef/alpha*(2*pi*Mw/R/temp)**0.5)
      b1 = nu*alpha*nuth*nsat*(Si-1)/4.
      !print*,nu,alpha,nuth,nsat,Si,b1
      b2 = alpha*nuth/(4*Dvm*Capf)

!      if (temp.lt.216) then
!       c = 100*(22.6-0.1*temp)
!      else
!       c = 100
!      end if


! calculate prognostic parameters, used dt here
      if (temp.lt.190) temp=190. 
1001  tempnew = temp - g/cp*w*dt   !!! if  abs(Sinew/Si-1.) > 0.01, come back here to start over
      presnew = pres *(tempnew/temp)**(cp/R*Ma)
      a1 = Ls*Mw*g/(cp*R*tempnew**2.)-Ma*g/(R*tempnew)
      a2 = 1/nsat
      a3 = Ls**2.*Mw**2./(cp*presnew*tempnew*Ma*Avog)
!      print*,dt,cp/R-1,(tempnew/temp)**(cp/R*Ma-1)

!---------------------------------------------------------------------
! Step 1: calcualte the initial growth/decay of existing ice and effect from air density change
!-----------------------------------------------------------------------
      !initailization
      sumice = 0.0
      evap_flag(:) = .False. ! accumulated ice is not all gone
      delta_ice_arr(:) = 0.0_r8
      ninew1 (:) = 0.0_r8
      rinew1 (:) = 0.0_r8       
      
      do imode=1,nmode
       delta_ice = 0.0_r8
       airdensity_factor = (tempnew/temp)**(cp/R*Ma-1) ! aerosol number changes due to the air density change
!       do ibin=1,nbin
!       nanew(imode,ibin)=na(imode,ibin)*(tempnew/temp)**(cp/R*Ma-1)! aerosol number changes due to the air density change
!       end do
       nanew(:,:)=na(:,:)*airdensity_factor

       phi(imode)=1      ![detail calculation of phi need Renold Number]
       if (ni(imode) .gt. 1.0e-20) then ! ice exist
       ninew1(imode)= ni(imode) * airdensity_factor !(tempnew/temp)**(cp/R*Ma-1) ! ice number changes due to the air density change

!!!!!!!!!!!!!!!!depositional growth     
       if (Si .gt. 1.0 ) then 
       rinew1(imode)= ((1+b2*ri(imode))*(1+2*phi(imode)*b1*b2*dt/(1+b2*ri(imode))**2.)**0.5-1 )/b2
       delta_ice    = (4*pi/nu)*phi(imode)*b1*rinew1(imode)**2./(1+b2*rinew1(imode))*ninew1(imode)! eq 12
       sumice       = sumice + delta_ice
       end if

!!!!!!!!!!!!!!!!sublimation
       if (Si .lt. 1.0 ) then 
       delta_ice =    (4*pi/nu)*phi(imode)*b1*ri(imode)**2./(1+b2*ri(imode))*ninew1(imode) ! eq 12
       
       rinew1(imode)= ((1+b2*ri(imode))*(1+2*phi(imode)*b1*b2*dt/(1+b2*ri(imode))**2.)**0.5-1 )/b2  
!        rinew1(imode)= max(rinew1(imode), 1e-7_r8)

       
!        delta_ice =    (4*pi/nu)*phi(imode)*b1*rinew1(imode)**2./(1+b2*rinew1(imode))*ninew1(imode)

        !if delta_ice is more than available ice, sublimate all ice. here we use the ice mass from the first ice nucleation event as a threshhold
!         if ( (ice_accum(imode)*airdensity_factor+delta_ice) .lt.  ice_init(imode)  )  then 
        if ( (ice_accum(imode)*airdensity_factor+delta_ice) .lt.  ice_init(imode) .or. rinew1(imode) .lt. 1e-7)  then 
         delta_ice        =  -ice_accum(imode)* airdensity_factor
         evap_flag(imode) = .True. !! all ice is gone
        end if
        
       rinew1(imode)= max(rinew1(imode), 1e-7_r8)    
       sumice = sumice +  delta_ice
       end if  
!!! store delta_ice       
       delta_ice_arr(imode) = delta_ice  
          
       end if      ! ice exist
              
      end do   ![end of imode]
      
      wp = (a2+a3*Si)/(a1*Si)*sumice  ! equivalent downdraft speed when Si>1
      Sinew = Si+a1*Si*(w-wp)*dt   !

!!check time step to insure Sinew does deviate old Si by 1%      
      if (abs(Sinew/Si-1.).ge.0.01) then
      !adjust dt and repeat current step
      ! print*, t/60.0, dt,'dt=dt*0.9'
       dt=dt*0.9
       goto 1001  !! go back to Step 1
        
      else 
      
!-----------------------------------------------------------------------
! Step 1.5: update variables when dt works
!-----------------------------------------------------------------------
        ! Now dt works, we can actually update ice_accum and Na. You can not do it above as dt may not work 
         na0(:,:)  =na0(:,:)*airdensity_factor  
         do imode=1,nmode
         
           ice_accum(imode) = ice_accum(imode) * airdensity_factor +  delta_ice_arr(imode)
        
           if (evap_flag(imode)) then ! if all ice is gone
!            print*, '------------all ice is gone---- in mode:', imode
           
            ! reset ice number/size
            ninew1 (imode) = 0.0_r8
            rinew1 (imode) = 0.0_r8  
            ! restore aerosol number
            nanew(imode,:) = na0(imode,:)  
            ice_init(imode) = 0.0_r8  !! set ice from initial ice nucleation event to 0. 
           end if
         end do  
                 
   
              
      	
!-----------------------------------------------------------------------
! Step 2:, check Nucleation, beware of the if above
!-----------------------------------------------------------------------
      do imode=1,nmode

       if(imode.eq.3) then
         c=100 !00 !! default 100, the smaller c menas bigger tau/kappa, smaller Ni.
       else
         c=200
       endif

      if(w.gt.0.0 .and. Si.gt.Sicr(imode).and.(w.gt.wp) .and. (na(imode,1).gt.0.0_r8) ) then ! only do when the first bin still has some aerosol.
 
        sumice  = 0.0
        nsum    = 0.0
        sumcomp = a1*Sicr(imode)/(a2+a3*Sicr(imode))*(w-wp) !! maximum allowed deposited ice amount.
        ice_accum(imode) = ice_accum(imode) + sumcomp !!  ice increases during nucleation step
        if  (ice_init(imode) .lt. 1e-20)  then 
           ice_init(imode) = sumcomp !! store the ice mass from the first (or 1st after total sublimation) ice nucleation event
!           print*, 'ice_init', imode, sumcomp
        end if
        ! consider the nucleation/growth starts from the largest bin 
        do ibin=nbin,1,-1
         
         ! add this if since the largest bins are depeleted first. Save computer time.
         if (na(imode,ibin) .gt. 0.0 ) then
         delta=b2*r0(ibin)
         taug=(1+delta)*r0(ibin)/b1
         !tau=(-c*(4.37-0.03*temp)*g/cp*w)**(-1) !! d log(J)/dT is from KL2002
         tau=(-0.01*c*(-0.004*temp**2+2.0*temp-304.4)*g/cp*w)**(-1) !! from Ren and MacKenzie 2005. 0.01 is added to use c=100 or 200 in the front 
         k=2*delta/(1+delta)*tau/taug
         f=3*sqrt(k)/(2*sqrt(1/k)+sqrt(1/k+9/pi))
         Rim=(4*pi/nu)*(b1/b2**2)*(delta**2/(1+delta))*(1-1/delta**2+1/delta**2*((1+delta)**2/2+1/k)*f)

         sumo=sumice
         nsumo=nsum
         sumice=sumice+na(imode,ibin)*Rim
         nsum=nsum+na(imode,ibin)
         end if ! end na(imode,ibin) .gt. 0.0
         
         ! see if maximum	amount is met
         if (sumice.gt.sumcomp) then
          ninew(imode)=  &
           (nsum*(sumcomp-sumo)+nsumo*(sumice-sumcomp))/(sumice-sumo)
          rsk=(r0(ibin)*(sumcomp-sumo)+  &
             r0(ibin-1)*(sumice-sumcomp))/(sumice-sumo)
          rinew(imode)=((1+f/2.)*(1+b2*rsk)-1)/b2
          exit
         end if    

         if (ibin.eq.1) then
!         	print*, imode, 'ibin=1', Si
          ninew(imode)= nsum
          rsk = rmin
          rinew(imode)=((1+f/2.)*(1+b2*rsk)-1)/b2
         end if
         		  
        end do                                     ![end of ibin]

        ! averaged radius. The way of averaging could be problematic. But as the ice nucleation generally occurrs within one step, it is fine. 
        rinew(imode)= (rinew(imode)*ninew(imode)+rinew1(imode)*ninew1(imode))/(ninew(imode)+ninew1(imode))
        ninew(imode)=  ninew(imode)+ninew1(imode)
        
       else  ! No new nucleation occurs
        
        rinew(imode)= rinew1(imode) ! only consider the growth/sumlimation 
        ninew(imode)= ninew1(imode) ! only consider the sublimation caused or air density change caused changes
       end if  !if(Si.gt.Sicr(imode).and.(w.gt.wp) .and. (na(imode,1).gt.0.0) ) then ! only do when the first bin still has some aerosol.
!      print*,imode,rinew(imode),rinew1(imode)
      end do                                       ![end of imode]

!-------------------
!Step3: update Ni and Ri also subract Na
!-------------------        
       na(:,:)=nanew(:,:) !! still need to substract the nucleated aerosols in the loop below.

       do imode=1,nmode
        ninewfm(imode)=ninew(imode)-ni(imode)*airdensity_factor!! Cheng added this factor, as ni should be after the adjust for this step. w/o this, nifewfm could be <0 
        ni(imode)=ninew(imode)
        ri(imode)=rinew(imode)
        
!        do ibin=nbin,1,-1
!         na(imode,ibin)=nanew(imode,ibin)
!        end do
        
        
        if (ninewfm(imode) .lt. 0.0_r8)  ninewfm(imode) = 0.0_r8  ! if ninew1 is set to 0, this  could be negative. 
        
        !substract new ice particle # from aerosol number, assume big ones nucleate first and all gone until the # equal to ice #
        do ibin=nbin,1,-1
         if (nanew(imode,ibin).lt.ninewfm(imode)) then
          ninewfm(imode)=ninewfm(imode)-nanew(imode,ibin)
          nanew(imode,ibin)=0.
          na(imode,ibin)=nanew(imode,ibin)
         else
          nanew(imode,ibin)=nanew(imode,ibin)-ninewfm(imode)
          na(imode,ibin)=nanew(imode,ibin)
          exit
         end if
        end do
  
       end do ! [end imode]


!-------------------
!Step4: set dt for next step, calculate new ni,ri,na
!-------------------  
       Si=Sinew
       temp=tempnew            
       if(Sinew.lt.Si) outloop=outloop+1  ![if Si decreases, outloop ++]
       if(Si.gt.RHmax) RHmax=Si ! record max RHi       	
       	
       t      = t+dt  ! total time elapsed
       dt_sum = dt_sum+dt ! time since this w is used.
       dt=dt*1.02
       dt=max(dt, 5.0_r8) ! let at least 10 second.
       dt=min(dt, dtmax) ! maximum timestep is dtmax
       w =ww(iw)+(ww(iw+1)-ww(iw))*dt_sum/wdt
       ! if all current wdt is used, the march to next w.
       if (abs(dt_sum-wdt) .lt. 1e-10) then
         dt_sum = 0.0
         iw     = iw + 1 
         w      = ww(iw)
       elseif ( (dt_sum+dt) .gt. wdt ) then  ! next dt will stride two w, limit dt
       	 dt = wdt-dt_sum
       end if
       	
!       write(100,'(i4,100e15.5e4)')iw,w,dt_sum,dt,t,temp,Si,Ni(1),ri(1),Ni(2),ri(2),Ni(3),ri(3), &
!                                   ice_accum(1) ,ice_accum(2) ,ice_accum(3)   
 
      ! print*,iw,w,dt_sum,dt,t,temp,Si,Ni(1),ri(1),Ni(2),ri(2),Ni(3),ri(3) 
!        write(100,'(100e13.5)')w,dt_sum,dt,t,temp,Si,Ni(1),ri(1),Ni(2),ri(2),Ni(3),ri(3) 
      
      end if  !! end if (abs(Sinew/Si-1.).lt.0.01) then


      end do                               ![end of large time step]

!-----------------------------------------------------------------------
!     Output
!-----------------------------------------------------------------------
       !!! Limit each mode to 10/cc, by Cheng on 8/21/2017
!       ni(1) = min(ni(1),10.0e6)
!       ni(2) = min(ni(2),10.0e6)
!       ni(3) = min(ni(3),10.0e6)

      Nitot = ni(1)+ni(2)+ni(3)
      ni(2) = ni(2)+1.0_r8 ! set minimum depositional mode ice number 1 #/m3
!      if(Nitot.gt.0) then
!       Fhomo = ni(3)/Nitot
!       Fimme = ni(1)/Nitot
!       Fdepo = ni(2)/Nitot
!      else
!       Fhomo=0
!       Fimme=0
!       Fdepo=0
!      end if
      Fhomo = ni(3)*1e-6
      Fimme = ni(1)*1e-6
      Fdepo = ni(2)*1e-6    
      Nitot = Nitot*1e-6                ! /m3 --> /cm3
      

!      write(*,'(E12.3)') Nitot
!      do imode=1,nmode
!       write(*,'(i3,2E12.3)') imode,ni(imode),ri(imode)
!      end do

!      end if                                 ![end of w and temp]
      	
      	
!      close(100)
      END SUBROUTINE KL_ni
      
      
      end module
