!-------------------
!Purpose: follow Chen et al. 2012 's treatment to determine whether contrail forms or not. 
!JOURNAL OF ADVANCES IN MODELING EARTH SYSTEMS, VOL. 4, M04003. doi:10.1029/2011MS000105, 2012
!Author: Cheng Zhou, zhouc@umich.edu
!-------------------

    module  um_contrail_form
    use shr_kind_mod,  only: r8=>shr_kind_r8
    	
    implicit none
    private 
    save
    
    public   um_contrail_flag
    
    contains
    
    subroutine  um_contrail_flag (pm0, t, qm, contrail)
    
    implicit none
    
    !input argument
    real(r8), intent(in) :: pm0 ! = 20000! Pa
    real(r8), intent(in) :: t  != 230  ! k
    real(r8), intent(in) :: qm != 1e-3 ! specific humidity, kg/kg
    real(r8), intent(out) :: contrail ! 1 : contrail form , 0: no contrail	
    		
    !local variable
    real(r8), parameter :: epsilon = 0.622_r8 !=18.016_r8 / 28.966_r8 ! water vapor molecular weight to dry air molecular weight
    real(r8), parameter :: EI = 1.21_r8  ! water vapor emission index (g) h2o per kg fuel (Schumann 1996),  Schumann 2012 uses 1.25?
    real(r8), parameter :: Q  = 43.e6_r8 ! specific combustion heat (J/kg), from Schummann 1996
    real(r8), parameter :: eta=	0.3_r8   ! propulsion effieciency of the jet engine (from Ponater 2002)
    real(r8), parameter :: cp = 1.00464e3_r8    ! specific heat of dry air   ~ J/kg/K
    real(r8) :: G,  Tc, esat_Tc, esat_T, RHc, pm
    real(r8) :: w, esat_i, RHi, RH, qsat
    integer  :: itype	
      
        pm = max (pm0, 1e3_r8) ! skip top layers. When pm is too small,log(G-0.053_r8) is not valid. 
 ! get the contrail formation temperature
        G = (EI*cp*pm)/(epsilon*Q*(1.0_r8-eta))   ! eq 12
        Tc = -46.46_r8 + 9.43_r8*log(G-0.053_r8) + 0.72_r8*log(G-0.053_r8)**2 ! eq 11
        Tc = 273.16_r8 + Tc  ! convert to K
       
!        print*, 'G=', G, 'Tc=',Tc
        
 ! calculate critical saturation RH w.r.t. liquid water    
        itype = 0    
        call gffgch2(Tc,esat_Tc,itype)    
        call gffgch2(t, esat_T, itype)
 
        RHc = ( G * (t-Tc)+ esat_Tc ) / esat_T
!        print*,'esat_Tc=',esat_Tc, 'esat_T=',esat_T, 'RHc=', RHc
        
        RHc = max(min(RHc, 1.0_r8),0.0_r8)

! calculate the ambient RH   
        qsat = epsilon * esat_T / pm  !saturated water vapor mass mixing ratio w.r.t. liquid
        RH   = qm / qsat              ! relative humidity w.r.t. liquid
!        print*, 'qsat=',qsat, 'RH=', RH
                

!        
!  calculate critical saturation RH w.r.t. liquid water  
        itype = 1  
        call gffgch2(t,esat_T,itype)       !saturation water vapor with respect to ice  (Pa)       
        qsat = epsilon * esat_T / pm   !saturated mixing ratio w.r.t. ice
        RHi  = qm / qsat              ! relative humidity w.r.t. ice
!        print*, 'qsat=',qsat, 'RHi=', RHi
!        
!
! persistent contrail condition    	  
        if( (t.lt.Tc) .and. (RH.gt.RHc) .and. (RHi.gt.1.0_r8) ) then
        	contrail = 1.0_r8
         else
         	contrail = 0.0_r8		
        end if
         
        end subroutine  um_contrail_flag
 
 
 subroutine gffgch2(t       ,es      ,itype   )
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Computes saturation vapor pressure over water and/or over ice using
! Goff & Gratch (1946) relationships. 
! <Say what the routine does> 
! 
! Method: 
! T (temperature), and itype are input parameters, while es (saturation
! vapor pressure) is an output parameter.  The input parameter itype
! serves two purposes: a value of zero indicates that saturation vapor
! pressures over water are to be returned (regardless of temperature),
! while a value of one indicates that saturation vapor pressures over
! ice should be returned when t is less than freezing degrees.  If itype
! is negative, its absolute value is interpreted to define a temperature
! transition region below freezing in which the returned
! saturation vapor pressure is a weighted average of the respective ice
! and water value.  That is, in the temperature range 0 => -itype
! degrees c, the saturation vapor pressures are assumed to be a weighted
! average of the vapor pressure over supercooled water and ice (all
! water at 0 c; all ice at -itype c).  Maximum transition range => 40 c
! 
! Author: J. Hack
! 
!-----------------------------------------------------------------------
!   use shr_kind_mod, only: r8 => shr_kind_r8
!   use physconst,    only: tmelt
!   use abortutils,   only: endrun
!   use cam_logfile,  only: iulog
    
   implicit none
!------------------------------Arguments--------------------------------
!
! Input arguments
!

   real(r8),parameter :: tmelt = 	273.15

   	
   real(r8), intent(in) :: t          ! Temperature
!
! Output arguments
!
   integer, intent(inout) :: itype   ! Flag for ice phase and associated transition

   real(r8), intent(out) :: es         ! Saturation vapor pressure
!
!---------------------------Local variables-----------------------------
!
   real(r8) e1         ! Intermediate scratch variable for es over water
   real(r8) e2         ! Intermediate scratch variable for es over water
   real(r8) eswtr      ! Saturation vapor pressure over water
   real(r8) f          ! Intermediate scratch variable for es over water
   real(r8) f1         ! Intermediate scratch variable for es over water
   real(r8) f2         ! Intermediate scratch variable for es over water
   real(r8) f3         ! Intermediate scratch variable for es over water
   real(r8) f4         ! Intermediate scratch variable for es over water
   real(r8) f5         ! Intermediate scratch variable for es over water
   real(r8) ps         ! Reference pressure (mb)
   real(r8) t0         ! Reference temperature (freezing point of water)
   real(r8) term1      ! Intermediate scratch variable for es over ice
   real(r8) term2      ! Intermediate scratch variable for es over ice
   real(r8) term3      ! Intermediate scratch variable for es over ice
   real(r8) tr         ! Transition range for es over water to es over ice
   real(r8) ts         ! Reference temperature (boiling point of water)
   real(r8) weight     ! Intermediate scratch variable for es transition
   integer itypo   ! Intermediate scratch variable for holding itype
!
!-----------------------------------------------------------------------
!
! Check on whether there is to be a transition region for es
!
!   print*,  't=',t
   if (itype < 0) then
      tr    = abs(real(itype,r8))
      itypo = itype
      itype = 1
   else
      tr    = 0.0_r8
      itypo = itype
   end if
   	
!   if (tr > 40.0_r8) then
!      write(iulog,900) tr
!!      call endrun ('gffgch2')                ! Abnormal termination
!   end if
!!

!   print*, '2'
   if(t < (tmelt - tr) .and. itype == 1) go to 10
!
! Water
!
   ps = 1013.246_r8
   ts = 373.16_r8
   e1 = 11.344_r8*(1.0_r8 - t/ts)
   e2 = -3.49149_r8*(ts/t - 1.0_r8)
   f1 = -7.90298_r8*(ts/t - 1.0_r8)
   f2 = 5.02808_r8*log10(ts/t)
   f3 = -1.3816_r8*(10.0_r8**e1 - 1.0_r8)/10000000.0_r8
   f4 = 8.1328_r8*(10.0_r8**e2 - 1.0_r8)/1000.0_r8
   f5 = log10(ps)
   f  = f1 + f2 + f3 + f4 + f5
   es = (10.0_r8**f)*100.0_r8
   eswtr = es
   
!   print*, '3'
!
   if(t >= tmelt .or. itype == 0) go to 20
!
! Ice
!
10 continue

!   print*, '4'
   
   t0    = tmelt
   term1 = 2.01889049_r8/(t0/t)
   term2 = 3.56654_r8*log(t0/t)
   term3 = 20.947031_r8*(t0/t)
   es    = 575.185606e10_r8*exp(-(term1 + term2 + term3))
!
   if (t < (tmelt - tr)) go to 20
!
! Weighted transition between water and ice
!
   weight = min((tmelt - t)/tr,1.0_r8)
   es = weight*es + (1.0_r8 - weight)*eswtr
!
20 continue
  ! itype = itypo
   
!   print*, '5'
   return
!
!900 format('gffgch2: FATAL ERROR ******************************',/, &
!           'TRANSITION RANGE FOR WATER TO ICE SATURATION VAPOR', &
!           ' PRESSURE, TR, EXCEEDS MAXIMUM ALLOWABLE VALUE OF', &
!           ' 40.0 DEGREES C',/, ' TR = ',f7.2)
!
end subroutine gffgch2

        
        end module  um_contrail_form 