
c=============================================================================
c
c $Id: wetdep_update.F,v 1.28 2003/06/04 00:52:39 jrt Exp $
c
c CODE DEVELOPER
c   Dan Bergmann, LLNL
c   dbergmann@llnl.gov
c
c FILE
c   wetdep_update.F
c
c ROUTINES
c   Update_Wetdep
c   Calc_Wet_Loss_Rate
c   E_Ice
c   Update_Wetdep_Orig_LLNL
c
c=============================================================================


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Update_Wetdep
c
c DESCRIPTION
c   This routine calculates both in cloud scavenging and below cloud washout,
c   based on a writeup by Daniel Jacob at Harvard.  This is loosely based on
c   Giorgi and Chameides JGR 91 14,367-14376, 1986.
c
c ARGUMENTS
c   pr_wet_depos       : are wet depositions be written to the const output
c                        file?
c   metdata_name_org   : first  part of metdata_name, e.g., "NCAR"
c   metdata_name_model : second part of metdata_name, e.g., "MATCH"
c   chem_opt     : chemistry option
c   ih2o2_num    : const array index for H2O2
c   ihno3_num    : const array index for HNO3
c   tdt          : model time step  (s)
c   mw           : array of species' molecular weights (g/mol)
c   coscen       : cosine of latitude of zone centers = cos(dlatr)
c   con_precip   : convective precipitation (mm/day)
c   tot_precip   : total      precipitation (mm/day)
c   mcor         : area of grid box (m^2)
c   grid_height  : grid box height  (m)
c   mass         : total mass of the atmosphere within each grid box   (kg)
c   moistq       : moisture changes due to wet processes (g/kg/day)
c   rain         : rainfall across cell edges (mm/day)
c   kel          : temperature (degK)
c   press3c      : atmospheric pressure at the center of each grid box (mb)
c   press3e      : atmospheric pressure at the edge   of each grid box (mb)
c   const        : species concentration, known at zone centers (mixing ratio)
c   wet_depos    : wet deposition accumulated since last output (kg/m^2)
c   humidity     : specific humidity (g/kg)
c   REL_SCAV_EFF_new
c                : relative scavenging efficiency for aerosols (0-1) ! from convection subroutine
c
c-----------------------------------------------------------------------------

      subroutine Update_Wetdep
!    &  (pr_wet_depos, pr_wet_depostype, metdata_name_org, metdata_name_model, chem_opt,
     &  (ih2o2_num, ihno3_num, tdt, mw, coscen, con_precip, tot_precip,
     &   mcor, grid_height, mass, moistq, kel, press3c, press3e,
     &   const, wet_depos_strwash, wet_depos_str, humidity, REL_SCAV_EFF_new)

      use h_imp_phys_constants
      use h_imp_time_constants
      use h_imp_dims
      use h_imp_aerosol

      use h_setkin_par
      use h_setkin_depos
      use h_umaerosol

      implicit none
 
!#     include "imp_phys_constants.h"
!#     include "imp_time_constants.h"
!#     include "imp_dims.h"
!#     include "imp_aerosol.h"
!cc?   Tight coupling to setkin?
!#     include "setkin_par.h"
!#     include "setkin_depos.h"
!#     include "umaerosol.h"


c     ----------------------
c     Argument declarations.
c     ----------------------
 
!      logical      :: pr_wet_depos
!      logical      :: pr_wet_depostype
!      character (len=16) :: metdata_name_org
!      character (len=16) :: metdata_name_model
      integer :: chem_opt
      
      integer :: ih2o2_num
      integer :: ihno3_num
      real*8  :: tdt
      real*8  :: mw(num_species)
      real*8  :: coscen     (i1:i2,   ju1:j2)
      real*8  :: con_precip (i1:i2,   ju1:j2)
      real*8  :: tot_precip (i1:i2,   ju1:j2)
      real*8  :: mcor       (i1:i2,   ju1:j2)
      real*8  :: grid_height(i1:i2,   ju1:j2,   k1:k2)
      real*8  :: mass       (i1:i2,   ju1:j2,   k1:k2)
      real*8  :: moistq     (i1:i2,   ju1:j2,   k1:k2)
!      real*8  :: rain       (i1:i2,   ju1:j2,   k1:k2)
      real*8  :: kel        (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: press3c    (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: press3e    (ilo:ihi, julo:jhi, k1-1:k2)
      real*8  :: const      (i1:i2,   ju1:j2,   k1:k2, num_species)
      real*8  :: wet_depos_strwash  (i1:i2,   ju1:j2,   1:num_species)
      real*8  :: wet_depos_str  (i1:i2,   ju1:j2,   1:num_species)
      real*8  :: humidity   (i1:i2,   ju1:j2,   k1:k2)
      real*8  :: REL_SCAV_EFF_new(i1:i2, ju1:j2, k1:k2, num_species)


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter :: CM2PM2    = CMPM * CMPM  ! cm^2/m^2

      real*8, parameter :: CMPMM     = 0.1d0        ! cm/mm

      real*8, parameter :: WATER_DEN = 1.0d0        ! gr/cm^3


c     ----------------------
c     Function declarations.
c     ----------------------

      logical, external :: Is_Fixed_Const


c     ----------------------
c     Variable declarations.
c     ----------------------

c     ---------------------------------------------------------
c     rainout : logical array pointing to locations for rainout
c     washout : logical array pointing to locations for washout
c     ---------------------------------------------------------

      logical :: rainout(i1:i2, ju1:j2)
      logical :: washout(i1:i2, ju1:j2)

      logical, save :: first = .true.

      integer :: il, ij, ik, ic
      integer :: inum

      real*8  :: f_max
      real*8  :: k_con, k_min
      real*8  :: l_plus_w_con, l_plus_w_str
      real*8  :: min_dtotau_1
      real*8  :: tau_con
   
c     ---------------------------------------------------------------------
c     conv_frac       : fraction of precipitation from convective processes
c     exp_fac         : temporary storage location
c     frac            : temporary storage location
c     frac_area_con   : fraction of grid box and above with convective
c                       precipitation
c     frac_area_str   : fraction of grid box and above with stratiform
c                       precipitation
c     frac_i_liq      : equilibrium fractionation of gas into rainwater
c     frac_not_evap   : fraction of precipitaion NOT evaporated in grid box
c     frac_wash_con   : washout fraction from convective precipitation
c     frac_wash_str   : washout fraction from stratiform precipitation
c     henry_eff       : effective Henry's law constant (mole/atm)
c     kloss           : species and temperature dependent wet scavenging
c                       rate constant (s^-1)
c     kloss_con       : kloss in convective clouds
c     kloss_str       : kloss in stratiform clouds
c     mbot            : mass of scaveged tracer through bottom of box (kg)
c     precip_bottom   : precipitation through bottom of box (g/cm^2/s)
c     precip_top      : precipitation through top    of box (g/cm^2/s)
c     q               : rate of evaporation (negative means condensation) 
c                       (g/cm^3/s)
c     --------------------------------------------------------------------

      real*8  :: conv_frac      (i1:i2, ju1:j2)
      real*8  :: exp_fac        (i1:i2, ju1:j2)
      real*8  :: frac           (i1:i2, ju1:j2)
      real*8  :: frac_area_con  (i1:i2, ju1:j2)
      real*8  :: frac_area_str  (i1:i2, ju1:j2)
      real*8  :: frac_i_liq     (i1:i2, ju1:j2)
      real*8  :: frac_not_evap  (i1:i2, ju1:j2)
      real*8  :: frac_wash_con  (i1:i2, ju1:j2)
      real*8  :: frac_wash_str  (i1:i2, ju1:j2)
      real*8  :: henry_eff      (i1:i2, ju1:j2)
      real*8  :: kloss          (i1:i2, ju1:j2)
      real*8  :: kloss_con      (i1:i2, ju1:j2)
      real*8  :: kloss_str      (i1:i2, ju1:j2)
      real*8  :: mbot           (i1:i2, ju1:j2)
      real*8  :: precip_bot     (i1:i2, ju1:j2)
      real*8  :: precip_top     (i1:i2, ju1:j2)
      real*8  :: q              (i1:i2, ju1:j2)

c     --------------------------------------------------------------
c     scav_tracer_con : amount of scavenged tracer due to convective 
c                       processes (kg)
c     scav_tracer_str : amount of scavenged tracer due to stratiform 
c                       processes (kg)
c     --------------------------------------------------------------

      real*8  :: scav_tracer_con(i1:i2, ju1:j2, 1:num_species)
      real*8  :: scav_tracer_str(i1:i2, ju1:j2, 1:num_species)
      real*8  :: scav_tracer_interstitial(i1:i2, ju1:j2, 1:num_species) !Cheng, to store washed out by large scale precip
      
c     --------------------
c     adding for umaerosol
c     --------------------

      real*8  :: relhume   (i1:i2, ju1:j2)
      real*8  :: h2osat    (i1:i2, ju1:j2)
      real*8  :: h2ogas    (i1:i2, ju1:j2)
      real*8  :: so4mfrac  (i1:i2, ju1:j2)
      real*8  :: so4dens   (i1:i2, ju1:j2)
      real*8  :: wetmas    (i1:i2, ju1:j2)
      real*8  :: so4aer    (i1:i2, ju1:j2, naer)
      real*8  :: so4radv   (i1:i2, ju1:j2, nso4)
      real*8  :: so4radg   (i1:i2, ju1:j2, nso4)

      real*8  :: WASH_COEFF       (i1:i2, ju1:j2, num_species)
      real*8  :: frac_wash_str_new(i1:i2, ju1:j2)
      real*8  :: frac_wash_con_new(i1:i2, ju1:j2)


c     ---------------------------------------------------------------
c     do_conv_scav : whether or not do the wet savenge due to the convective preicipitation.
c                    The wet scavenge due to the convective precipiation is to simulate 
c                    the wet scavenge due to  precipitation from the anvil. In recent models (CAM5 or AM3), the precipiation 
c                    from the avil (detrainment from convection updraft) is part of the large scale precipiation.  So 
c                    this type of wet scavege is not necessary.  
c                    Added by Cheng Zhou , 2012/06/10
c     wet_deposition inside convective updrafts is done in the convection module.
c     ---------------------------------------------------------------

c     ----------------
c     Begin execution.
c     ----------------

      if (pr_diag) then
        Write (6,*) 'Update_Wetdep called by ', loc_proc
      end if


c     ==========
      if (first) then
c     ==========

        first = .false.

c       if (pr_wet_depos) wet_depos(i1:i2,ju1:j2,1:num_species) = 0.0d0

      end if


      henry_eff (:,:) = 0.0d0
      kloss     (:,:) = 0.0d0
      kloss_con (:,:) = 0.0d0
      kloss_str (:,:) = 0.0d0
      precip_bot(:,:) = 0.0d0

      scav_tracer_con(:,:,:) = 0.0d0
      scav_tracer_str(:,:,:) = 0.0d0
      scav_tracer_interstitial(:,:,:) = 0.0d0

c     -----------------------------------------------------------------
c     Find the wet scavenged fraction using a first order rainout
c     parameterization from Giorgi and Chameides
c     (JGR v91 pp14,367-14376 1986).  Use this for both the large-scale
c     and the convective precipitation.
c     -----------------------------------------------------------------

ccc!? For now just hardwire a few values for the radon/lead problem
cc    (chem_opt = 1).  Setkin will eventually provide this information
cc    for all species when doing a full chemistry calculation.
      chem_opt = 8
      if (chem_opt == 1) then

        aerosol(1) = 0
        aerosol(2) = 1
  
        hstar(1)   = 9.3d-3
        hstar(2)   = 0.0d0
  
        delh_298_over_r(1) = 2600.0d0
        delh_298_over_r(2) = 0.0d0

        retention_eff(1)   = 0.0d0
        retention_eff(2)   = 0.0d0

      else if (chem_opt == 6) then

        aerosol(1) = 15
        aerosol(2) = 15

        hstar(1)   = 0.0d0
        hstar(2)   = 0.0d0

        delh_298_over_r(1) = 0.0d0
        delh_298_over_r(2) = 0.0d0

        retention_eff(1)   = 0.0d0
        retention_eff(2)   = 0.0d0

      else if (chem_opt == 8) then

        inum = IFSO2
        hstar(inum) = 600.0d0

        inum = INSO2
        hstar(inum) = 600.0d0

      end if


!      if ((metdata_name_org(1:4) == 'GISS') .or.
!     &    ((metdata_name_org  (1:4) == 'NCAR' ) .and.
!     &     (metdata_name_model(1:5) == 'MATCH'))) then
!
!c       -------------------------------------------------------------------
!c       moistq (moisture change) needs to be calculated from the difference
!c       in the rain variable across the grid box.  The units need to be
!c       changed from mm/day to g/kg/day.
!c       -------------------------------------------------------------------
!
!        moistq(:,:,k1:k2-1) =
!     &    (rain(:,:,k1+1:k2) - rain(:,:,k1:k2-1)) *
!     &    Spread (mcor(:,:), 3, ivert-1) * CM2PM2 * CMPMM * WATER_DEN /
!     &    mass(:,:,k1:k2-1) 
!
!        moistq(:,:,k2) = 0.0d0
!
!      end if

c     --------------------------------------------------------------------
c     Calculate fraction of moistq which is associated with convection.
c     If there is no precipitation at the surface, set the fraction
c     equal to the cosine of the latitude.  This is done because 
c     occasionally the data has negative values of moistq (condensation),
c     but has no precipitation.  Also, there are some places where convec-
c     tive precipitation is slightly larger than the total precipitation??
c     --------------------------------------------------------------------

      where ((tot_precip(:,:) > 1.0d-5) .and.
     &       (con_precip(:,:) > 1.0d-5))

        conv_frac(:,:) = con_precip(:,:) / tot_precip(:,:)

      elsewhere

        conv_frac(:,:) = coscen(:,:)
        
      end where

      where (conv_frac(:,:) > 1.0d0) conv_frac(:,:) = 1.0d0
      

c     --------------------------------------------------------
c     First do the first-order rainout for large-scale clouds.
c     --------------------------------------------------------

c     -----------------------------------------------------
c     Calculate the fractional cloud coverage
c     (negative moistq means condensation).
c     Calculate a value of q for each level (g H2O/cm^3/s).
c     -----------------------------------------------------

      f_max        = 0.3d0

      k_con        = 1.5d-3
      k_min        = 1.0d-4

      l_plus_w_con = 2.0d-6
      l_plus_w_str = 0.5d-6      ! change back to 1.5d-6 from Daniel Jacob 2000, xhliu used 0.5d-6, Cheng Zhou

      tau_con      = 1800.0d0
      min_dtotau_1 = Min ((tdt / tau_con), 1.0d0)

      frac_area_con(:,:) = 1.0d-20
      frac_area_str(:,:) = 1.0d-20


c     =======
      ikloop: do ik = k2, k1, -1
c     =======

        q(:,:) =
     &    -moistq(:,:,ik) *
     &    (press3c(i1:i2,ju1:j2,ik) * MWTAIR * MB2CGS) /
     &    (GPKG * SECPDY * kel(i1:i2,ju1:j2,ik) * BOLTZMN_E * AVOGAD)

c       ---------------------------------------------------------------------
c       Store the precipitation from the bottom of the previous box into
c       the top of the current box.  Calculate new precipitation value for
c       the bottom of the current box.  These will be used later for washout.
c       ---------------------------------------------------------------------

        precip_top(:,:) = precip_bot(:,:)

        precip_bot(:,:) =
     &    Max (0.0d0,
     &         precip_bot(:,:) -
     &         (moistq(:,:,ik) * mass(:,:,ik) /
     &          (SECPDY * mcor(i1:i2,ju1:j2) * CM2PM2)))

c       ---------------------------------------------------------------------
c       Calculate fractional area of convective and stratiform precipitation.
c       This is equation no. 13 in Daniel Jacob's scavenging paper.
c       ---------------------------------------------------------------------

        where (q(:,:) > 0.0d0)

          frac_area_con(:,:) =
     &      Max (f_max*q(:,:) * conv_frac(:,:) * min_dtotau_1 /
     &           (q(:,:) * conv_frac(:,:) * min_dtotau_1 +
     &            f_max*k_con * l_plus_w_con),
     &           frac_area_con(:,:))

          frac_area_str(:,:) =
     &      Max (q(:,:) * (1.0d0 - conv_frac(:,:)) /
     &           ((k_min + q(:,:) * (1.0d0 - conv_frac(:,:)) /
     &             l_plus_w_str) *
     &            l_plus_w_str),
     &           frac_area_str(:,:))

        end where

c       ---------------------------------------------------------
c       Calculate some fractional washout terms which may be used 
c       regardless of species.
c       ---------------------------------------------------------

        where (frac_area_str(:,:) > 0.0d0)

          frac_wash_str(:,:) =
     &      frac_area_str(:,:) *
     &      (1.0d0 - Exp (-precip_bot(:,:) * (1.0d0 - conv_frac(:,:)) *
     &       tdt / frac_area_str(:,:)))

        elsewhere

          frac_wash_str(:,:) = 0.0d0

        end where

        where (frac_area_con(:,:) > 0.0d0)

          frac_wash_con(:,:) =
     &      frac_area_con(:,:) *
     &      (1.0d0 - Exp (-precip_bot(:,:) * conv_frac(:,:) * 
     &       tdt / frac_area_con(:,:)))

        elsewhere

          frac_wash_con(:,:) = 0.0d0

        end where

c       ------------------------------------------------------      
c       Create an array showing where washout occurs.
c       It only occurs where there is no precipitation forming
c       and the temperature is above 268 K.
c       ------------------------------------------------------

c Cheng, 2013/11/14, now extend below cloud washout to ice clouds 
        washout(:,:) = ((moistq(:,:,ik)        >=   0.0d0) .and.
c     &                  (kel (i1:i2,ju1:j2,ik) >= 268.0d0) .and.
     &                  (precip_top(:,:)       >    0.0d0))


c       ----------------------------------------------------
c       Create an array showing where rainout occurs.
c       It only occurs where there is precipitation forming.
c       ----------------------------------------------------

        rainout(:,:) = (moistq(:,:,ik) < -1.0d-10)

c       ------------------------------------------------------------------
c       First do the reevaporation by calculating fraction NOT evaporated.
c       ------------------------------------------------------------------

        where (washout(:,:))                               

          frac_not_evap(:,:) =
     &      1.0d0 - 
     &      ((precip_top(:,:) - precip_bot(:,:)) / precip_top(:,:)) *
     &      0.5d0

        elsewhere

          frac_not_evap(:,:) = 1.0d0

        end where


cc      --------------------
cc      adding for umaerosol
cc      --------------------

cc      compute relative humidity (0-1)

        relhume(:,:) = 1.0d0 - (373.15d0 / kel(i1:i2,ju1:j2,ik))
        relhume(:,:) =
     &    1013.25d0 * Exp (13.3185d0 * relhume(:,:)    -
     &                      1.9760d0 * relhume(:,:)**2 -
     &                      0.6445d0 * relhume(:,:)**3 -
     &                      0.1299d0 * relhume(:,:)**4)

        relhume(:,:) =
     &    humidity(:,:,ik) * MWTAIR / 18.0d0 /
     &    GPKG * press3c(i1:i2,ju1:j2,ik) / relhume(:,:)

        relhume(:,:) = Max (Min (relhume(:,:), 0.95d0), 0.0d0)
 
cc      compute so4 aerosol size (m)

        so4aer(:,:,1) = const(:,:,ik,ISO4M1)
        so4aer(:,:,2) = const(:,:,ik,ISO4N1)
        so4aer(:,:,3) = const(:,:,ik,ISO4M2)
        so4aer(:,:,4) = const(:,:,ik,ISO4N2)
        so4aer(:,:,5) = const(:,:,ik,ISO4M3)
        so4aer(:,:,6) = const(:,:,ik,ISO4N3)

        do il = i1, i2
          do ij = ju1, j2
            h2osat(il,ij) = h2osat_f(kel(il,ij,ik))
            h2ogas(il,ij) = relhume(il,ij)*h2osat(il,ij)

            so4mfrac(il,ij) = so4mfrac_f(kel(il,ij,ik),
     &                                   h2osat(il,ij),
     &                                   h2ogas(il,ij))
            so4dens(il,ij) = so4dens_f(kel(il,ij,ik),
     &                                 so4mfrac(il,ij))
          end do
        end do

        do iso4 = 1, nso4

          iso4n = iso4 * nmomso4
          iso4m = iso4n - 1

          wetmas(:,:) = max(r2so4min, so4aer(:,:,iso4m)
     &                / max(epsilo,   so4aer(:,:,iso4n)))
     &                / so4mfrac(:,:)

          so4radv(:,:,iso4) = (r3q * wetmas(:,:)
     &                      / (IMP_PI * so4dens(:,:)))**r1td
          so4radv(:,:,iso4) = max(so4radvmin,
     &                        min(so4radv(:,:,iso4),so4radvmax))

        end do

cc      calculate washout coefficients (cm^-1) of so4 aerosol

        WASH_COEFF(:,:,:) = 0.0d0
        rg(:) = rg(:) * 1.0d-6         ! um -> meter

        do il = i1, i2
          do ij = ju1, j2

            do iso4 = 1, nso4

              xlnsg = log(sigmod(iso4))
              xlnsg2 = xlnsg * xlnsg

              so4radg(il,ij,iso4) = so4radv(il,ij,iso4)
     &                            * exp(-r3h*xlnsg2)

              if (so4radg(il,ij,iso4) <= rg(1)) then

                if (iso4 == 1) then
                  WASH_COEFF(il,ij,ISO4M1) = k1_q3(1)
                  WASH_COEFF(il,ij,ISO4N1) = k0_q0(1)
                else if (iso4 == 2) then
                  WASH_COEFF(il,ij,ISO4M2) = k1_q3(1)
                  WASH_COEFF(il,ij,ISO4N2) = k0_q0(1)
                else if (iso4 == 3) then
                  WASH_COEFF(il,ij,ISO4M3) = k2_q3(1)
                  WASH_COEFF(il,ij,ISO4N3) = k0_q0(1)
                end if                 

              else if (so4radg(il,ij,iso4) >= rg(nrg)) then

                if (iso4 == 1) then
                  WASH_COEFF(il,ij,ISO4M1) = k1_q3(nrg)
                  WASH_COEFF(il,ij,ISO4N1) = k0_q0(nrg)
                else if (iso4 == 2) then
                  WASH_COEFF(il,ij,ISO4M2) = k1_q3(nrg)
                  WASH_COEFF(il,ij,ISO4N2) = k0_q0(nrg)
                else if (iso4 == 3) then
                  WASH_COEFF(il,ij,ISO4M3) = k2_q3(nrg)
                  WASH_COEFF(il,ij,ISO4N3) = k0_q0(nrg)
                end if

              else

                irg = 1
                do while((rg(irg) < so4radg(il,ij,iso4))
     &                 .and. (irg < nrg))
                  irg = irg + 1
                end do

                if (iso4 == 1) then
                  WASH_COEFF(il,ij,ISO4M1) =
     &                             k1_q3(irg-1)
     &                           + (k1_q3(irg)-k1_q3(irg-1))
     &                           / (rg(irg)-rg(irg-1))
     &                           * (so4radg(il,ij,iso4)-rg(irg-1))
                  WASH_COEFF(il,ij,ISO4N1) =
     &                             k0_q0(irg-1)
     &                           + (k0_q0(irg)-k0_q0(irg-1))
     &                           / (rg(irg)-rg(irg-1))
     &                           * (so4radg(il,ij,iso4)-rg(irg-1))
                else if (iso4 == 2) then
                  WASH_COEFF(il,ij,ISO4M2) =
     &                             k1_q3(irg-1)
     &                           + (k1_q3(irg)-k1_q3(irg-1))
     &                           / (rg(irg)-rg(irg-1))
     &                           * (so4radg(il,ij,iso4)-rg(irg-1))
                  WASH_COEFF(il,ij,ISO4N2) =
     &                             k0_q0(irg-1)
     &                           + (k0_q0(irg)-k0_q0(irg-1))
     &                           / (rg(irg)-rg(irg-1))
     &                           * (so4radg(il,ij,iso4)-rg(irg-1))
                else if (iso4 == 3) then
                  WASH_COEFF(il,ij,ISO4M3) =
     &                             k2_q3(irg-1)
     &                           + (k2_q3(irg)-k2_q3(irg-1))
     &                           / (rg(irg)-rg(irg-1))
     &                           * (so4radg(il,ij,iso4)-rg(irg-1))
                  WASH_COEFF(il,ij,ISO4N3) =
     &                             k0_q0(irg-1)
     &                           + (k0_q0(irg)-k0_q0(irg-1))
     &                           / (rg(irg)-rg(irg-1))
     &                           * (so4radg(il,ij,iso4)-rg(irg-1))
                end if

              end if

            end do
          end do
        end do

        WASH_COEFF(:,:,:) = 10.0d0 * WASH_COEFF(:,:,:) ! mm-1 -> cm-1


c       =======
        icloop: do ic = 1, num_species
c       =======

c         ====================================
          if (Is_Fixed_Const(ic)) cycle icloop
c         ====================================

          if (ic == ihno3_num .or. ic == ISO4G) then

            where (q(:,:) > 0.0d0)
              kloss_con(:,:) = k_con
              kloss_str(:,:) = k_min +
     &          q(:,:) * (1.0d0 - conv_frac(:,:)) / l_plus_w_str
            elsewhere
              kloss_con(:,:) = 0.0d0
              kloss_str(:,:) = 0.0d0
            end where

          else if (aerosol(ic) >= 1) then

            where (q(:,:) > 0.0d0)
              kloss_con(:,:) = k_con * REL_SCAV_EFF_new(:,:,ik,ic)
              kloss_str(:,:) = (k_min +
     &          q(:,:) * (1.0d0 - conv_frac(:,:)) / l_plus_w_str)
     &          * REL_SCAV_EFF_new(:,:,ik,ic)
            elsewhere
              kloss_con(:,:) = 0.0d0
              kloss_str(:,:) = 0.0d0
            end where

          else if (hstar(ic) > 0.0d0) then

            do ij = ju1, j2

c             =======================
              call Calc_Wet_Loss_Rate 
c             =======================
     &          (ic, ih2o2_num, delh_298_over_r(ic), hstar(ic),
     &           retention_eff(ic), press3e(:,ij,ik), 
     &           kel(:,ij,ik), kloss(:,ij))
           
            end do
 
            where (q(:,:) > 0.0d0)
              kloss_con(:,:) = k_con * kloss(:,:)
              kloss_str(:,:) = (k_min +
     &          q(:,:) * (1.0d0 - conv_frac(:,:)) / l_plus_w_str)
     &          * kloss(:,:)
            elsewhere
              kloss_con(:,:) = 0.0d0
              kloss_str(:,:) = 0.0d0
            end where


          else 

            kloss_con(:,:) = 0.0d0
            kloss_str(:,:) = 0.0d0

          end if

c         ----------------------------------------------------------
c         Calculate some fractional washout terms related to species
c         ----------------------------------------------------------

          if (aerosol(ic) == 1 .or. aerosol(ic) == 2 .or.
     &        aerosol(ic) == 3) then

            where (frac_area_str(:,:) > 0.0d0)

              frac_wash_str_new(:,:) =
     &          frac_area_str(:,:) *
     &          (1.0d0 - Exp (-precip_bot(:,:) *
     &          (1.0d0 - conv_frac(:,:)) * WASH_COEFF(:,:,ic)
     &           * tdt / frac_area_str(:,:)))

            elsewhere

              frac_wash_str_new(:,:) = 0.0d0

            end where

            where (frac_area_con(:,:) > 0.0d0)

              frac_wash_con_new(:,:) =
     &          frac_area_con(:,:) *
     &          (1.0d0 - Exp (-precip_bot(:,:)
     &          * conv_frac(:,:) * WASH_COEFF(:,:,ic)
     &          * tdt / frac_area_con(:,:)))

            elsewhere

              frac_wash_con_new(:,:) = 0.0d0

            end where

          else

            frac_wash_str_new(:,:) = frac_wash_str(:,:)
            frac_wash_con_new(:,:) = frac_wash_con(:,:)

          end if


c         -----------------------------------------------------------
c         Only apply rainout in grid boxes where precipitation forms.
c         -----------------------------------------------------------

          where (rainout(:,:))

c           --------------------------
c           Do the stratiform rainout.
c           --------------------------

            exp_fac(:,:) = Exp (-kloss_str(:,:) * tdt)

            scav_tracer_str(:,:,ic) =
     &        scav_tracer_str(:,:,ic) + 
     &        frac_area_str(:,:) * (1.0d0 - exp_fac(:,:)) *
     &        const(:,:,ik,ic) * mass(:,:,ik) *
     &        mw(ic) / MWTAIR

            const(:,:,ik,ic) =
     &        const(:,:,ik,ic) *
     &        (1.0d0 - 
cc   &         (frac_area_str(:,:) * (1.0d0 - Exp (-kloss(:,:) * tdt))))
     &         (frac_area_str(:,:) * (1.0d0 - exp_fac(:,:))))


c           --------------------------
c           Do the convective rainout.
c           --------------------------
c     comment out the wet scavenge by the convective precipitation by Cheng Zhou
c            exp_fac(:,:) = Exp (-kloss_con(:,:) * tdt)
c
c            scav_tracer_con(:,:,ic) =
c     &        scav_tracer_con(:,:,ic) + 
c     &        frac_area_con(:,:) * (1.0d0 - exp_fac(:,:)) *
c     &        const(:,:,ik,ic) * mass(:,:,ik) *
c     &        mw(ic) / MWTAIR
c
c            const(:,:,ik,ic) =
c     &        const(:,:,ik,ic) *
c     &        (1.0d0 - 
c     &         (frac_area_con(:,:) * (1.0d0 - exp_fac(:,:))))
c     end comment out wet scavenge by the convective precipitation by Cheng Zhou       
           
          end where

c         -------------------------------------------------------------
c         Do the washout and reevaporation of aerosols and nitric acid.
c         -------------------------------------------------------------

          if ((aerosol(ic) >= 1) .or. (ic == ihno3_num) .or.
     &        (ic == ISO4G)) then

            if ((ic == ISO4G)  .or. (ic == ISO4M1) .or.
     &          (ic == ISO4M2) .or. (ic == ISO4M3)) then

              where (washout(:,:))
                const(:,:,ik,ISO4M3) =
     &            const(:,:,ik,ISO4M3) +
     &            (1.0d0 - frac_not_evap(:,:)) *
     &            (scav_tracer_str(:,:,ic) + scav_tracer_con(:,:,ic)) *
     &            (MWTAIR / mw(ic)) / mass(:,:,ik)
              end where

            else if ((ic == ISO4N1) .or. (ic == ISO4N2) .or.
     &               (ic == ISO4N3)) then

              where (washout(:,:))
c               const(:,:,ik,ISO4N2) =
c    &            const(:,:,ik,ISO4N2) +
c    &            (1.0d0 - frac_not_evap(:,:)) *
c    &            (scav_tracer_str(:,:,ic-1) +
c    &             scav_tracer_con(:,:,ic-1)) *
c    &            (MWTAIR / mw(ic)) / mass(:,:,ik) *
c    &            const(:,:,ik,ISO4N2) / const(:,:,ik,ISO4M2)
                const(:,:,ik,ISO4N1) =
     &            const(:,:,ik,ISO4N1)
                const(:,:,ik,ISO4N2) =
     &            const(:,:,ik,ISO4N2)
                const(:,:,ik,ISO4N3) =
     &            const(:,:,ik,ISO4N3)
              end where

            else

              where (washout(:,:))
                const(:,:,ik,ic) =
     &            const(:,:,ik,ic) +
     &            (1.0d0 - frac_not_evap(:,:)) *
     &            (scav_tracer_str(:,:,ic) + scav_tracer_con(:,:,ic)) *
     &            (MWTAIR / mw(ic)) / mass(:,:,ik)
              end where

            end if

            where (washout(:,:))                                      

              scav_tracer_str(:,:,ic) =
     &          scav_tracer_str(:,:,ic) * frac_not_evap(:,:)
          
              scav_tracer_con(:,:,ic) =
     &          scav_tracer_con(:,:,ic) * frac_not_evap(:,:)


              scav_tracer_interstitial(:,:,ic) =
     &          scav_tracer_interstitial(:,:,ic) * frac_not_evap(:,:)

c             ------------------------------
c             Now do the stratiform washout.
c             ------------------------------

              scav_tracer_str(:,:,ic) =
     &          scav_tracer_str(:,:,ic) +
     &          frac_wash_str_new(:,:) *
     &          const(:,:,ik,ic) * mass(:,:,ik) *
     &          mw(ic) / MWTAIR

c  interstitial asersols being washed out.         
              scav_tracer_interstitial(:,:,ic) =
     &          scav_tracer_interstitial(:,:,ic) +
     &          frac_wash_str_new(:,:) *
     &          const(:,:,ik,ic) * mass(:,:,ik) *
     &          mw(ic) / MWTAIR
     
              const(:,:,ik,ic) =
     &          const(:,:,ik,ic) * (1.0d0 - frac_wash_str_new(:,:)) 

       
c             ------------------------------
c             Now do the convective washout.
c             ------------------------------

c     comment out the wet scavenge by the convective precipitation by Cheng Zhou
              scav_tracer_con(:,:,ic) =
     &          scav_tracer_con(:,:,ic) +
     &          frac_wash_con_new(:,:) *
     &          const(:,:,ik,ic) * mass(:,:,ik) *
     &          mw(ic) / MWTAIR

              const(:,:,ik,ic) =
     &          const(:,:,ik,ic) * (1.0d0 - frac_wash_con_new(:,:))
c     end. comment out the wet scavenge by the convective precipitation by Cheng Zhou              
            end where

          else

c           ---------------------------------------------
c           Do washout and reevaporation of species other 
c           than aerosols or nitric acid.
c           ---------------------------------------------
 
            henry_eff(:,:) =
     &        hstar(ic) *
     &        Exp (-delh_298_over_r(ic) * 
     &             ((1.0d0 / kel(i1:i2,ju1:j2,ik)) -
     &              (1.0d0 / 298.0d0)))

c           -----------------------------------------------
c           Stratiform or large scale component of washout.
c           -----------------------------------------------

            frac(:,:) =
     &        henry_eff(:,:) * GAS_CONST_J / 100.0d0 * 
     &        kel(i1:i2,ju1:j2,ik) * precip_bot(:,:) *
     &        (1.0d0 - conv_frac(:,:)) * tdt /
     &        (frac_area_str(:,:) * (grid_height(:,:,ik) * 100.0d0))

            frac_i_liq(:,:) =
     &        frac(:,:) / (1.0d0 + frac(:,:))

c           ---------------------------------------
c           Scavenging is limited by mass transfer.
c           ---------------------------------------

            where ((frac_i_liq(:,:) >= 
     &              (frac_wash_str(:,:) / frac_area_str(:,:))) .and.
     &             washout(:,:))

c             ---------------------------
c             First do the reevaporation.
c             ---------------------------

              const(:,:,ik,ic) =
     &          const(:,:,ik,ic) +
     &          (1.0d0 - frac_not_evap(:,:)) * scav_tracer_str(:,:,ic) *
     &          (MWTAIR / mw(ic)) /  mass(:,:,ik) 

              scav_tracer_str(:,:,ic) =
     &          scav_tracer_str(:,:,ic) * frac_not_evap(:,:)
          
c             ------------------------------
c             Now do the stratiform washout.
c             ------------------------------

              scav_tracer_str(:,:,ic) =
     &          scav_tracer_str(:,:,ic) +
     &          frac_wash_str(:,:) *
     &          const(:,:,ik,ic) * mass(:,:,ik) *
     &          mw(ic) / MWTAIR

c  interstitial asersols being washed out.         
              scav_tracer_interstitial(:,:,ic) =
     &          scav_tracer_interstitial(:,:,ic) +
     &          frac_wash_str(:,:) *
     &          const(:,:,ik,ic) * mass(:,:,ik) *
     &          mw(ic) / MWTAIR
     
              const(:,:,ik,ic) =
     &          const(:,:,ik,ic) * (1.0d0 - frac_wash_str(:,:)) 

       
            end where

c           -------------------------------------------------
c           Scavenging is limited by Henry's law equilibrium.
c           -------------------------------------------------

            where ((frac_i_liq(:,:) < 
     &              (frac_wash_str(:,:) / frac_area_str(:,:))) .and.
     &             washout(:,:))

              mbot(:,:) =
     &          frac_i_liq(:,:) * 
     &          (frac_area_str(:,:) * const(:,:,ik,ic) * 
     &           mass(:,:,ik) * mw(ic) / MWTAIR + 
     &           scav_tracer_str(:,:,ic)) 

              const(:,:,ik,ic) =
     &          const(:,:,ik,ic) -
     &          (mbot(:,:) - scav_tracer_str(:,:,ic)) / 
     &          mass(:,:,ik) * MWTAIR / mw(ic) 

              scav_tracer_str(:,:,ic) = mbot(:,:)

            end where

c           --------------------------------
c           Convective component of washout.
c           --------------------------------
c     comment out the wet scavenge by the convective precipitation by Cheng Zhou
            frac(:,:) =
     &        henry_eff(:,:) * GAS_CONST_J / 100.0d0 * 
     &        kel(i1:i2,ju1:j2,ik) * precip_bot(:,:) *
     &        conv_frac(:,:) * tdt /
     &        (frac_area_con(:,:) * (grid_height(:,:,ik) * 100.0d0))

            frac_i_liq(:,:) =
     &        frac(:,:) / (1.0d0 + frac(:,:))

c           ---------------------------------------
c           Scavenging is limited by mass transfer.
c           ---------------------------------------

            where ((frac_i_liq(:,:) >= 
     &              (frac_wash_con(:,:) / frac_area_con(:,:))) .and.
     &             washout(:,:))

c             ---------------------------
c             First do the reevaporation.
c             ---------------------------

              const(:,:,ik,ic) =
     &          const(:,:,ik,ic) +
     &          (1.0d0 - frac_not_evap(:,:)) * scav_tracer_con(:,:,ic) *
     &          (MWTAIR / mw(ic)) /  mass(:,:,ik) 



              scav_tracer_con(:,:,ic) =
     &          scav_tracer_con(:,:,ic) * frac_not_evap(:,:)
          
c             ------------------------------
c             Now do the convective washout.
c             ------------------------------

              scav_tracer_con(:,:,ic) =
     &          scav_tracer_con(:,:,ic) +
     &          frac_wash_con(:,:) *
     &          const(:,:,ik,ic) * mass(:,:,ik) *
     &          mw(ic) / MWTAIR

              const(:,:,ik,ic) =
     &          const(:,:,ik,ic) * (1.0d0 - frac_wash_con(:,:)) 

            
            end where

c           -------------------------------------------------
c           Scavenging is limited by Henry's law equilibrium.
c           -------------------------------------------------

            where ((frac_i_liq(:,:) < 
     &              (frac_wash_con(:,:) / frac_area_con(:,:))) .and.
     &             washout(:,:))

              mbot(:,:) =
     &          frac_i_liq(:,:) * 
     &          (frac_area_con(:,:) * const(:,:,ik,ic) * 
     &           mass(:,:,ik) * mw(ic) / MWTAIR + 
     &           scav_tracer_con(:,:,ic)) 

              const(:,:,ik,ic) =
     &          const(:,:,ik,ic) -
     &          (mbot(:,:) - scav_tracer_con(:,:,ic)) / 
     &           mass(:,:,ik) * MWTAIR / mw(ic) 

              scav_tracer_con(:,:,ic) = mbot(:,:)

            end where
            
c     end comment out the wet scavenge by the convective precipitation by Cheng Zhou
           
          end if

c         -------------------------------------------------------------
c         If this is the bottom grid box, then put the scavenged amount
c         into the wet_depos array.
c         -------------------------------------------------------------


          if (ik == k1) then
c            wet_depos(:,:,ic) =
c     &        wet_depos(:,:,ic) + 
c     &        scav_tracer_con(:,:,ic) / 
c     &        mcor(:,:)

            wet_depos_strwash(:,:,ic) =
     &        wet_depos_strwash(:,:,ic) + 
     &        scav_tracer_interstitial(:,:,ic) / 
     &        mcor(:,:)  
     
            wet_depos_str(:,:,ic) =
     &        wet_depos_str(:,:,ic) + 
     &        scav_tracer_str(:,:,ic) / 
     &        mcor(:,:) + 
     &        scav_tracer_con(:,:,ic) / 
     &        mcor(:,:)    
          end if
          
c       =============
        end do icloop
c       =============

c     =============
      end do ikloop
c     =============


      return
 
      end subroutine

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Calc_Wet_Loss_Rate
c
c DESCRIPTION
c   This routine calculates the wet loss rate for 1 species at each latitude.
c
c ARGUMENTS
c   ic              : species index for calculating wet loss rate.             
c   ih2o2_num       : index of species H2O2 (zero if not present).
c   delh_298_over_r : temperature scaling term for Henry's law.
c   hstar           : Henry's law constant (no temperature dependence).
c   retention_eff   : retention efficiency
c   presse1d        : atmospheric pressure at the edge   of each grid box (mb)
c   kel1d           : temperature (degK)
c   kloss           : wet loss rate returned from routine (s^-1)
c
c-----------------------------------------------------------------------------

      subroutine Calc_Wet_Loss_Rate
     &  (ic, ih2o2_num, delh_298_over_r, hstar,
     &   retention_eff, presse1d, kel1d, kloss)

      use h_imp_phys_constants
      use h_imp_dims

      implicit none
 
!#     include "imp_phys_constants.h"
!#     include "imp_dims.h"


c     ----------------------
c     Argument declarations.
c     ----------------------
 
      integer :: ic             
      integer :: ih2o2_num
      real*8  :: delh_298_over_r
      real*8  :: hstar
      real*8  :: retention_eff
      real*8  :: presse1d(ilo:ihi)
      real*8  :: kel1d   (ilo:ihi)
      real*8  :: kloss   (i1:i2)


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8, external  :: E_Ice


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: il
   
      real*8  :: sqrt19

c     ---------------------------------------------------------------------
c     ci_ice_over_ci_gas : mixing ratio in ice over mixing ratio in gas
c     ci_liq_over_ci_gas : mixing ratio in liquid over mixing ratio in gas
c     cloud_liq_water    : estimate of cloud liquid water (g/cm^3)
c     henry_eff : effective Henry's law constant (includes temperature
c                 dependence)
c     retention : retention efficiency of tracer in the liquid cloud
c                 condensate as it is converted to precipitation
c                 (retention < 1 accounts for volitilization during riming)
c     ---------------------------------------------------------------------

      real*8  :: ci_ice_over_ci_gas(i1:i2)
      real*8  :: ci_liq_over_ci_gas(i1:i2)
      real*8  :: cloud_liq_water   (i1:i2)
      real*8  :: henry_eff         (i1:i2)
      real*8  :: retention         (i1:i2)


c     ----------------
c     Begin execution.
c     ----------------

c     -------------------------------------------------------------
c     Calculate the wet scavenging species dependent rate constant.
c     -------------------------------------------------------------

      henry_eff(:) =
     &  hstar * Exp (-delh_298_over_r * 
     &  ((1.0d0 / kel1d(i1:i2)) - (1.0d0 / 298.0d0)))

      retention(:) = 0.0d0
 
      where (kel1d(i1:i2) > 248.0d0) 
     &  retention(:) = retention_eff

      where (kel1d(i1:i2) > 268.0d0) 
     &  retention(:) = 1.0d0

      cloud_liq_water(:) = 0.0d0

      where (kel1d(i1:i2) >= 248.0d0) 
     &  cloud_liq_water(:) =
     &    2.0d-6 * (kel1d(i1:i2) - 248.0d0) / 20.0d0

      where (kel1d(i1:i2) >= 268.0d0) 
     &  cloud_liq_water(:) = 2.0d-6
        
      ci_liq_over_ci_gas(:) =
     &  henry_eff(:) * cloud_liq_water(:) *
     &  GAS_CONST_J / 100.0d0 * kel1d(i1:i2)
           
      ci_ice_over_ci_gas(:) =  0.0d0

      if (ic == ih2o2_num) then
 
        sqrt19 = Sqrt (1.9d0)

        do il = i1, i2

          if (kel1d(il) < 273.0d0) 
     &      ci_ice_over_ci_gas(il) = 
     &      (2.0d-6 - cloud_liq_water(il)) * 
     &      presse1d(il) / 
     &      E_Ice (Max (-120.0d0, (kel1d(il) - 273.0d0))) *
cc   &      0.6d0 * Sqrt (1.9d0)
     &      0.6d0 * sqrt19

        end do

      end if
 
      kloss(:) =
     &  (retention(:) * 
     &   ci_liq_over_ci_gas(:) / 
     &   (1.0d0 + ci_liq_over_ci_gas(:) + ci_ice_over_ci_gas(:)) +
     &   ci_ice_over_ci_gas(:) / 
     &   (1.0d0 + ci_liq_over_ci_gas(:) + ci_ice_over_ci_gas(:))) 


      return

      end subroutine


c-----------------------------------------------------------------------------
c
c ROUTINE
c   E_Ice
c
c DESCRIPTION
c   This routine computes the saturation vapor pressure of ice at a given
c   Celsius temperature.
c  
c   NOTE:
c     The saturation vapor pressure of ice is generated by a polynomial
c     formulation from -50 C < T < 0 C.  From -120 C to -50 C, log10
c     interpolations are done in 10 C bins.  This should be good enough for
c     most purposes.
c
c   The polynomial formulation for Eice(T), -50 C <= T <= 0 C, is as follows:
c
c     Eice(T) = a0 + T(a1 + T(a2 + T(a3 + T(a4 + T(a5 + a6T)))))
c
c   with coefficients a0...a6 given above by Lowe & Ficke, 1974.
c
c   The log10 interpolations for temperatures below -50 C are done in 
c   10 degree C bins.  The equation is as follows:
c
c     T0  = Temperature in C at lower bound of temperature bin
c     T1  = Temperature in C at upper bound of temperature bin
c     T   = Temperature for which Eice(T) is desired
c     E0  = Eice(T0) = value of Eice at T0 (obtained from literature)
c     E1  = Eice(T1) = value of Eice at T1 (obtained from literature)
c     E   = Eice(T)  = value of Eice at desired temperature T
c
c                             {  T - T0                              }
c     log10(E) =  log10(E0) + { -------- * [ log10(E1) - log10(E0) ] }
c                             {    10                                }
c
c   The relevant values of T0, T1, E0, E1 for each bin are: 
c
c       Bin      T0       T1      E0 = Eice(T0)    E1 = Eice(T1)     
c     --------------------------------------------------------------------     
c        1     -60 C    -50 C     1.0765e-2 mb     3.947e-2  mb
c        2     -70 C    -60 C     2.577e-3  mb     1.0765e-2 mb
c        3     -80 C    -70 C     5.307e-4  mb     2.577e-3  mb
c        4     -90 C    -80 C     9.17e-5   mb     5.307e-4  mb
c        5    -100 C    -90 C     1.29e-5   mb     9.17e-5   mb
c        6    -110 C   -100 C     1.4e-6    mb     1.29e-5   mb
c        7    -120 C   -110 C     1.0e-7    mb     1.4e-6    mb
c
c   For computational expediency, the following terms were also computed
c   for use in the code below:
c
c       Bin           log10(E0)         [ log10(E1) - log10(E0) ]  
c     --------------------------------------------------------------------     
c        1        -1.96798596584            5.6425309224e-01            
c        2        -2.58888558145            6.2089961561e-01            
c        3        -3.27515091237            6.8626533092e-01 
c        4        -4.03763066433            7.6247975196e-01  
c        5        -4.88941028970            8.5177962537e-01 
c        6        -5.85387196432            9.6446167462e-01
c        7        -7.00000000000            1.14612803568e+00
c      
c ARGUMENTS
c   tc : ambient temperature (degC)
c
c-----------------------------------------------------------------------------

      function E_Ice (tc)

      implicit none

 
c     ----------------------
c     Argument declarations.
c     ----------------------

      real*8, intent(in)  :: tc


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8  :: E_Ice


c     -----------------------
c     Parameter declarations.
c     -----------------------

c     ---------------------------------------------------
c     Parameters for the polynomial (Lowe & Ficke, 1974).
c     ---------------------------------------------------

      real*8, parameter :: A0 = 6.109177956d+00
      real*8, parameter :: A1 = 5.034698970d-01
      real*8, parameter :: A2 = 1.886013408d-02
      real*8, parameter :: A3 = 4.176223716d-04
      real*8, parameter :: A4 = 5.824720280d-06
      real*8, parameter :: A5 = 4.838803174d-08
      real*8, parameter :: A6 = 1.838826904d-10


c     ----------------
c     Begin execution.
c     ----------------

      if ((tc >= -50.0d0) .and. (tc <= 0.0d0)) then

c       -----------------------------------------------------
c       Use a polynomial formulation for the saturation vapor 
c       pressure over ice (Lowe & Ficke, 1974).
c       -----------------------------------------------------

        E_Ice = A0 + tc * (A1 + tc * (A2 + tc * (A3 + 
     &               tc * (A4 + tc * (A5 + A6 * tc)))))

      else if ((tc >= -60.0d0) .and. (tc < -50.0d0)) then

c       -----------------------------
c       Bin 1 => Log10 interpolation.
c       -----------------------------

        E_Ice = -1.96798596584d0 + 
     &          (((tc + 60.0d0) / 10.0d0) * 5.6425309224d-1)

        E_Ice = 10.0d0**E_Ice

      else if ((tc >= -70.0d0) .and. (tc < -60.0d0)) then

c       -----------------------------
c       Bin 2 => Log10 interpolation.
c       -----------------------------

        E_Ice = -2.58888558145d0 +
     &          (((tc + 70.0d0) / 10.0d0) * 6.2089961561d-1) 
                 
        E_Ice = 10.0d0**E_Ice

      else if ((tc >= -80.0d0) .and. (tc < -70.0d0)) then

c       -----------------------------
c       Bin 3 => Log10 interpolation.
c       -----------------------------

        E_Ice = -3.27515091237d0 +
     &          (((tc + 80.0d0) / 10.0d0) * 6.8626533092d-1) 

        E_Ice = 10.0d0**E_Ice

      else if ((tc >= -90.0d0) .and. (tc < -80.0d0)) then

c       -----------------------------
c       Bin 4 => Log10 interpolation.
c       -----------------------------

        E_Ice = -4.03763066433d0 +
     &          (((tc + 90.0d0) / 10.0d0) * 7.6247975196d-1) 

        E_Ice = 10.0d0**E_Ice

      else if((tc >= -100.0d0) .and. (tc < -90.0d0)) then

c       -----------------------------
c       Bin 5 => Log10 interpolation.
c       -----------------------------

        E_Ice =  -4.88941028970d0  + 
     &           (((tc + 100.0d0) / 10.0d0) * 8.5177962537d-1) 

        E_Ice = 10.0d0**E_Ice

      else if ((tc >= -110.0d0) .and. (tc < -100.0d0)) then

c       -----------------------------
c       Bin 6 => Log10 interpolation.
c       -----------------------------

        E_Ice =  -5.85387196432d0 + 
     &           (((tc + 110.0d0) / 10.0d0) * 9.6446167462d-1) 

        E_Ice = 10.0d0**E_Ice

      else if ((tc >= -120.0d0) .and. (tc < -110.0d0)) then

c       -----------------------------
c       Bin 7 => Log10 interpolation.
c       -----------------------------

        E_Ice = -7.0d0 +
     &          (((tc + 120.0d0) / 10.0d0) * 1.14612803568d0) 

        E_Ice = 10.0d0**E_Ice

      else

c       -----------------------------------------
c       Below -120 C, stop with an error message.
c       -----------------------------------------

        Write (6, '(''Temperature: '', f10.5, '' C'')') tc
        Write (6, '(''T must be in the range -120 C <= T <= 0C'')')
        Write (6, '(''STOPPING in E_Ice'')')
        stop  'Temperature is < -120C in E_Ice'

      end if


      return

      end function



!----------------------------------------------------
!    function Is_Fixed_Const is used to 
!     1) skip the wet deposition 
!----------------------------------------------------
      function Is_Fixed_Const (ic)
      use h_setkin_par ,  only: NACT
      
      implicit none
      
      !INPUT argument
      integer :: ic
 
 
c     ----------------------
c     Function declarations.
c     ----------------------

      logical :: Is_Fixed_Const
 
 
c     ----------------
c     Begin execution.
c     ----------------

      Is_Fixed_Const = .false.
      
      !The first NACT species are active, followings are not updated by IMPACT module (from CAM5 or read in).
      if ( ic .gt. NACT ) then
       Is_Fixed_Const = .true.
      end if
               
      end function
