c=============================================================================
c
c $Id: aqu_sulfchem.F,v 1.2 2003/06/30 22:50:11 dan Exp $
c
c CODE DEVELOPER
c   originated from GRANTOUR programmed by John Walton
c   modified by Xiaohong Liu
c
c FILE
c   aqu_sulfchem.F
c
c ROUTINES
c   Do_Aqu_Sulfchem
c
c=============================================================================

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Do_Aqu_Sulfchem (chem30)
c
c DESCRIPTION
c
c     ******************************************************************
c     * Uses a simple centered difference scheme to solve sulfur  
c     * chemistry.
c     ******************************************************************
c
c     *     ndt          ==  the number of time steps
c     *     ndtmin       ==  the initial number of time steps to
c                            be used in this routine.
c     *     dtchem       ==  model time step in seconds
c     *     dtclc        ==  the time step (s) for the coupling of
c                            the clear & cloudy sky chemistry
c     *     dtaq         ==  working time step (sec)
c     *     acc          ==  max difference permitted between
c                            succesive solutions for the chemisty
c                            to be considered solved.
c     *     ndone        ==  the number of grid points completed
c     *     emin         ==  the minimum concentration (cm-3) to
c                            which a species is permitted to go.
c     *     numcl        ==  no. of grid points in cloud
c     *     numwk        ==  no. of grid points currently being solved
c     *     nfin         ==  -1,  grid points finished
c                             1,  grid points still integrating
c     *     idxcl        ==  the numbers of the numcl points in cloud
c     *     idxwk        ==  the numbers of the numwk points still
c                            being solved
c
c     *     The following species are categorized by the type
c           of source driving them.
c
c     *     fso4a      ==  H2SO4,  fossil - aqueous
c     *     fso4n      ==  H2SO4,  fossil - clear sky
c     *     fso2       ==  SO2  ,    "
c
c     *     nso4a      ==  H2SO4,  natural - aqueous
c     *     nso4n      ==  H2SO4,     "    - clear sky
c     *     nso2       ==  SO2  ,     "
c     *     ndms       ==  DMS  ,     "
c
c     *     h2o2       ==  H2O2
c
c     ********** output:
c
c     *     dfso4a      ==  H2SO4,  fossil - aqueous
c     *     dfso2       ==  SO2  ,    "
c
c     *     dnso4a      ==  H2SO4,  natural- aqueous
c     *     dnso2       ==  SO2  ,     "
c     *     dndms       ==  DMS  ,     "
c
c     *     dh2o2       ==  H2O2
c
c
c     ********** notes:
c
c     *     The calculation is performed with a centered difference,
c           with and initial predictor step followed by a corrector.
c
c     *     This routine is designed so that the total number of
c           grid points being integrated, numwk at each time, are
c           contiguous.  That is, there are no gaps between the slots
c           being integrated.
c
c     *     The routine is entered with the contribution of species 
c           change due to clear sky reactions, stored in dcp.  It
c           computes the aqueous contribution and adds it to that for 
c           clear sky.
c
c----------------------------------------------------------------------
c Change note by Cheng
c 1) include to use
c 2) set ndtmin = 3, so dtaq = 30min/3 = 10 min. Originally, dtcl=4hour and dtaq = 4hour/24 also equals to 10min

      subroutine Do_Aqu_Sulfchem
     &  (acc, dtaq, dtchem, dtcl, dtclc,
     &   itloop, numcl, idxcl, semiss, srcmas, srcm_aso2,
     &   massc, pr_sulf_src, 
     &   dms_oh, dms_no3, so2_oh, so2_h2o2, so2_o3,
     &   cp, dcp, cfp, zmp, a, loc_proc)

      use h_setkin_par
      use h_sulfchem !use a lot, but not those in commons
      
      implicit none

c#     include "setkin_par.h"
c#     include "sulfchem.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop, numcl
      integer :: idxcl(itloop)

      real*8
     &  acc,
     &  dtaq,  ! time step used in integration of the aqueous equations in seconds
     &  dtchem,! model time step in seconds
     &  dtcl,  ! time step between mixing calculations in seconds
     &  dtclc  ! time step for clear air chemistry

      real*8  :: cp      (itloop, NSP)
      real*8  :: dcp     (itloop, NSP)
      real*8  :: semiss  (itloop, NSP)
      real*8  :: a       (itloop, 9)
      real*8  :: cfp     (itloop)
      real*8  :: zmp     (itloop)
      real*8  :: srcmas  (itloop)
      real*8  :: srcm_aso2  (itloop)  ! added by Roy
      real*8  :: massc   (itloop)

      logical :: pr_sulf_src
      real*8  :: dms_oh  (itloop)
      real*8  :: dms_no3 (itloop)
      real*8  :: so2_oh  (itloop)
      real*8  :: so2_h2o2(itloop)
      real*8  :: so2_o3  (itloop)
 
      integer :: loc_proc

c -------------------------------------------------------------
c  *** LOCAL  VARIABLES WERE DECLARED HERE.
c -------------------------------------------------------------

      real*8  ndtmin, emin

      data ndtmin /3.0d0/ !/ 24.0d0 /, changed by Cheng, since now dtclc = 30 min not 4 hour. 
      data emin / 100.0d0 /

      integer :: nfin(numcl)
      integer :: idxwk(numcl)

      real*8 :: h2o2(itloop)
      real*8 :: ndms(itloop)
      real*8 :: nso2(itloop)
      real*8 :: fso2(itloop)

      real*8 :: dh2o2 (itloop)
      real*8 :: dndms (itloop)
      real*8 :: dnso2 (itloop)
      real*8 :: dfso2 (itloop)
      real*8 :: dnso4a(itloop)
      real*8 :: dfso4a(itloop)

      real*8 :: sfso2(itloop)
      real*8 :: snso2(itloop)
      real*8 :: sndms(itloop)

      real*8 :: h2o2w1(numcl)
      real*8 :: fso2w1(numcl)
      real*8 :: nso2w1(numcl)
      real*8 :: ndmsw1(numcl)
      real*8 :: h2o2w2(numcl)
      real*8 :: fso2w2(numcl)
      real*8 :: nso2w2(numcl)
      real*8 :: ndmsw2(numcl)

      real*8 :: dh2o21(numcl)
      real*8 :: dfso21(numcl)
      real*8 :: dnso21(numcl)
      real*8 :: dndms1(numcl)
      real*8 :: dh2o22(numcl)
      real*8 :: dfso22(numcl)
      real*8 :: dnso22(numcl)
      real*8 :: dndms2(numcl)

      real*8 :: rh2o2(numcl)
      real*8 :: rfso2(numcl)
      real*8 :: rnso2(numcl)

      real*8 :: h2o2o(numcl)
      real*8 :: fso2o(numcl)
      real*8 :: nso2o(numcl)
      real*8 :: ndmso(numcl)

      real*8 :: zdms_oh  (itloop)
      real*8 :: zdms_no3 (itloop)
      real*8 :: zso2_oh  (itloop)
      real*8 :: zso2_h2o2(itloop)
      real*8 :: zso2_o3  (itloop)

c++Roy
      real*8 :: aso2(itloop)
      real*8 :: daso2 (itloop)
      real*8 :: saso2(itloop)
      real*8 :: daso4a(itloop)
      real*8 :: aso4dp(itloop)

      real*8 :: aso2w1(numcl)
      real*8 :: aso2w2(numcl)
      real*8 :: daso21(numcl)
      real*8 :: daso22(numcl)
      real*8 :: raso2(numcl)
      real*8 :: aso2o(numcl)

      aso2 = 0.0d0
      daso2  = 0.0d0
      saso2  = 0.0d0
      aso2w1 = 0.0d0
      aso2w2 = 0.0d0
      daso21 = 0.0d0
      daso22 = 0.0d0
      raso2 = 0.0d0
      aso2o = 0.0d0
      daso4a = 0.0d0
c--

      nfin = 0.0d0
      idxwk = 0.0d0

      h2o2 = 0.0d0
      ndms = 0.0d0
      nso2 = 0.0d0
      fso2 = 0.0d0

      dh2o2  = 0.0d0
      dndms  = 0.0d0
      dnso2  = 0.0d0
      dfso2  = 0.0d0
      dnso4a = 0.0d0
      dfso4a = 0.0d0

      sfso2  = 0.0d0
      snso2  = 0.0d0
      sndms  = 0.0d0

      h2o2w1 = 0.0d0
      fso2w1 = 0.0d0
      nso2w1 = 0.0d0
      ndmsw1 = 0.0d0
      h2o2w2 = 0.0d0
      fso2w2 = 0.0d0
      nso2w2 = 0.0d0
      ndmsw2 = 0.0d0

      dh2o21 = 0.0d0
      dfso21 = 0.0d0
      dnso21 = 0.0d0
      dndms1 = 0.0d0
      dh2o22 = 0.0d0
      dfso22 = 0.0d0
      dnso22 = 0.0d0
      dndms2 = 0.0d0

      rh2o2 = 0.0d0
      rfso2 = 0.0d0
      rnso2 = 0.0d0

      h2o2o = 0.0d0
      fso2o = 0.0d0
      nso2o = 0.0d0
      ndmso = 0.0d0

      zdms_oh   = 0.0d0
      zdms_no3  = 0.0d0
      zso2_oh   = 0.0d0
      zso2_h2o2 = 0.0d0
      zso2_o3   = 0.0d0


      h2o2(:) = cp(:, IH2O2)
      ndms(:) = cp(:, INDMS)
      nso2(:) = cp(:, INSO2)
      fso2(:) = cp(:, IFSO2)

c++Roy
      aso2(:) = cp(:, IASO2)
      saso2(:) = semiss(:, IASO2)
      daso2(:) = dcp(:, IASO2)
      aso4dp(:)= dcp(:, IASO4)
c--
      sfso2(:) = semiss(:, IFSO2)
cMinghuai     &         * 0.98d0 ! 98% of anthropogenic sulfur emission as SO2
      snso2(:) = semiss(:, INSO2)
      sndms(:) = semiss(:, INDMS)

      dh2o2(:) = dcp(:, IH2O2)
      dndms(:) = dcp(:, INDMS)
      dfso2(:) = dcp(:, IFSO2)
      dnso2(:) = dcp(:, INSO2)


c     *--------------------------------------------------
c     * Initialize parameters for this set of grid-points
c     *--------------------------------------------------

c.... ndone, the number of grid points finished.  When this = numcl
c            the routine is finished

      ndone = 0

c.... nfin =  1,  if the grid-point is still being integrated
c          = -1,  if the grid-point is done

c.... idxwk, a working index containing the numbers of grid-points to be
c            integrated. This will be initially set = idxcl, the numbers
c            of the grid-points within clouds.
c            idxwk and its length, numwk, will change as the integration
c            advances.  idxcl and numcl, on the other hand, will remain
c            unchanged.

      do i = 1, numcl
        nfin(i)  = 1
        idxwk(i) = idxcl(i)
      enddo

c.... initially the grid-point index and number is the same as
c     you come in with

      numwk = numcl

c     The initial dtaq will be dtclc / ndtmin. 

      ndt = ndtmin

       
c     *-----------------------------------------------------------------
c     * This is the point to which one returns when a new time step,
c       dtaq, integration is to be started.
c     *-----------------------------------------------------------------

c     ========
  100 continue
c     ========

c.... set dtaq 

      dtaq = dtclc / float( ndt )


c     ******************************************************************
c       If dtaq < 0.1 seconds the method has failed to converge.
c       An implicit calculation with a 1 second time step will be used.
c       The relation solved, for species f, is
c
c         [ f(t+dt) - f(t) ] / dt
c         = Source + Production( f' ) - L( f' ) [ f(t+dt) + f(t) ] / 2
c
c       where f' refers to a species list that does not include f.
c     ******************************************************************

      if( dtaq .lt. 0.1 ) then

        write(6, 10)  numwk, 2. * dtaq, loc_proc
   10   format( / 'chem30: ', i6, ' grid cells failed to converge with',
     &            ' dtaq = ', e12.4, ' sec.',
     &            'This loc_proc is ', i4,
     &          / 'Set dtaq = 1 sec. and ',
     &            'do an implicit integration for these grid cells.' )

        dtaq = 1.
        ndt  = ( dtclc + 0.1 ) / dtaq


        do 920 i = 1, numwk

          iw = idxwk(i)

c     Species
          zh2o2 = h2o2( iw )
          zfso2 = fso2( iw )
          znso2 = nso2( iw )
          zndms = ndms( iw )
c         zbso2 = bso2( iw )
          zaso2 = aso2( iw )  ! added by Roy

c     Sources
          zsfso2 = sfso2( iw )
          zsndms = sndms( iw )
          zsnso2 = snso2( iw )
c         zsbso2 = sbso2( iw )
          zsaso2 = saso2( iw ) ! added by Roy

c     a's
          za1 = a( iw, 1 )
          za2 = a( iw, 2 )
          za3 = a( iw, 3 )
          za4 = a( iw, 4 )
          za5 = a( iw, 5 )
          za6 = a( iw, 6 )
          za7 = a( iw, 7 )
          za8 = a( iw, 8 )
          za9 = a( iw, 9 )

c     Sulf budget
          zdms_oh  (iw) = 0.0d0
          zdms_no3 (iw) = 0.0d0
          zso2_oh  (iw) = 0.0d0
          zso2_h2o2(iw) = 0.0d0
          zso2_o3  (iw) = 0.0d0


          do 900 m = 1, ndt

c     Calculate the production and lose terms.

c.... H2O2
            prh2o2 = za3 + za8
            dsh2o2 = ( za4 + za5 
     &               + za6 * ( zfso2 + znso2 ) ) * 0.5 * dtaq

c.... fossil
            prfso2 = zsfso2
            dsfso2 = ( za2 + za7 + za6 * zh2o2 ) * 0.5 * dtaq

c.... air soot added by Roy
            praso2 = zsaso2
            dsaso2 = ( za2 + za7 + za6 * zh2o2 ) * 0.5 * dtaq

c.... natural
            prndms = zsndms
            dsndms = ( za1 + za9 ) * 0.5 * dtaq

            prnso2 = zsnso2 + ( za1 + za9 ) * zndms
            dsnso2 = ( za2 + za7 + za6 * zh2o2 ) * 0.5 * dtaq

c.... biomass
c           prbso2 = zsbso2
c           dsbso2 = ( za2 + za7 + za6 * zh2o2 ) * 0.5 * dtaq
c           dbso21 = sbso2
c    #               - ( a2 + a7
c    #               + a6 * h2o2 ) * bso2

c     Advance species concentrations.

            zh2o2 = ( prh2o2 * dtaq + ( 1. - dsh2o2  ) * zh2o2 )
     &            /                   ( 1. + dsh2o2  )
            zfso2 = ( prfso2 * dtaq + ( 1. - dsfso2  ) * zfso2 )
     &            /                   ( 1. + dsfso2  )
            zndms = ( prndms * dtaq + ( 1. - dsndms  ) * zndms )
     &            /                   ( 1. + dsndms  )
            znso2 = ( prnso2 * dtaq + ( 1. - dsnso2  ) * znso2 )
     &            /                   ( 1. + dsnso2  )
c           zbso2 = ( prbso2 * dtaq + ( 1. - dsbso2  ) * zbso2 )
c    &            /                   ( 1. + dsbso2  )
c added by Roy
            zaso2 = ( praso2 * dtaq + ( 1. - dsaso2  ) * zaso2 )
     &            /                   ( 1. + dsaso2  )

            if(pr_sulf_src) then

              zdms_oh(iw)   = zdms_oh(iw) + 
     &                        za1 * zndms * dtaq
              zdms_no3(iw)  = zdms_no3(iw) +
     &                        za9 * zndms * dtaq
              zso2_oh(iw)   = zso2_oh(iw) +
     &                        za2 * (zfso2 + znso2 + zaso2) * dtaq
              zso2_h2o2(iw) = zso2_h2o2(iw) +
     &                        za6 * zh2o2 * (zfso2 + znso2 + 
     &                                               zaso2) * dtaq
              zso2_o3(iw)   = zso2_o3(iw) +
     &                        za7 * (zfso2 + znso2 + zaso2) * dtaq

            end if

  900     continue


c     Include the species change due to aqueous reaction in the
c     total change due to chemistry.

c.... H2O2
          dh2o2( iw )  = dh2o2( iw )
     &                 + cfp( iw ) * ( zh2o2 - h2o2( iw ) )

c.... fossil
          dfso2( iw )  = dfso2( iw )
     &                 + cfp( iw ) * ( zfso2 - fso2( iw ) )
          dfso4a( iw ) = cfp( iw )
     &                 * (   sfso2(iw) * dtclc     ! dtchem
     &                   - ( zfso2 - fso2( iw ) ) )

c.... air soot SO2 added by Roy
          daso2( iw )  = daso2( iw )
     &                 + cfp( iw ) * ( zaso2 - aso2( iw ) )
          daso4a( iw ) = cfp( iw )
     &                 * (   saso2(iw) * dtclc     ! dtchem
     &                   - ( zaso2 - aso2( iw ) ) )

c.... natural
          dnso2( iw )  = dnso2( iw )
     &                 + cfp( iw ) * ( znso2 - nso2( iw ) )
          dndms( iw )  = dndms( iw )
     &                 + cfp( iw ) * ( zndms - ndms( iw ) )
          dnso4a( iw ) = cfp( iw ) 
     &                 * ( ( snso2(iw) + sndms(iw) ) * dtclc  ! dtchem
     &                   - ( znso2 - nso2( iw ) )
     &                   - ( zndms - ndms( iw ) ) )

c.... biomass
c         dbso2( iw )  = dbso2( iw )
c     &                + cfp( iw ) * ( zbso2 - bso2( iw ) )
c         dbso4a( iw ) = cfp( iw ) 
c     &                * (   sbso2(iw) * dtclc     ! dtchem 
c     &                  - ( zbso2 - bso2( iw ) ) )

  920   continue


c     This completes the calculation using the implicit scheme

c       =========
        go to 787
c       =========

c.... this is the end of the dtaq if statement
c     =====
      endif
c     =====


c     Note, since the only change to H2SO4 is through chemical
c     sources, it can be set = 0 here and the routine will
c     calculate the H2SO4 change due to aqueous reactions.

      do i = 1, numwk

        iw = idxwk(i)

c.... H2O2
        h2o2w1(i) = h2o2( iw )

c.... fossil
        fso2w1(i) = fso2( iw )

c.... air soot SO2 added by Roy
        aso2w1(i) = aso2( iw )

c.... natural
        nso2w1(i) = nso2( iw )
        ndmsw1(i) = ndms( iw )

c.... biomass
c       bso2w1(i) = bso2( iw )

      enddo


c     ******************************************************************
c     * Advance the chemistry for each active grid-points.
c     * In what follows, it is necessary to point to the correct a's
c       and sources for each grid point being run.  The idxwk does this.
c     ******************************************************************

c     *--------------------
c     Begin time step loop.
c     *--------------------

      do i = 1, numwk

        iw = idxwk(i)

        zdms_oh  (iw) = 0.0d0
        zdms_no3 (iw) = 0.0d0
        zso2_oh  (iw) = 0.0d0
        zso2_h2o2(iw) = 0.0d0
        zso2_o3  (iw) = 0.0d0

      enddo

      do 200 m = 1, ndt


c     *-----------------------------------------------------------------
c     * Tentative step.
c     * Compute species derivatives based on initial values.
c     *-----------------------------------------------------------------

        do i = 1, numwk

          iw = idxwk(i)

c.... H2O2
          dh2o21(i) = a(iw, 3) + a(iw, 8) - ( a(iw, 4) + a(iw, 5)
     &              + a(iw, 6) * ( fso2w1(i) + nso2w1(i) ) ) 
     &              * h2o2w1(i)

c.... fossil
          dfso21(i) = sfso2(iw)
     &              - ( a(iw, 2) + a(iw, 7)
     &              + a(iw, 6) * h2o2w1(i) ) * fso2w1(i)

c.... air soot SO2 added by Roy
          daso21(i) = saso2(iw)
     &              - ( a(iw, 2) + a(iw, 7)
     &              + a(iw, 6) * h2o2w1(i) ) * aso2w1(i)

c.... natural
          dndms1(i) = sndms(iw) - ( a(iw, 1) + a(iw, 9) ) * ndmsw1(i)

          dnso21(i) = snso2(iw)
     &              + ( a(iw, 1) + a(iw, 9) ) * ndmsw1(i)
     &              - ( a(iw, 2) + a(iw, 7)
     &              + a(iw, 6) * h2o2w1(i) ) * nso2w1(i)

c.... biomass
c         dbso21(i) = sbso2(iw)
c    &              - ( a2(iw) + a7(iw)
c    &              + a6(iw) * h2o2w1(i) ) * bso2w1(i)

        enddo


c     *-----------------------------------------------------------------
c     * Evaluate tentative the species based on initial values.
c     *-----------------------------------------------------------------

        do i = 1, numwk

c.... H2O2
          h2o2w2(i) = max( h2o2w1(i) + dh2o21(i) * dtaq, 0.0d0)

c.... fossil
          fso2w2(i) = max( fso2w1(i) + dfso21(i) * dtaq, 0.0d0)

c.... air soot SO2 added by Roy
          aso2w2(i) = max( aso2w1(i) + daso21(i) * dtaq, 0.0d0)

c.... natural
          ndmsw2(i) = max( ndmsw1(i) + dndms1(i) * dtaq, 0.0d0)
          nso2w2(i) = max( nso2w1(i) + dnso21(i) * dtaq, 0.0d0)

c.... biomass
c         bso2w2(i) = max( bso2w1(i) + dbso21(i) * dtaq, 0.0d0)

        enddo


c     *-----------------------------------------------------------------
c     * Final step.
c     * Compute species derivatives based on the tentative 
c       concentrations.
c     *-----------------------------------------------------------------

        do i = 1, numwk

          iw = idxwk(i)

c.... H2O2
          dh2o22(i) = a(iw, 3) + a(iw, 8) - ( a(iw, 4) + a(iw, 5)
     &              + a(iw, 6) * ( fso2w2(i) + nso2w2(i) ) ) 
     &              * h2o2w2(i)

c.... fossil
          dfso22(i) = sfso2(iw)
     &              - ( a(iw, 2) + a(iw, 7)
     &              + a(iw, 6) * h2o2w2(i) ) * fso2w2(i)

c.... air soot SO2 added by Roy
          daso22(i) = saso2(iw)
     &              - ( a(iw, 2) + a(iw, 7)
     &              + a(iw, 6) * h2o2w2(i) ) * aso2w2(i)

c.... natural
          dndms2(i) = sndms(iw) - ( a(iw, 1) + a(iw, 9) ) * ndmsw2(i)

          dnso22(i) = snso2(iw)
     &              + ( a(iw, 1) + a(iw, 9) ) * ndmsw2(i)
     &              - ( a(iw, 2) + a(iw, 7)
     &              + a(iw, 6) * h2o2w2(i) ) * nso2w2(i)

c.... biomass
c         dbso22(i) = sbso2(iw)
c    &              - ( a2(iw) + a7(iw)
c    &                + a6(iw) * h2o2w2(i) ) * bso2w2(i)

        enddo

cc sulf budget
        if(pr_sulf_src) then

          do i = 1, numwk

            iw = idxwk(i)

              zdms_oh(iw) = zdms_oh(iw) +
     &          a(iw, 1) * 0.5 *
     &          (ndmsw1(i) + ndmsw2(i)) * dtaq
              zdms_no3(iw) = zdms_no3(iw) +
     &          a(iw, 9) * 0.5 *
     &          (ndmsw1(i) + ndmsw2(i)) * dtaq
              zso2_oh(iw) = zso2_oh(iw) +
     &          a(iw, 2) * 0.5 *
     &          (fso2w1(i) + nso2w1(i) + fso2w2(i) + nso2w2(i) +
     &            aso2w1(i) + aso2w2(i)) * dtaq       ! added by Roy
              zso2_h2o2(iw) = zso2_h2o2(iw) +
     &          a(iw, 6) * 0.5 *
     &          (h2o2w1(i) * (fso2w1(i) + nso2w1(i) + aso2w1(i)) +
     &           h2o2w2(i) * (fso2w2(i) + nso2w2(i) + aso2w2(i)))*dtaq 
              zso2_o3(iw) = zso2_o3(iw) +
     &          a(iw, 7) * 0.5 *
     &          (fso2w1(i) + nso2w1(i) + fso2w2(i) + nso2w2(i) +
     &           aso2w1(i) + aso2w2(i)) * dtaq    ! added by Roy

          enddo

        end if

c     *-----------------------------------------------------------------
c     * Compute a final species concentration based on the mean of
c       the two derivatives.
c     *-----------------------------------------------------------------

        do i = 1, numwk

c.... H2O2
          h2o2w1(i) = max( h2o2w1(i) 
     &              + 0.5 * ( dh2o21(i) + dh2o22(i) ) * dtaq, 0.0d0)

c.... fossil
          fso2w1(i) = max( fso2w1(i)  
     &              + 0.5 * ( dfso21(i) + dfso22(i) ) * dtaq, 0.0d0)

c.... air soot SO2 added by Roy
          aso2w1(i) = max( aso2w1(i)
     &              + 0.5 * ( daso21(i) + daso22(i) ) * dtaq, 0.0d0)

c.... natural
          ndmsw1(i) = max( ndmsw1(i)
     &              + 0.5 * ( dndms1(i) + dndms2(i) ) * dtaq, 0.0d0)
          nso2w1(i) = max( nso2w1(i)  
     &              + 0.5 * ( dnso21(i) + dnso22(i) ) * dtaq, 0.0d0)

c.... biomass
c         bso2w1(i) = max( bso2w1(i)  
c    &              + 0.5 * ( dbso21(i) + dbso22(i) ) * dtaq, 0.0d0)

        enddo


c     *--------------------
c     * End time step loop.
c     *--------------------

  200 continue


c     ******************************************************************
c     * Perform time step analysis.                       
c     ******************************************************************

      if( ndt .eq. ndtmin ) then


c     *-----------------------------------------------------------------
c     * If ndt = ndtmin halve the time step.
c     * No grid-point solution can exist at this point.
c     *-----------------------------------------------------------------

        ndt = 2 * ndtmin

        do i = 1, numwk

c.... H2O2
          h2o2o(i) = h2o2w1(i)

c.... fossil
          fso2o(i) = fso2w1(i)

c.... air soot SO2 added by Roy
          aso2o(i) = aso2w1(i)

c.... natural
          ndmso(i) = ndmsw1(i)
          nso2o(i) = nso2w1(i)

c.... biomass
c         bso2o(i) = bso2w1(i)

        enddo

c       =========
        go to 100
c       =========

      else


c     *-----------------------------------------------------------------
c     * If ndt > ndtmin check the solution.
c     * If the solution satisfies the accuracy condition store it.
c     * Note, aqueous H2SO4, ()so4a, has no clear sky contribution.
c     *-----------------------------------------------------------------

c.... compute the fractional changes from the previous cycle

        do i = 1, numwk

          rh2o2(i) = ( h2o2o(i)  - h2o2w1(i) ) 
     &             / ( h2o2w1(i) + emin )

          rfso2(i) = ( fso2o(i)  - fso2w1(i) ) 
     &             / ( fso2w1(i) + emin )
c added by Roy
          raso2(i) = ( aso2o(i)  - aso2w1(i) )
     &             / ( aso2w1(i) + emin )

c         rndms(i) = ( ndmso(i)  - ndmsw1(i) ) 
c    &             / ( ndmsw1(i) + emin )

          rnso2(i) = ( nso2o(i)  - nso2w1(i) ) 
     &             / ( nso2w1(i) + emin )

c         rbso2(i) = ( bso2o(i)  - bso2w1(i) ) 
c    &             / ( bso2w1(i) + emin )

        enddo


c.... if the solution passes the accuray test store the solution
c     from the working slot "i" back into grid-point idxwk(i)

c     The change in H2SO4 will be computed using the DMS and SO2 
c     sources and the changes in DMS and SO2.

c     The changes in H2O2, SO2 and DMS due to clear sky chemistry have
c     already been computed, and weighted by ( 1 - cfp ), in chem11,
c     so here it remains to add their aqueous changes, weighted by
c     cfp, to these clear air changes.

        do i = 1, numwk

          if( abs( rh2o2(i) ) .lt. acc .and.
     &        abs( rfso2(i) ) .lt. acc .and.       
     &        abs( rnso2(i) ) .lt. acc .and.
     &        abs( raso2(i) ) .lt. acc      ) then    !added by Roy
c    &        abs( rndms(i) ) .lt. acc .and.       

            ndone   = ndone + 1
            nfin(i) = -1

            iw = idxwk(i)

c.... H2O2
            dh2o2( iw )  = dh2o2( iw )
     &                   + cfp( iw ) * ( h2o2w1(i) - h2o2( iw ) )

c.... fossil
            dfso2( iw )  = dfso2( iw )
     &                   + cfp( iw ) * ( fso2w1(i) - fso2( iw ) )

            dfso4a( iw ) = cfp( iw )
     &                   * ( sfso2(iw) * dtclc   ! dtchem
     &                   - ( fso2w1(i) - fso2( iw ) ) )

c.... air soot SO2 by Roy
            daso2( iw )  = daso2( iw )
     &                   + cfp( iw ) * ( aso2w1(i) - aso2( iw ) )

            daso4a( iw ) = cfp( iw )
     &                   * ( saso2(iw) * dtclc   ! dtchem
     &                   - ( aso2w1(i) - aso2( iw ) ) )

c.... natural
            dnso2( iw )  = dnso2( iw )
     &                   + cfp( iw ) * ( nso2w1(i) - nso2( iw ) )

            dndms( iw )  = dndms( iw )
     &                   + cfp( iw ) * ( ndmsw1(i) - ndms( iw ) )

            dnso4a( iw ) = cfp( iw ) 
     &                   * ( ( snso2(iw) + sndms(iw) ) * dtclc   ! dtchem
     &                     - ( nso2w1(i) - nso2( iw ) )
     &                     - ( ndmsw1(i) - ndms( iw ) ) )

c.... biomass
c           dbso2( iw )  = dbso2( iw )
c     &                  + cfp( iw ) * ( bso2w1(i) - bso2( iw ) )
c           dbso4a( iw ) = cfp( iw ) 
c     &                  * (   sbso2(iw) * dtclc   ! dtchem 
c     &                    - ( bso2w1(i) - bso2( iw ) ) )

          endif

        enddo

      endif  ! if( ndt .eq. ndtmin ) then


c     *****************************************************************
c     * If ndone = numcl all grid points have a solution, return.
c     * If ndone < numcl halve the time step of those that have
c       not converged and try again.
c     *****************************************************************


c.... if all grid points have been solved, return

      if(ndone .eq. numcl) then
c       =========
        go to 787
c       =========
      endif

c.... some grid points are yet to be solved, 
c     compress out solved grid points and store the latest 
c     computed values in h2o2o, etc.

      ndt = 2 * ndt

      num1 = 0

      do i = 1, numwk

        if( nfin(i) .eq. 1 ) then

          num1 = num1 + 1

          nfin( num1 )  = 1

          idxwk( num1 ) = idxwk(i)

          h2o2o( num1 ) = h2o2w1(i)
          fso2o( num1 ) = fso2w1(i)
          ndmso( num1 ) = ndmsw1(i)
          nso2o( num1 ) = nso2w1(i)
c         bso2o( num1 ) = bso2w1(i)
          aso2o( num1 ) = aso2w1(i)   ! added by Roy

        endif

      enddo

      numwk = num1
         
c     =========
      go to 100
c     =========

 787  continue


c     conv: convert SO4 concentration from # cm-3 to gm gm-1
      conv = wtmair / wtmso4

c.... H2O2

        dcp(:, IH2O2)  = dh2o2(:)

c.... fossil

        dcp(:, IFSO2)  = dfso2(:)
c       dcp(:, IFSO4A) = dfso4a(:)
c    &                     / (conv * zmp(:))   ! unit: g/g

c.... air soot SO2 added by Roy

        dcp(:, IASO2)  = daso2(:)

c.... natural

        dcp(:, INSO2)  = dnso2(:)
        dcp(:, INDMS)  = dndms(:)
c       dcp(:, INSO4A) = dnso4a(:)
c    &                     / (conv * zmp(:))   ! unit: g/g

        srcmas(:) = ( dfso4a(:) + dnso4a(:) )
     &            / dtchem * 1.0d+6            ! unit: molecules/m3/s

        srcm_aso2(:) = daso4a(:) / dtchem * 1.0d+6 ! added by Roy
c Roy: double count scrmas
        dcp(:, IASO4) = aso4dp(:) + daso4a(:) / (conv * zmp(:))   ! unit: g/g

        if(pr_sulf_src) then

          dms_oh(:) = dms_oh(:)
     &                + cfp(:) * zdms_oh(:) * massc(:)
     &                / zmp(:) * wtmso2 / wtmair
          dms_no3(:) = dms_no3(:)
     &                + cfp(:) * zdms_no3(:) * massc(:)
     &                / zmp(:) * wtmso2 / wtmair
          so2_oh(:) = so2_oh(:)
     &                + cfp(:) * zso2_oh(:) * massc(:)
     &                / zmp(:) * wtmso4 / wtmair
          so2_h2o2(:) = so2_h2o2(:)
     &                + cfp(:) * zso2_h2o2(:) * massc(:)
     &                / zmp(:) * wtmso4 / wtmair
          so2_o3(:) = so2_o3(:)
     &                + cfp(:) * zso2_o3(:) * massc(:)
     &                / zmp(:) * wtmso4 / wtmair

        end if


      return

      end

