c=============================================================================
c
c $Id: const_sulf_update.F,v 1.2 2003/06/30 22:50:11 dan Exp $
c
c CODE DEVELOPER
c   originated from GRANTOUR programmed by John Walton
c   modified by Xiaohong Liu
c
c FILE
c   const_sulf_update.F
c
c ROUTINES
c   Do_Const_Sulf_Update
c
c=============================================================================

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Do_Const_Sulf_Update (chem40)
c
c DESCRIPTION
c
c     *****************************************************************
c     * advances the species concentrations and does mixing of clear
c     *   and cloudy regions
c     *****************************************************************
c
c     ********** input:
c
c     *     cp     : concentration (# cm-3 for gas-species, g/g for aerosols)
c     *     dcp    : storage for conc. change
c
c     *     fso4a  : aqueous fossil H2SO4 conc.
c     *     fso4n  : clear sky fossil H2SO4 conc.
c     *     fso2   : fossil SO2 conc.
c
c     *     nso4a  : aqueous natural H2SO4 conc.
c     *     nso4n  : clear sky natural H2SO4 conc.
c     *     nso2   : natural SO2 conc.
c     *     ndms   : natural DMS conc.
c
c     *     h2o2   : H2O2 conc.
c
c     *     dfso4a : aqueous fossil H2SO4 conc.
c     *     dfso4n : clear sky fossil H2SO4 conc.
c     *     dfso2  : fossil SO2 conc.
c
c     *     dnso4a : aqueous natural H2SO4 conc.
c     *     dnso4n : clear sky natural H2SO4 conc.
c     *     dnso2  : natural SO2 conc.
c     *     dndms  : natural DMS conc.
c
c     *     dh2o2  : H2O2 conc.
c
c     *     grid_height : height of each grid box (m)
c
c     * For H2SO4 the aqueous and clear sky parts remain seperate.
c       However, the H2O2, DMS and SO2 are a combination of the aqueous
c       and clear sky parts, weighted by the cloud fraction cfp.
c
c     * The clear sky changes were computed and stored in chem11.  
c       In chem30 the aqueous changes were computed and,
c       except for H2SO4, added to the clear sky.  e.g., with cfp
c       the cloud fraction at a grid point
c
c       dcp(ns) = (1-cfp) * d[clear sky] + cfp * d[aqueous]
c
c----------------------------------------------------------------------

      subroutine Do_Const_Sulf_Update
     $  (itloop, cp, dcp, kel, relh, grid_height, gmair)

      use h_setkin_par
      use h_sulfchem,only: PI, WTMSO4
      use h_umaerosol
      use h_imp_phys_constants
      
      implicit none

!#     include "setkin_par.h"
!#     include "sulfchem.h"
!#     include "umaerosol.h"
!#     include "imp_phys_constants.h"


c     ----------------------
c     IMPACT-related values.
c     ----------------------

      integer :: itloop           ! # of zones (ilong * ilat * ivert)

      real*8  :: cp (itloop, NSP)
      real*8  :: dcp(itloop, NSP)
      real*8  :: kel(itloop)         ! temperature [K]
      real*8  :: relh(itloop)        ! relative humidity [0 - 1]
      real*8  :: h2osat              ! h2o saturation pressure [#molec/m3]
      real*8  :: h2ogas              ! h2o gas concentration [#molec/m3]
      real*8  :: so4mfrac(itloop)    ! so4 mass fraction in aerosol [1]
      real*8  :: so4dens(itloop)     ! aerosol density [kg/m3]
      real*8  :: rso4min(itloop)     ! radius of each so4 molecule [m] 
      real*8  :: nso4non(itloop,NSP) ! so4 molecules on non-so4 aerosol surface [#molec/particle]  
      real*8  :: lso4non(itloop,NSP) ! area fraction of so4 molecules on non-so4 aerosol surface 
                                     ! [number of layers] 
      real*8  :: gmair(itloop)       ! air density (g/cm3)
      real*8  :: grid_height(itloop)

      real*8  :: BOCaging(itloop,8)  !  amount of BOC aged (accumulated since last output (kg/m^2))  

      integer :: il
      integer :: RYUN                ! index for which so4 molecule radius to use
      real*8  :: so4bc_cr1,so4oc_cr1,so4bc_cr2,so4oc_cr2

      nso4non(itloop,:) = 0.0d0      ! set initial value zero
      lso4non(itloop,:) = 0.0d0      ! set initial value zero
      BOCaging(itloop,:) = 0.0d0     ! set initial value zero

c     Write (0,*) 'Do_Const_Sulf_Update '

      do inon=1,nnon
       radvolm(:)=radgnon(:,inon)*exp(r3h*log(siggnon(:,inon))**2)
       radsufm(:)=radgnon(:,inon)*exp(log(siggnon(:,inon))**2)
       radvnon(inon)=sum(fracnon(:,inon)*radvolm(:)**3)
       radsnon(inon)=sum(fracnon(:,inon)*radsufm(:)**2)
       pmsnon(inon)=r4td*pi*rhonon(inon)*radvnon(inon)
       radvnon(inon)=radvnon(inon)**r1td
       radsnon(inon)=radsnon(inon)**0.5
      enddo
 
c      Write (0,*) 'radsnon(:)=', radsnon(:)
c      Write (0,*) 'radvnon(:)=', radvnon(:)

      so4oc_cr1 = 1.0         ! 1 layer of sulfate    
      so4bc_cr1 = 1.0 
c      so4oc_cr2 = 2.3        ! 2.3 layers of sulfate
c      so4bc_cr2 = 2.3 

      so4oc_cr2 = 3.0         ! 3 layers of sulfate
      so4bc_cr2 = 3.0

      cp(:, IH2O2)   = cp(:, IH2O2) + dcp(:,IH2O2)
      cp(:, INDMS)   = cp(:, INDMS) + dcp(:,INDMS)
      cp(:, INSO2)   = cp(:, INSO2) + dcp(:,INSO2)
      cp(:, IFSO2)   = cp(:, IFSO2) + dcp(:,IFSO2)
      cp(:, IASO2)   = cp(:, IASO2) + dcp(:,IASO2)
      cp(:, IASO4)   = cp(:, IASO4) + dcp(:,IASO4)

c      write(0,*) 'sulf_aSO2->',Sum(cp(:, IASO4))

c     cp(:, IFSO4A)  = cp(:, IFSO4A ) + dcp(:,IFSO4A)
c     cp(:, IFSO4N1) = cp(:, IFSO4N1) + dcp(:,IFSO4N1)
c     cp(:, IFSO4N2) = cp(:, IFSO4N2) + dcp(:,IFSO4N2)
c     cp(:, IFSO4N3) = cp(:, IFSO4N3) + dcp(:,IFSO4N3)

c     cp(:, INSO4A)  = cp(:, INSO4A ) + dcp(:,INSO4A)
c     cp(:, INSO4N1) = cp(:, INSO4N1) + dcp(:,INSO4N1)
c     cp(:, INSO4N2) = cp(:, INSO4N2) + dcp(:,INSO4N2)
c     cp(:, INSO4N3) = cp(:, INSO4N3) + dcp(:,INSO4N3)

      where(cp(:, IH2O2)    < 1.0d-30) cp(:, IH2O2) = 1.0d-30
      where(cp(:, INDMS)    < 1.0d-30) cp(:, INDMS) = 1.0d-30
      where(cp(:, INSO2)    < 1.0d-30) cp(:, INSO2) = 1.0d-30
      where(cp(:, IFSO2)    < 1.0d-30) cp(:, IFSO2) = 1.0d-30
      where(cp(:, ISO4G)    < 1.0d-30) cp(:, ISO4G) = 1.0d-30
      where(cp(:, IASO2)    < 1.0d-30) cp(:, IASO2) = 1.0d-30
      where(cp(:, IASO4)    < 1.0d-30) cp(:, IASO4) = 1.0d-30

      where(cp(:, ISO4M1)   < 1.0d-30) cp(:, ISO4M1) = 1.0d-30
      where(cp(:, ISO4N1)   < 1.0d-30) cp(:, ISO4N1) = 1.0d-30
      where(cp(:, ISO4M2)   < 1.0d-30) cp(:, ISO4M2) = 1.0d-30
      where(cp(:, ISO4N2)   < 1.0d-30) cp(:, ISO4N2) = 1.0d-30
      where(cp(:, ISO4M3)   < 1.0d-30) cp(:, ISO4M3) = 1.0d-30
      where(cp(:, ISO4N3)   < 1.0d-30) cp(:, ISO4N3) = 1.0d-30

c=========================================================================
c added by Roy: hydrophobic to hydrophilic
c  modified by Yuxing Yun 7/6/2010
c=========================================================================
 
      do il=1, itloop

        h2osat=h2osat_f(kel(il))
        h2ogas=relh(il)*h2osat
        so4mfrac(il)=so4mfrac_f(kel(il),h2osat,h2ogas)
        so4dens(il)=so4dens_f(kel(il),so4mfrac(il))

c-------------------------------
c use which so4 molecule radius
c 0: old
c 1: new
c-------------------------------

      RYUN = 1

c----------------------------------------------------------------
c     radius of each so4 molecule (m)
c----------------------------------------------------------------

      if (RYUN == 0) then
         rso4min(il)=(so4min*r3q /
     $              (pi*so4mfrac(il)*so4dens(il)))**r1td

      else
c        rso4min(il) = 160.0d-12                ! unit: m source: wikipedia 
         rso4min(il) = 2.8d-10                  ! unit: m source: olga

      endif


c     rhonon  aerosol density                   [kg/m3]
c     radvnon(nnon)      volume mean radius     [m]
c     wtmso4       ! molecular weight of H2SO4  [g/mol]

c----------------------------------------------------------------
c    # of so4 molecules on non-so4 aerosol surface (#molec/particle)
c----------------------------------------------------------------

      nso4non(il,ISO4FOC) = (cp(il,ISO4FOC)/(wtmso4/avno)) 
     $              / (max(r1,cp(il,IFOC)/(pmsnon(2)*1.0d3)))

      nso4non(il,ISO4OCPHL1) = (cp(il,ISO4OCPHL1)/(wtmso4/avno)) 
     $              / (max(r1,cp(il,IOCPHL1)/(pmsnon(6)*1.0d3)))

      nso4non(il,ISO4OCPHL2) = (cp(il,ISO4OCPHL2)/(wtmso4/avno)) 
     $              / (max(r1,cp(il,IOCPHL2)/(pmsnon(7)*1.0d3)))

      nso4non(il,ISO4FBC) = (cp(il,ISO4FBC)/(wtmso4/avno)) 
     $              / (max(r1,cp(il,IFBC)/(pmsnon(3)*1.0d3)))

      nso4non(il,ISO4BCPHL1) = (cp(il,ISO4BCPHL1)/(wtmso4/avno)) 
     $              / (max(r1,cp(il,IBCPHL1)/(pmsnon(8)*1.0d3)))

      nso4non(il,ISO4BCPHL2) = (cp(il,ISO4BCPHL2)/(wtmso4/avno)) 
     $              / (max(r1,cp(il,IBCPHL2)/(pmsnon(9)*1.0d3)))
cRoy
      nso4non(il,ISO4CPOM) = (cp(il,ISO4CPOM)/(wtmso4/avno))
     $              / (max(r1,cp(il,ICPOM)/(pmsnon(10)*1.0d3)))

      nso4non(il,ISO4CBC) = (cp(il,ISO4CBC)/(wtmso4/avno))
     $              / (max(r1,cp(il,ICBC)/(pmsnon(11)*1.0d3)))

      nso4non(il,ISO4APOM) = (cp(il,ISO4APOM)/(wtmso4/avno))
     $              / (max(r1,cp(il,IAPOM)/(pmsnon(12)*1.0d3)))

      nso4non(il,ISO4ABC) = (cp(il,ISO4ABC)/(wtmso4/avno))
     $              / (max(r1,cp(il,IABC)/(pmsnon(13)*1.0d3)))

c      Write (0,*) 'nso4non(il,ISO4FOC)=', nso4non(il,ISO4FOC)

c---------------------------------------------------------------- 
c     area fraction of so4 molecules on non-so4 aerosol surface
c----------------------------------------------------------------

       lso4non(il,ISO4FOC)=rso4min(il)**2.d0*nso4non(il,ISO4FOC)
     $               /(4.d0*(radsnon(2)+rso4min(il))**2.d0)

       lso4non(il,ISO4FBC)=rso4min(il)**2.d0*nso4non(il,ISO4FBC)
     $               /(4.d0*(radsnon(3)+rso4min(il))**2.d0)

       lso4non(il,ISO4OCPHL1)=rso4min(il)**2.d0*nso4non(il,ISO4OCPHL1)
     $               /(4.d0*(radsnon(6)+rso4min(il))**2.d0)

       lso4non(il,ISO4OCPHL2)=rso4min(il)**2.d0*nso4non(il,ISO4OCPHL2)
     $               /(4.d0*(radsnon(7)+rso4min(il))**2.d0)

       lso4non(il,ISO4BCPHL1)=rso4min(il)**2.d0*nso4non(il,ISO4BCPHL1)
     $               /(4.d0*(radsnon(8)+rso4min(il))**2.d0)

       lso4non(il,ISO4BCPHL2)=rso4min(il)**2.d0*nso4non(il,ISO4BCPHL2)
     $               /(4.d0*(radsnon(9)+rso4min(il))**2.d0)
cRoy
       lso4non(il,ISO4CPOM)=rso4min(il)**2.d0*nso4non(il,ISO4CPOM)
     $               /(4.d0*(radsnon(10)+rso4min(il))**2.d0)

       lso4non(il,ISO4CBC)=rso4min(il)**2.d0*nso4non(il,ISO4CBC)
     $               /(4.d0*(radsnon(11)+rso4min(il))**2.d0)

       lso4non(il,ISO4APOM)=rso4min(il)**2.d0*nso4non(il,ISO4APOM)
     $               /(4.d0*(radsnon(12)+rso4min(il))**2.d0)

       lso4non(il,ISO4ABC)=rso4min(il)**2.d0*nso4non(il,ISO4ABC)
     $               /(4.d0*(radsnon(13)+rso4min(il))**2.d0)


c      Write (0,*) 'lso4non(il,ISO4FBC)=', lso4non(il,ISO4FBC)

c------------------------------------------------------------------------------------------------------
c if hydrophilic BC/OC meets the criteria for hygroscopic BC/OC, 
c move all hydrophilic BC/OC and their coatings to hygroscopic,
c and set hydrophilic BC/OC and their coatings to zero

c if hydrophobic BC/OC meets the criteria for hygroscopic BC/OC,
c move all hydrophobic BC/OC and their coatings to hygroscopic,
c and set hydrophobic BC/OC and their coatings to zero

c if hydrophobic BC/OC meets the criteria for hydrophilic BC/OC,
c move all hydrophobic BC/OC and their coatings to hydrophilic,
c and set hydrophobic BC/OC and their coatings to zero
c------------------------------------------------------------------------------------------------------

        if(lso4non(il,ISO4BCPHL1) > so4bc_cr2) then
          cp(il,ISO4BCPHL2) = cp(il,ISO4BCPHL2) + cp(il,ISO4BCPHL1)
          cp(il,IBCPHL2)    = cp(il,IBCPHL2) + cp(il,IBCPHL1)
          cp(il,ISO4BCPHL1) = 0.0d0
          cp(il,IBCPHL1)    = 0.0d0
        endif

        BOCaging(il,1) = BOCaging(il,1)+cp(il,IBCPHL1)*1.0d6
     $       *gmair(il)*1.0d-3*grid_height(il)     ! kg/kg => kg/m2 

        if(lso4non(il,ISO4FBC) > so4bc_cr2) then
          cp(il,ISO4BCPHL2) = cp(il,ISO4BCPHL2) + cp(il,ISO4FBC)
          cp(il,IBCPHL2)    = cp(il,IBCPHL2) + cp(il,IFBC)
          cp(il,ISO4FBC) = 0.0d0
          cp(il,IFBC)    = 0.0d0

          BOCaging(il,2) = BOCaging(il,2)+cp(il,IFBC)*1.0d6
     $       *gmair(il)*1.0d-3*grid_height(il)     ! kg/kg => kg/m2 

        else if(lso4non(il,ISO4FBC) > so4bc_cr1) then
          cp(il,ISO4BCPHL1) = cp(il,ISO4BCPHL1) + cp(il,ISO4FBC)
          cp(il,IBCPHL1)    = cp(il,IBCPHL1) + cp(il,IFBC)
          cp(il,ISO4FBC) = 0.0d0
          cp(il,IFBC)    = 0.0d0

          BOCaging(il,3) = BOCaging(il,3)+cp(il,IFBC)*1.0d6
     $       *gmair(il)*1.0d-3*grid_height(il)     ! kg/kg => kg/m2

        endif

        if(lso4non(il,ISO4OCPHL1) > so4oc_cr2) then
          cp(il,ISO4OCPHL2) = cp(il,ISO4OCPHL2) + cp(il,ISO4OCPHL1)
          cp(il,IOCPHL2)    = cp(il,IOCPHL2) + cp(il,IOCPHL1)
          cp(il,ISO4OCPHL1) = 0.0d0
          cp(il,IOCPHL1)    = 0.0d0
        endif

          BOCaging(il,4) = BOCaging(il,4)+cp(il,IOCPHL1)*1.0d6
     $       *gmair(il)*1.0d-3*grid_height(il)     ! kg/kg => kg/m2

        if(lso4non(il,ISO4FOC) > so4oc_cr2) then
          cp(il,ISO4OCPHL2) = cp(il,ISO4OCPHL2) + cp(il,ISO4FOC)
          cp(il,IOCPHL2)    = cp(il,IOCPHL2) + cp(il,IFOC)
          cp(il,ISO4FOC) = 0.0d0
          cp(il,IFOC)    = 0.0d0

          BOCaging(il,5) = BOCaging(il,5)+cp(il,IFOC)*1.0d6
     $       *gmair(il)*1.0d-3*grid_height(il)     ! kg/kg => kg/m2

        else if(lso4non(il,ISO4FOC) > so4oc_cr1) then
          cp(il,ISO4OCPHL1) = cp(il,ISO4OCPHL1) + cp(il,ISO4FOC)
          cp(il,IOCPHL1)    = cp(il,IOCPHL1) + cp(il,IFOC)
          cp(il,ISO4FOC) = 0.0d0
          cp(il,IFOC)    = 0.0d0

          BOCaging(il,6) = BOCaging(il,6)+cp(il,IFOC)*1.0d6
     $       *gmair(il)*1.0d-3*grid_height(il)     ! kg/kg => kg/m2

        endif
cRoy
        if(lso4non(il,ISO4CPOM) > so4oc_cr2) then
          cp(il,ISO4APOM) = cp(il,ISO4APOM) + cp(il,ISO4CPOM)
          cp(il,IAPOM)    = cp(il,IAPOM) + cp(il,ICPOM)
          cp(il,ISO4CPOM) = 0.0d0
          cp(il,ICPOM)    = 0.0d0
        endif

          BOCaging(il,7) = BOCaging(il,7)+cp(il,ICPOM)*1.0d6
     $       *gmair(il)*1.0d-3*grid_height(il)     ! kg/kg => kg/m2

        if(lso4non(il,ISO4CBC) > so4oc_cr2) then
          cp(il,ISO4ABC) = cp(il,ISO4ABC) + cp(il,ISO4CBC)
          cp(il,IABC)    = cp(il,IABC) + cp(il,ICBC)
          cp(il,ISO4CBC) = 0.0d0
          cp(il,ICBC)    = 0.0d0
        endif

          BOCaging(il,8) = BOCaging(il,8)+cp(il,ICBC)*1.0d6
     $       *gmair(il)*1.0d-3*grid_height(il)     ! kg/kg => kg/m2

      end do ! itloop

c      where(cp(:, ISO4M1)   < 1.0d-30) cp(:, ISO4M1) = 1.0d-30
c      where(cp(:, ISO4N1)   < 1.0d-30) cp(:, ISO4N1) = 1.0d-30
c      where(cp(:, ISO4M2)   < 1.0d-30) cp(:, ISO4M2) = 1.0d-30
c      where(cp(:, ISO4N2)   < 1.0d-30) cp(:, ISO4N2) = 1.0d-30

c     cp(:, ISO4M1) = max(cp(:, ISO4N1)*r2so4min, cp(:, ISO4M1))
c     cp(:, ISO4M2) = max(cp(:, ISO4N2)*r2so4min, cp(:, ISO4M2))
c     cp(:, ISO4N1) = max(cp(:, ISO4M1)/rso4max,  cp(:, ISO4N1))
c     cp(:, ISO4N2) = max(cp(:, ISO4M2)/rso4max,  cp(:, ISO4N2))

      where(cp(:, ISO4NOC:ISO4S4) < 1.0d-30)
     &      cp(:, ISO4NOC:ISO4S4) = 1.0d-30

c     where(cp(:, IFSO4A)   < 1.0d-30) cp(:, IFSO4A ) = 1.0d-30
c     where(cp(:, IFSO4N1)  < 1.0d-30) cp(:, IFSO4N1) = 1.0d-30
c     where(cp(:, IFSO4N2)  < 1.0d-30) cp(:, IFSO4N2) = 1.0d-30
c     where(cp(:, IFSO4N3)  < 1.0d-30) cp(:, IFSO4N3) = 1.0d-30

c     where(cp(:, INSO4A)   < 1.0d-30) cp(:, INSO4A ) = 1.0d-30
c     where(cp(:, INSO4N1)  < 1.0d-30) cp(:, INSO4N1) = 1.0d-30
c     where(cp(:, INSO4N2)  < 1.0d-30) cp(:, INSO4N2) = 1.0d-30
c     where(cp(:, INSO4N3)  < 1.0d-30) cp(:, INSO4N3) = 1.0d-30


      return

      end

