c=============================================================================
c
c $Id: sulfchem.F,v 1.2 2003/06/30 22:50:11 dan Exp $
c
c CODE DEVELOPER
c   originated from GRANTOUR programmed by John Walton
c   modified by Xiaohong Liu
c
c FILE
c   sulfchem.F
c
c ROUTINES
c   Do_Sulfchem
c
c=============================================================================

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Do_Sulfchem
c
c DESCRIPTION
c
c     *****************************************************************
c     * carries out the sulfur chemistry calculation
c     *****************************************************************
c
c     Performs a numerical integration of the full aqueous chemistry
c      equations for the species H2SO4, DMS, SO2 & H2O2 in cloudy air.
c     Performs an analytic calculation of the clear sky part of the
c      chemistry equations for the species H2SO4, DMS, SO2 & H2O2 in 
c      clear air.
c     Combines the clear and cloudy concentrations to simulate mixing 
c      between clear and cloudy regions on the basis of cloud fraction
c
c     ********** input:
c     *     dtchem  =  model time step in seconds
c     *     dtclc   =  time step for clear air chemistry
c     *     dtclmx  =  max. time step between mixing calculations in seconds
c     *     dtcl    =  time step between mixing calculations in seconds
c     *     dtaq    =  time step used in integration of the aqueous
c                      equations in seconds
c     *     dph     =  assumed pH of the water drops
c
c     *     tgcm       air temperature (K)
c     *     zmair      air density (# cm-3)
c     *     gmair      air density (gm cm-3)
c     *     oh         OH  concentration (cm-3)
c     *     ho2        HO2 concentration (cm-3)
c     *     no3        NO3 concentration (cm-3)
c     *     o3         O3  concentration (cm-3)
c     *     cfac       total cloud fraction (0-1)
c     *     cwac       cloud water mixing ratio for aq chem (g/g) (in-cloud)
c
c     *     qh1p       DMS + OH  reaction rate coefficient
c     *     qh2p       SO2 + OH     "      "        "
c     *     qh3p       HO2 + HO2    "      "        "
c     *     qj4p       H2O2 + hv photolysis rate coef.
c     *     qh5p       H2O2 + OH reaction rate coefficient
c     *     qh6p       SO2(aq) + H2O2(aq)  "        "
c     *     qh7p       SO2(aq) + O3(aq)    "        "
c     *     qh8p       HO2 + HO2 + H2O     "        "
c     *     qh9p       DMS + NO3 reaction rate coefficient
c
c     *     a1->a9     reaction coefficients
c
c
c     ********** output:
c
c     * Here is how I treat separate changes due to the aqueous
c       and clear sky chemical paths.
c
c       I assume that aqueous and clear sky chemistry act on the 
c       total concentration, C, giving the concentration changes 
c       dCa(C) & dCn(C).
c
c       The new values of the concentration at t+dt are
c
c           C(t+dt) = C(t) + cfp * dCa(C) + ( 1 - cfp ) * dCn(C)
c
c       In the case of H2SO4, these two terms contribute separately
c       to aqeuous and clear sky concentration.
c
c       In the case of all clear air or all cloud, these reduce to 
c       what one would expect. 
c
c       Definitions
c
c       C           species concentration
c
c       dCa, dCn    Concentration changes for aqueous & clear sky
c                   chemistry.
c
c       cfp         fractional cloud cover.
c
c     *----------------------------------------------------------------
c
c ARGUMENTS 
c   itloop    : # of zones (ilong * ilat * ivert)
c   time      : time interval for each chemical calculation (seconds)
c   tgcm      : temperature (K)
c   pgcm      : atmospheric pressure at the center of each grid box (mb)
c   ispbcm    : the flag for boundary layer
c   xr        : species concentration, known at zone centers 
c               (molecules/cm^3 for gas-species, kg/kg for aerosol mass,
c                #particles/kg for sulfate aerosol number)
c   humidity  : specific humidity (g/kg) 
c   semiss    : array of sulfur emissions (molecules/cm^3/s)
c   qjimp     : photolysis rate constants (s^-1)
c   qkimp     : thermal    rate constants (units vary)
c   cfac      : total cloud fraction [0 - 1]
c   cwac      : in-cloud liquid water in each grid box (g/g)
c   relh      : relative humidity [0 - 1]
c   aqua_infile_name : aquachem input file name
c
c-----------------------------------------------------------------------------
C change note by Cheng Zhou, 12/20/2012
c 1) change include to use
c 2) comment of dtclmx, set dtcl = dtime so aqu is run every time step, same as gas phase reaction.

      subroutine Do_Sulfchem
     &  (itloop, time, tgcm, pgcm, ispbcm, xr, humidity,
     &   semiss, qjimp, qkimp, cfac, cwac, relh,
     &   aqua_infile_name, pr_diag, loc_proc, num_time_steps,
     &   massc, pr_sulf_src, dms_oh, dms_no3, so2_oh, so2_h2o2, so2_o3,
     &   grid_height)
c     &   so2_hete)

c
c---->load modules
c

      use h_imp_phys_constants
      use h_setkin_par
      use h_sulfchem, only:ic,wtmair,wtmso4,hpp,rhpp,ijk 
      use h_umaerosol

      use modinit_umaer

      implicit none

c#     include "imp_phys_constants.h"
c#     include "setkin_par.h"
c#     include "sulfchem.h"
c#     include "umaerosol.h"

c     ----------------------
c     IMPACT-related values.
c     ----------------------

      character*(*) :: aqua_infile_name

      integer :: itloop
      real*8  :: time
      real*8  :: tgcm    (itloop)
      real*8  :: pgcm    (itloop)
      integer :: ispbcm  (itloop)
      real*8  :: xr      (itloop, NSP)
      real*8  :: humidity(itloop)
      real*8  :: semiss  (itloop, NSP)
      real*8  :: qjimp   (itloop, NUM_J)
      real*8  :: qkimp   (itloop, NUM_K)
      real*8  :: cfac    (itloop)
      real*8  :: cwac    (itloop)
      real*8  :: relh    (itloop)
c      real*8  :: relh1   (itloop)
      real*8  :: grid_height(itloop)

      logical :: pr_diag
      integer :: loc_proc
      integer :: num_time_steps

      real*8  :: massc   (itloop)
      logical :: pr_sulf_src
      real*8  :: dms_oh  (itloop)
      real*8  :: dms_no3 (itloop)
      real*8  :: so2_oh  (itloop)
      real*8  :: so2_h2o2(itloop)
      real*8  :: so2_o3  (itloop)
c      real*8  :: so2_hete(itloop)


c ----------------------------------------------------------
c  *** LOCAL  VARIABLES WERE DECLARED HERE.
c ----------------------------------------------------------
 
      integer :: numcl

      real*8   dtclmx, dtaqset,
     &         accset, dph

      real*8, parameter :: CMPM3 = CMPM * CMPM * CMPM

c      parameter (dtclmx = 14400.d0)
      parameter (dtaqset = 600.d0) !useless, dtaq is determined as dtaq=dtcl/ndtmin, should set ndtmin in aqu_sulfchem.F
      parameter (accset = 0.0001d0)
      parameter (dph = 4.5d0)

      real*8
     &  dtaq,  ! time step used in integration of aqueous equations in seconds
     &  dtchem,! model time step in seconds
     &  dtcl,  ! time step between mixing calculations in seconds
     &  dtclc  ! time step for clear air chemistry

c     qh6       SO2(aq) + H2O2(aq) reaction rate coefficient
      real*8  qh6

c---->stuff for umaerosol
      logical, save :: first = .true.

      integer :: ng_i, nso4_i, nmomso4_i, nnon_i, nmomnon_i

      real*8 :: press (itloop)
      real*8 :: so4gas(itloop)
      real*8 :: srcgas(itloop)
      real*8 :: srcmas(itloop)
      real*8 :: srcpar(itloop)
      real*8 :: so4aer(nmomso4,nso4,itloop)
      real*8 :: so4non(nnon,itloop)
      real*8 :: aernon(nmomnon,nnon,itloop)
c---->end of stuff for umaerosol
c++Roy
      real*8 :: srcg_aso2(itloop)
      real*8 :: srcm_aso2(itloop)
c--

c     air density (molecules/cm3).
      real*8 :: zmair(itloop)

c     air density (g/cm3).
      real*8 :: gmair(itloop)

c     air pressure (bar)
      real*8 :: pgcmb(itloop)

c     H2O concentration in molecules/cm3.
      real*8 :: h2o(itloop)

c     concentrations of O3, OH, HO2 and NO3 in molecules/cm3.
      real*8 :: o3(itloop)
      real*8 :: oh(itloop)
      real*8 :: ho2(itloop)
      real*8 :: no3(itloop)

c     rtcd (s-1) rate coef. for SO2 -> dust transfer
c     rtcs (s-1) rate coef. for SO2 -> sslt transfer
      real*8 :: rtcd(itloop,4)
      real*8 :: rtcs(itloop,4)

c     cp and dcp have units [molecules/cm3 for gas-species and 
c     gm spec./gm air for aerosols]
      real*8 :: cp (itloop,NSP)
      real*8 :: dcp(itloop,NSP)

c     H2O2 photolysi rate coefficient in 1/s
      real*8 :: qj4p(itloop)

c     qh1p       DMS + OH  reaction rate coefficient
c     qh2p       SO2 + OH     "      "        "
c     qh3p       HO2 + HO2    "      "        "
c     qh5p       H2O2 + OH reaction rate coefficient
c     qh7p       SO2(aq) + O3(aq)    "        "
c     qh8p       HO2 + HO2 + H2O     "        "
c     qh9p       DMS + NO3    "      "        "

      real*8 :: qh1p(itloop)
      real*8 :: qh2p(itloop)
      real*8 :: qh3p(itloop)
      real*8 :: qh5p(itloop)
      real*8 :: qh7p(itloop)
      real*8 :: qh8p(itloop)
      real*8 :: qh9p(itloop)

c     henry's law coefficients (hwp, hyp, h3p) for SO2, H2O2, O3
      real*8 :: hwp(itloop)
      real*8 :: hyp(itloop)
      real*8 :: h3p(itloop)

c     array of grid numbers for grids in clouds
      integer :: idxcl(itloop)

c     a1-9 coefficients in aqueous equations (only for cloudy grids)
      real*8 :: acoef(itloop,9)

c     the fraction of primary-emitted sulfate particles:
      real*8 :: frc_srcpar
c     the real emission for so2
      real*8  :: semiss2  (itloop, NSP)

      frc_srcpar = 0.02d0
      semiss2(:, :) = semiss(:,:)
      semiss2(:, IFSO2) = semiss(:,IFSO2) * (1.0d0-frc_srcpar)
      semiss2(:, IASO2) = semiss(:,IASO2) ! Added by Roy

      zmair = 0.0d0
      gmair = 0.0d0
      h2o   = 0.0d0
      o3    = 0.0d0
      oh    = 0.0d0
      ho2   = 0.0d0
      no3   = 0.0d0
      rtcd  = 0.0d0
      rtcs  = 0.0d0
      cp    = 0.0d0
      dcp   = 0.0d0

      qj4p  = 0.0d0
      qh1p  = 0.0d0
      qh2p  = 0.0d0
      qh3p  = 0.0d0
      qh5p  = 0.0d0
      qh7p  = 0.0d0
      qh8p  = 0.0d0
      qh9p  = 0.0d0
      hwp   = 0.0d0
      hyp   = 0.0d0
      h3p   = 0.0d0
      idxcl = 0.0d0
      acoef = 0.0d0

      so4gas = 0.0d0
      srcgas = 0.0d0
      srcmas = 0.0d0
      srcpar = 0.0d0
      so4aer = 0.0d0
      so4non = 0.0d0
      aernon = 0.0d0
c++Roy
      srcg_aso2 = 0.0d0
      srcm_aso2 = 0.0d0
c--

c     ----------------
c     Begin execution.
c     ----------------

      if (pr_diag) then
        Write (6,*) 'Do_Sulfchem called by ', loc_proc
      end if


      if (first) then

        first = .false.
        
        nso4_i   =nso4
        nmomso4_i=nmomso4
        nnon_i   =nnon
        nmomnon_i=nmomnon
        ng_i     =ng

        call umaerinit(nso4_i,nmomso4_i,nnon_i,nmomnon_i,ng_i,
     $                 sigmod,radmerg,
     $                 radgsrc,siggsrc,frcmsrc,
     $                 radgso4,siggso4,fracso4,
     $                 radgnon,siggnon,fracnon,
     $                 rhonon,xmolnon,nrh_flg,
     $                 crh1,crh2,crh3,crh4)

      end if


c     *****************************************************************
c     * Set chemistry parameters
c     *****************************************************************

      dtchem = time
      dtcl   = time !dtclmx ,Change to time also, so aqua phase is run every timestep.
      dtaq   = dtaqset

      hpp  = 10**(-dph)
      rhpp = 1./hpp

c     *****************************************************************
c     * Compute air density, "zmair" [#/cm3] and "gmair" [gm/cm3] at
c       model grid points.
c     *****************************************************************

      do ijk = 1, itloop
        zmair(ijk) = xr(ijk, IMGAS)
        gmair(ijk) = ( MWTAIR / AVOGAD ) * zmair(ijk)
      end do

c     *****************************************************************
c     * Calculate water vapor concentration in molecules/cm^3
c     *****************************************************************

      do ijk = 1, itloop
        h2o(ijk) = humidity(ijk) * MWTAIR / MWTH2O / GPKG
     &             * xr(ijk, IMGAS)
      end do

      do ijk = 1, itloop
        o3(ijk)  = xr(ijk, IO3)
        oh(ijk)  = xr(ijk, IOH)
        ho2(ijk) = xr(ijk, IHO2)
        no3(ijk) = xr(ijk, INO3)
      end do

c     change unit of pressure at the grid points from millibar to bar
      do ijk = 1, itloop
        pgcmb(ijk) = pgcm(ijk) / MBPB      ! bar
      end do

      do ijk = 1, itloop
        press(ijk) = pgcm(ijk) * PASPMB    ! unit: Pa
      end do

c     *****************************************************************
c     * Define the H2O2 photolysis rate coefficient
c     *****************************************************************

      do ijk = 1, itloop
        qj4p(ijk) = qjimp(ijk, 1)
      end do

c     *****************************************************************
c     * Given by IMPACT concentrations for species changed by
c       sulfur chemistry
c     *****************************************************************

      do ic = 1, NSP
        cp(:, ic) = xr(:, ic)
      end do


c     *****************************************************************
c     * Calculate the SO2 mass transfer rate coefficients for
c       dust and sea salt
c     *****************************************************************

#ifndef NONON
c          =================
c     call Do_So2toPar_Rates
c          =================
c    &  (itloop, pgcmb, tgcm,
c    &   relh, aqua_infile_name,
c    &   gmair, cp, rtcd, rtcs)
#endif

c     *****************************************************************
c     * Calculate gas-phase reaction rates 
c       (qh1p, qh2p, qh3p, qh5p, qh8p, qh9p)
c     *****************************************************************

c          ===============
      call Do_QK_Gas_Rates
c          ===============
     &  (itloop, tgcm, zmair,
     &   qh1p, qh2p, qh3p, qh5p, qh8p, qh9p)


c     ================================================================
      if (Mod (num_time_steps, (Nint(dtcl) / Nint(dtchem))) == 0) then
c     ================================================================

c     *****************************************************************
c     * Set up for aqueous chemistry
c       (generate numcl, idxcl)
c     *****************************************************************

c          ================
      call Do_Setup_Aquchem
c          ================
     &  (itloop, numcl, idxcl, cfac, cwac)


      if (numcl .gt. 0) then

c     *******************************************************************
c     * There are some grid points in clouds. Set up for aqueous chem.
c       calculate the aqueous-phase reaction rates, effective Henry's law
c       coefficients, and aqueous equation coeff. (a1-a9)
c     ******************************************************************* 

c            ===============
        call Do_QK_Aqu_Rates
c            ===============
     &    (itloop, tgcm, hpp, rhpp, numcl, idxcl,
     &     oh, ho2, h2o, no3, o3,
     &     cwac, zmair,
     &     qh1p, qh2p, qh3p, qj4p, qh5p, qh8p, qh9p,
     &     qh6, qh7p, hwp, hyp, h3p,
     &     acoef)

        dtclc = dtcl

      else

c     ******************************************************************
c     * There are no cloudy grid points, advance the entire dtchem step.
c     * Advance the clear air chemistry only.
c     ******************************************************************

        dtclc = dtcl   ! dtchem

      endif

c     =====
      endif
c     =====

c.... time step over cloud mixing time (dtclc)

c     tcl = 0.0d0

c     ========
c 20  continue
c     ========


c     ***********************************************************
c     * Calculate the species changes due to clear sky chemistry.     
c     ***********************************************************

c      write(0,9998) 'gas_sulf1->',
c     &                    Sum(semiss2(:,IFSO2)),
c     &                    Sum(semiss2(:,IASO2)),
c     &         Sum(cp(:, IFSO2)),
c     &         Sum(dcp(:, IFSO2)),
c     &         Sum(cp(:, IASO2)),
c     &         Sum(dcp(:, IASO2))

9998  format(A15,6E12.4)

c          ===============
      call Do_Gas_Sulfchem
c          ===============
c    &  (dtclc, itloop, cp, dcp, cfac, zmair, semiss,
     &  (dtchem, itloop, cp, dcp, cfac, zmair, semiss2, srcgas,
     &   srcg_aso2, oh, ho2, h2o, no3,
     &   massc, pr_sulf_src, dms_oh, dms_no3, so2_oh,
     &   qh1p, qh2p, qh3p, qj4p, qh5p, qh8p, qh9p)

c      write(0,9998) 'gas_sulf2->',
c     &                    Sum(semiss(:,IFSO2)),
c     &                    Sum(semiss(:,IASO2)),
c     &         Sum(cp(:, IFSO2)),
c     &         Sum(dcp(:,IFSO2)),
c     &         Sum(cp(:, IASO2)),
c     &         Sum(dcp(:,IASO2))

c     =================================================================
      if (Mod (num_time_steps, (Nint(dtcl) / Nint(dtchem))) == 0) then
c     =================================================================

c     *****************************************************************
c     * Integrate aqueous equations over dtclc and add the aqueous 
c       change to the clear sky changes already computed. 
c     *****************************************************************
 
      if (numcl .gt. 0) then

c            ===============
        call Do_Aqu_Sulfchem
c            ===============
     &    (accset, dtaq, dtchem, dtcl, dtclc,
     &     itloop, numcl, idxcl, semiss2, srcmas,
     &     srcm_aso2, massc, pr_sulf_src, 
     &     dms_oh, dms_no3, so2_oh, so2_h2o2, so2_o3,
     &     cp, dcp, cfac, zmair, acoef, loc_proc)

      end if

c     ======
      end if
c     ======
c      write(0,9998) 'aqu_sulf->',
c     &                    Sum(semiss(:,IFSO2)),
c     &                    Sum(semiss(:,IASO2)),
c     &         Sum(cp(:, IFSO2)),
c     &         Sum(dcp(:, IFSO2)),
c     &         Sum(cp(:, IASO2)),
c     &         Sum(dcp(:, IASO2))

c
c---->call umaerosol
c
      where(cp(:, ISO4G)    < 1.0d-30) cp(:, ISO4G)  = 1.0d-30

      where(cp(:, ISO4M1)   < 1.0d-30) cp(:, ISO4M1) = 1.0d-30
      where(cp(:, ISO4N1)   < 1.0d-30) cp(:, ISO4N1) = 1.0d-30
      where(cp(:, ISO4M2)   < 1.0d-30) cp(:, ISO4M2) = 1.0d-30
      where(cp(:, ISO4N2)   < 1.0d-30) cp(:, ISO4N2) = 1.0d-30
      where(cp(:, ISO4M3)   < 1.0d-30) cp(:, ISO4M3) = 1.0d-30
      where(cp(:, ISO4N3)   < 1.0d-30) cp(:, ISO4N3) = 1.0d-30

      where(cp(:, ISO4NOC:ISO4S4) < 1.0d-30)
     &      cp(:, ISO4NOC:ISO4S4) = 1.0d-30

c     assume 2% of anthropogenic sulfur emission as H2SO4, 98% as SO2
c      srcpar(:)     = semiss(:,IFSO2) * 0.02d0 * CMPM3      ! #molecules/m3/s
      srcpar(:)     = semiss(:,IFSO2) * frc_srcpar * CMPM3   ! #molecules/m3/s
c     srcpar(:)     = 1.0d-30                               ! no direct emission

      where(srcgas(:)       < 1.0d-30) srcgas(:)     = 1.0d-30
      where(srcmas(:)       < 1.0d-30) srcmas(:)     = 1.0d-30
      where(srcpar(:)       < 1.0d-30) srcpar(:)     = 1.0d-30

c++Roy
      where(srcg_aso2(:)    < 1.0d-30) srcg_aso2(:)  = 1.0d-30
      where(srcm_aso2(:)    < 1.0d-30) srcm_aso2(:)  = 1.0d-30
      srcgas(:)     = srcgas(:) + srcg_aso2(:)
      srcmas(:)     = srcmas(:) + srcm_aso2(:)
c--

      so4gas(:)     = cp(:,ISO4G) * CMPM3                   ! #molecules/m3

      so4aer(1,1,:) = cp(:,ISO4M1)                          ! #molecules/m3
     &              * wtmair/wtmso4*zmair(:)*CMPM3
      so4aer(2,1,:) = cp(:,ISO4N1)                          ! #particles/m3
     &              * gmair(:)*CMPM3/GPKG
      so4aer(1,2,:) = cp(:,ISO4M2)                          ! #molecules/m3
     &              * wtmair/wtmso4*zmair(:)*CMPM3
      so4aer(2,2,:) = cp(:,ISO4N2)                          ! #particles/m3
     &              * gmair(:)*CMPM3/GPKG
      so4aer(1,3,:) = cp(:,ISO4M3)                          ! #molecules/m3
     &              * wtmair/wtmso4*zmair(:)*CMPM3
      so4aer(2,3,:) = cp(:,ISO4N3)                          ! #particles/m3
     &              * gmair(:)*CMPM3/GPKG

      do ijk = 1, itloop

        so4non(1:nnon,ijk) = cp(ijk,ISO4NOC:ISO4S4)         ! #molecules/m3
     &                     * wtmair/wtmso4*zmair(ijk)*CMPM3

        aernon(1,1:nnon,ijk) = cp(ijk,INOC:ISSLT4)          ! kg/m3
     &                       * gmair(ijk)*CMPM3/GPKG

      end do

cc    do a sensitivity test of upper tropospheric RH (100-300 mb)
c     relh1(:) = relh(:)
c     where ((press(:) <= 300.0d0*PASPMB) .and.
c    &       (press(:) >  100.0d0*PASPMB))
c       relh1(:) = relh(:) / 1.5d0
c     end where

c          =========
      call umaerosol
c          =========
     $     (dtchem, itloop, tgcm, press, ispbcm, relh, cfac,
c    $     (dtchem, itloop, tgcm, press, relh1, cfac,
     $      so4gas, srcgas, srcmas, srcpar,
     $      so4aer, so4non, aernon)


c     give back concentrations

      cp(:,ISO4G)  = so4gas(:) / CMPM3                      ! molecules/cm3

      cp(:,ISO4M1) = so4aer(1,1,:)                          ! kg/kg
     &             / (wtmair/wtmso4*zmair(:)*CMPM3)
      cp(:,ISO4N1) = so4aer(2,1,:)                          ! #particles/kg
     &             / (gmair(:)*CMPM3/GPKG)
      cp(:,ISO4M2) = so4aer(1,2,:)                          ! kg/kg
     &             / (wtmair/wtmso4*zmair(:)*CMPM3)
      cp(:,ISO4N2) = so4aer(2,2,:)                          ! #particles/kg
     &             / (gmair(:)*CMPM3/GPKG)
      cp(:,ISO4M3) = so4aer(1,3,:)                          ! kg/kg
     &             / (wtmair/wtmso4*zmair(:)*CMPM3)
      cp(:,ISO4N3) = so4aer(2,3,:)                          ! #particles/kg
     &             / (gmair(:)*CMPM3/GPKG)

      do ijk = 1, itloop

        cp(ijk,ISO4NOC:ISO4S4) = so4non(1:nnon,ijk)         ! kg/kg
     &                 / (wtmair/wtmso4*zmair(ijk)*CMPM3)

      end do


c     *****************************************************************
c     * Impose the clear plus cloudy sky concentration changes on
c       the grid cells.
c     *****************************************************************

c          ====================
      call Do_Const_Sulf_Update
c          ====================
     &  (itloop, cp, dcp, tgcm, relh, grid_height, gmair)

c     *****************************************************************
c     * Calculate the mass transfer of SO2 vapor to water on the 
c       surface of particles.
c     * The SO2 will appear on the particles as SO4.
c     *****************************************************************
#ifndef NONON
c          =================
c     call Do_Const_So2toPar
c          =================
c    &  (dtchem, itloop, zmair,
c    &   massc, pr_sulf_src, so2_hete,
c    &   rtcd, rtcs, cp)
#endif

c.... advance the time

c     tcl = tcl + dtclc
c                                 ========
c     if (tcl .lt. dtchem - 0.01d0) go to 20
c                                 ========

c     *****************************************************************
c     * Give back to IMPACT data
c     *****************************************************************

c      write(0,9998) 'afaqu_sulf->',
c     &                    Sum(semiss(:,IFSO2)),
c     &                    Sum(semiss(:,IASO2)),
c     &         Sum(cp(:, IFSO2)),
c     &         Sum(dcp(:, IFSO2)),
c     &         Sum(cp(:, IASO2)),
c     &         Sum(dcp(:, IASO2))

      do ic = 1, NSP
        xr(:, ic) = cp(:, ic)
      end do

      return

      end

